//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		ScopeControllerConstants.h
//
//	Contains:	Constants used in the Telescope Controller plugin. Including physical 
//				constants and sypported telescope types.
//								
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/11/15	MK			5.7			Additional telescopes listed under existing classes:
//											Orion Atlas EQ-G supported under NexStar58
//											3 more scopes supported under OrionSkyQuest driver
//	05/09/01	MK			5.7			More stars added to the array of stars used for alignment
//  05/03/10	MK			5.02		Addition of BrightStarT structure and an array of
//										stars used for alignment through the plugin.
//										Also added a few constants and errors
//	01/09/15	DWW			4.0.0		New internal telescope support for Pro 4
//----------------------------------------------------------------------



#include "StarryNightPlugins.h"
#include "PluginLString.h"

// some useful constants
const double kAuToMeters 			= 1.4959787066e11;		// number of metres per astronomical unit
const double kMetersToAu 			= 1.0/kAuToMeters;		// number of astronomical unit per metres
const double kLightYearsPerParsec	= 3.261633;				// Pete got this number from somewhere
const double kParsecsPerAu 			= 4.848136811e-6;		// explanatory suppl, page 716
const double kAuPerParsec 			= 1.0/kParsecsPerAu;		// explanatory suppl, page 716
const double kAuPerLightYear 		= kAuPerParsec/kLightYearsPerParsec;	
const double kLightYearPerAu 		= 1.0/kAuPerLightYear;		
const double kMetrePerLightYear		= kAuPerLightYear*kMetersToAu;

// Some more useful constants
const double kPiDefine			= 3.14159265358979323846;
const double kHalfPi			= kPiDefine/2.0;
const double kThreeQuarterPi	= (3.0*kPiDefine)/4.0;
const double kThreeHalfPi		= (3.0*kPiDefine)/2.0;
const double kTwicePi			= 2.0*kPiDefine;
const double kOneOverTwicePi	= (1.0/kTwicePi);
const double kRadiansPerHour 	= (2.0*kPiDefine)/24.0;
const double kDegreesToRadians 	= (kPiDefine/180.0);
const double kRadiansToDegrees 	= 1/kDegreesToRadians;
const double kSiderealPerSolar  = 1.002738;
const SInt32 kSecondsPerDay		= 60*60*24;
const SInt32 kMinsPerHour 		= 60;
const SInt32 kMinsPerDegree		= 60;
const SInt32 kSecondsPerMinute	= 60;

const double kTicksPerSecond	= 60.0;

const double k35Seconds			= 35000.0;
const double k10Seconds			= 10000.0;
const double k2KMilliSeconds	= 2000.0;
const double k1500MilliSeconds	= 1500.0;
const double k1KMilliSeconds	= 1000.0;
const double k800MilliSeconds	= 800.0;
const double k500MilliSeconds	= 500.0;
const double k300MilliSeconds	= 300.0;
const double k200MilliSeconds	= 200.0;
const double k100MilliSeconds	= 100.0;
const double k50MilliSeconds	= 50.0;
const double k10MilliSeconds	= 10.0;
const double kZEROMilliSeconds	= 0.0;

// MEADE Specific constants
const double kMeadeWaitTime			= k300MilliSeconds;
const double kMeadeETXWaitTime		= k300MilliSeconds;
const double kLX200SettleTime		= k2KMilliSeconds;
const double kAUTOSTARSettleTime	= k10Seconds;

// CELESTRON Specific Constants
const double kCelestronEncoderLowResolution		= 64800.0;		// 1/3 Arcminute
const double kCelestronEncoderHighResolution	= 15552000.0;	// 1/12 ArcSeccond
const double kCelestron16BitResolution			= 65536.0;		// 16 bit
const double kCelestron32BitResolution			= 16777216.0;		// Actually 24 bit since last two bits always 0
const double kCelestronLowResEncoderToSoftwareConversion = kCelestron16BitResolution/kCelestronEncoderLowResolution;
const double kCelestronHighResEncoderToSoftwareConversion = kCelestron32BitResolution/kCelestronEncoderHighResolution;
const double kNS58PointingRes		= 64800.0;
const double kNS58MaxSlewTime		= k35Seconds;
const double kNS58WaitTime			= k500MilliSeconds;
const double kNS11U2KWaitTime		= k500MilliSeconds;
const double kNS11U2KSettleTime		= k10Seconds;

// ORION Specific Constants
const double kOrionEncoderResolution = 9216; //~ 2.3 Arcminutes

// Windows Comm Port Timeouts
const double kReadIntervalTimeout 			= k1KMilliSeconds;
const double kReadTotalTimeoutMultiplier	= kZEROMilliSeconds;
const double kReadTotalTimeoutConstant 		= k1KMilliSeconds;
const double kWriteTotalTimeoutMultiplier 	= kZEROMilliSeconds;
const double kWriteTotalTimeoutConstant 	= kZEROMilliSeconds;

// Scope Configure Utility Constants
const UInt32 kMaxCharsToReadFromScope			= 20;
const double kReturnStringTimeoutInMilliseconds	= k2KMilliSeconds;

///////////////////////////////////////////////////////////////////////////
// Plugin Errors														 //
///////////////////////////////////////////////////////////////////////////
// Note that the Starry Night API uses 0 (snNoErr) and negative numbers for
// its errors. Your plugin should restrict to positive numbers if you wish to
// implement an internal error handling function
const snErr pluginErr_Failed					= 1;	// failed, for some unknown reason	
const snErr pluginErr_CommunicationError		= 2;	// there was an error communicating with the scope
const snErr pluginErr_ObjectBelowHorizon		= 3;	// the requested slew is below the horizon
const snErr pluginErr_LX200SpecificError		= 4;	// error specific to an LX200 (could be lots of things...check your LX200 manual)
const snErr pluginErr_SetTimeError				= 5;	// the time  was not properly set
const snErr pluginErr_SetLocationError			= 6;	// the location was not properly set	
const snErr pluginErr_SynchError				= 7;	// the synch was not perfored properly
const snErr pluginErr_ReturnedBadValue			= 8;	// the scope returned a bad value, could be a communication error
const snErr pluginErr_SlewAlreadyInProgress		= 9;	// the scope is already slewing
const snErr pluginErr_FunctionNotSupported		= 10;	// the selected function is not supported for this scope type
const snErr pluginErr_MatrixInitError			= 11;   // transformation matrix needed for alignment could not be calculated
const snErr pluginErr_UserCancelled				= 12;   // action in progress was cancelled by the user

///////////////////////////////////////////////////////////////////////////
// Telescope Types and Names (DO NOT EDIT WITHOUT READING THE NOTE BELOW)//
///////////////////////////////////////////////////////////////////////////

// When adding a scope:
// 1) Remember to increment kNumberOfScopeTypesSupported. Note that this total should include scope_NONE.
// 2) Add the scopeType to the BOTTOM of the list (so we can keep track easily), just ABOVE kNumberOfScopeTypesSupported
// 3) Add the constant scope name (this will appear in the scope popup menu and is different from the pretty name)
// 4) Add the name string to the BOTTOM of scopeNameArray. This will allow us to access the correct string in
//    the scope name array with the scopeType (just an int) as the index.

// Supported telescope types
typedef UInt32 scopeType;
const scopeType 	scope_NONE 					= 0;
const scopeType 	scope_LX200					= 1;
const scopeType 	scope_LX200GPS				= 2;
const scopeType 	scope_GENERICLX200			= 3;
const scopeType 	scope_AUTOSTAR				= 4;
const scopeType 	scope_NEXSTAR6080114		= 5;
const scopeType 	scope_NEXSTAR4GT			= 6;
const scopeType 	scope_NEXSTAR58				= 7;
const scopeType 	scope_NEXSTAR5i8i			= 8;
const scopeType 	scope_NEXSTAR811GPS			= 9;
const scopeType 	scope_CELESTCGE				= 10;
const scopeType 	scope_CELESTASC				= 11;
const scopeType 	scope_ASTROPGERMAN			= 12;
const scopeType		scope_ORIONSKYQUEST			= 13;
const scopeType		scope_NGCMAX				= 14;
const scopeType		scope_LUMICON				= 15;
const scopeType		scope_DSTARFINDER			= 16;
const scopeType		scope_ORIONATLAS			= 17;
const scopeType 	kNumberOfScopeTypesSupported = 18;	// This MUST remain the last item in the list 
//const scopeType 	scope_U2K					= 12;
//const scopeType 	scope_U2KCOMPATIBLE			= 13;


// Scope names for configure dialog
const LStr255 kScopeName_NONE(			""									);
const LStr255 kScopeName_LX200(			"Meade LX200"						);
const LStr255 kScopeName_LX200GPS(		"Meade LX200 GPS"					);
const LStr255 kScopeName_GENERICLX200(	"LX200 Compatible"					);
const LStr255 kScopeName_AUTOSTAR(		"Meade ETX Autostar"				);
const LStr255 kScopeName_NEXSTAR6080114("Celestron NexStar 60/80/114"		);
const LStr255 kScopeName_NEXSTAR4GT(	"Celestron NexStar 4 GT"			);
const LStr255 kScopeName_NEXSTAR58(		"Celestron NexStar 5/8"				);
const LStr255 kScopeName_NEXSTAR5i8i(	"Celestron NexStar 5i/8i"			);
const LStr255 kScopeName_NEXSTAR811GPS(	"Celestron NexStar 8/11 GPS"		);
const LStr255 kScopeName_CELESTCGE(		"Celestron CGE Series"				);
const LStr255 kScopeName_CELESTASC(		"Celestron Advanced Series"			);
const LStr255 kScopeName_ASTROPGERMAN(	"Astro-Physics German Equatorial"	);
const LStr255 kScopeName_ORIONSKYQUEST( "Orion SkyQuest IntelliScope"		);
const LStr255 kScopeName_ORIONATLAS(	"Orion Atlas EQ-G"					);
const LStr255 kScopeName_NGCMAX(		"NGCMAX"							);
const LStr255 kScopeName_LUMICON(		"Lumicon Sky Vector"				);
const LStr255 kScopeName_DSTARFINDER(	"Discovery Digital Star Finder"		);
//const LStr255 kScopeName_U2K(			"Ultima 2000"						);
//const LStr255 kScopeName_U2KCOMPATIBLE(	"Ultima 2000 Compatible"		);

// Scope name array 
// NOTE that these MUST be in the same order as the scopeType indexes or things will get out of synch
const LStr255 kScopeNameArrary[kNumberOfScopeTypesSupported] =	{
																	kScopeName_NONE,
																	kScopeName_LX200,
																	kScopeName_LX200GPS,
																	kScopeName_GENERICLX200,
																	kScopeName_AUTOSTAR,
																	kScopeName_NEXSTAR6080114,
																	kScopeName_NEXSTAR4GT,
																	kScopeName_NEXSTAR58,
																	kScopeName_NEXSTAR5i8i,
																	kScopeName_NEXSTAR811GPS,
																	kScopeName_CELESTCGE,
																	kScopeName_CELESTASC,
																	kScopeName_ASTROPGERMAN,
																	kScopeName_ORIONSKYQUEST,
																	kScopeName_NGCMAX,
																	kScopeName_LUMICON,
																	kScopeName_DSTARFINDER
																};
																	//kScopeName_U2K,
																	//kScopeName_U2KCOMPATIBLE,

///////////////////////////////////////////////////////////////////////////
// Alignment related objects and constants								 //
///////////////////////////////////////////////////////////////////////////

typedef struct {
    char displayText[80];
    double ra;
    double dec;
} BrightStarT;

const short	kBrightStarListSize	= 59;
																	
const BrightStarT starList[kBrightStarListSize] = {
      { "Achernar (alpha Eri), 01h 37m 42.9s, -57 14' 12.5\"", 0.426362, -0.998971 },
      { "Acrux (alpha Cru), 12h 26m 35.9s, -63 05' 57\"", 3.257648, -1.101287 },
      { "Adhara (epsilon CMa), 06h 58m 37.5s, -28 58' 20\"", 1.826596, -0.505661 },
      { "Albireo (beta Cyg), 19h 30m 43.2s, +27 57' 35\"", 5.108230, 0.487989 },
	  { "Aldebaran (alpha Tau), 04h 35m 55.2s, +16 30' 33\"", 1.203928, 0.288139 },
      { "Alhena (gamma Gem), 06h 37m 42.7s, +16 23' 57\"", 1.735345, 0.286219 },
      { "Al Nair (alpha Gru), 22h 08m 13.9s, -46 57' 40\"", 5.795505,  -0.819628 },      
	  { "Alnilam (epsilon Ori), 05h 36m 12.8s, -01 12' 07\"", 1.467007, -0.020978 },
      { "Alnitak (zeta Ori), 05h 40m 45.5s, -01 56' 34\"", 1.486839, -0.033908 },
      { "Alpha Centauri (alpha Cen), 14h 39m 33.8s, -60 49' 59.8\"", 3.837819, -1.061741 },
	  { "Alphard (alpha Hya), 09h 27m 35.2s, -08 39' 31\"", 2.476564, -0.151121 },
      { "Alphecca (alpha CrB), 15h 34m 41.3s, +26 42' 53\"", 4.078347, 0.466260 },
      { "Alpheratz (alpha And), 00h 08m 23.3s, +29 05' 26\"", 0.036601, 0.507726 },
      { "Altair (alpha Aql), 19h 50m 47.0s, +08 52' 06\"", 5.195772, 0.154782 },
      { "Aludra (eta CMa), 07h 24m 05.7s, -29 18' 11\"", 1.937730, -0.511435 },
      { "Antares (alpha Sco), 16h 29m 24.4s, -26 25' 55\"", 4.317101, -0.461324 },
      { "Arcturus (alpha Boo), 14h 15m 39.7s, +19 10' 57\"", 3.733528, 0.334798 },
      { "Arneb (alpha Lep), 05h 32m 43.8s, -17 49' 20\"", 1.451809, -0.311056 },
      { "Bellatrix (gamma Ori), 05h 25m 07.9s, +06 20' 59\"", 1.418655, 0.110824 },
      { "Betelgeuse (alpha Ori), 05h 55m 10.3s, +07 24' 25\"", 1.549729, 0.129276 },
	  { "Canopus (alpha Car), 06h 23m 57.1, -52 41' 44.3\"", 1.675305, -0.919712 },
	  { "Capella (alpha Aur), 05h 16m 41.5, +45 59' 50\"", 1.381830, 0.802804 },
	  { "Castor (alpha Gem), 07h 34m 36.0s, +31 53' 18\"", 1.983567, 0.556556 },
      { "Deneb (alpha Cyg), 20h 41m 25.8s, +45 16' 49\"", 5.416760, 0.790290 },
      { "Deneb Kaitos (beta Cet), 00h 43m 35.4s, -17 59' 12\"", 0.190197, -0.313927 },
      { "Denebola (beta Leo), 11h 49m 03.6s, +14 34' 19\"", 3.093858, 0.254328 },
      { "Dschubba (delta Sco), 16h 00m 20.0s, -22 37' 18\"", 4.190245, -0.394823 },
      { "Elnath (beta Tau), 05h 26m 17.5s, +28 36' 27\"", 1.423716, 0.499295 },
      { "Enif (epsilon Peg), 21h 44m 11.2s, +09 52' 30\"", 5.690588, 0.172351 },
      { "Fomalhaut (alpha PsA), 22h 57m 39.1s, -29 37' 20\"", 6.011139, -0.517005 },
      { "Gienah (epsilon Cyg), 20h 46m 12.7s, +33 58' 13\"", 5.437624, 0.592893 },
      { "Hamal (alpha Ari), 02h 07m 10.4s, +23 27' 45\"", 0.554898, 0.409498 },
      { "Kaus Australis (epsilon Sgr), 18h 24m 10.3s, -34 23' 05\"", 4.817858, -0.600127 },
      { "Markab (alpha Peg), 23h 04m 45.7s, +15 12' 19\"", 6.042163, 0.265382 },
      { "Menkar (alpha Cet), 03h 02m 16.8s, +04 05' 23\"", 0.795347, 0.071379 },
      { "Menkent (theta Cen), 14h 06m 41.0s, -36 22' 12\"", 3.694353, -0.634776 },
      { "Mintaka (delta Ori), 05h 32m 00.4s, +00 17' 57\"", 1.448652, 0.005221 },
      { "Mirach (beta And), 01h 09m 43.9s, +35 37' 14\"", 0.304262, 0.621696 },
      { "Mirfak (alpha Per), 03h 24m 19.4s, +49 51' 40\"", 0.891532, 0.870240 },
	  { "Mirzam (beta CMa), 06h 22m 42.0s, -17 57' 21\"", 1.669844, -0.313388 },
	  { "Mizar (zeta Uma), 13h 23m 55.7s, +54 55' 31\"", 3.507798, 0.958627 },
	  { "Navi (gamma Cas), 00h 56m 42.7s, +60 43' 00\"", 0.247453, 1.059706 },
	  { "Nunki (sigma Sgr), 18h 55m 15.9s, -26 17' 48\"", 4.953528, -0.458963 },
      { "Polaris (alpha Umi), 02h 31m 50.5s, +89 15' 51\"", 0.662536, 1.557952 },
	  { "Pollux (beta Gem), 07h 45m 18.9s, +28 01' 34\"", 2.030320, 0.489148 },
      { "Procyon (alpha CMi), 07h 39m 18.1s, +05 13' 30\"", 2.004082, 0.091193 },
      { "Rasalhague (alpha Oph), 17h 34m 56.1s, +12 33' 36\"", 4.603022, 0.219213 },
      { "Regulus (alpha Leo), 10h 08m 22.3s, +11 58' 02\"", 2.654522, 0.208867 },
      { "Rigel (beta Ori), 05h 14m 32.3s, -08 12' 06\"", 1.372432, -0.143146 },
      { "Sabik (eta Oph), 17h 10m 22.7s, -15 43' 29\"", 4.495874, -0.274448 },
      { "Saiph (kappa Ori), 05h 47m 45.4s, -09 40' 11\"", 1.517375, -0.168768 },
      { "Scheat (beta Peg), 23h 03m 46.5s, +28 04' 58\"", 6.037857, 0.490137 },
      { "Shaula (lambda Sco), 17h 33m 36.5s, -37 06' 14\"", 4.597234, -0.647585 },
      { "Sirius (alpha Cma), 06h 45m 08.9s, -16 42' 58\"", 1.767793, -0.291751 },
      { "Spica (alpha Vir), 13h 25m 11.6s, -11 09' 41\"", 3.513319, -0.194803 },
      { "Al Suhail (lambda Vel), 09h 07m 59.9s, -43 25' 57\"", 2.391092, -0.758042 },
	  { "Vega (alpha Lyr), 18h 36m 56.3s, +38 47' 01\"", 4.873563, 0.676902 },
      { "Wezen (delta CMa), 07h 08m 23.5s, -26 23' 36\"", 1.869211, -0.460651 },
      { "Zosma (delta Leo), 11h 14m 06.5s, +20 31' 25\"", 2.941352, 0.358205 }
};

