//----------------------------------------------------------------------
//	Copyright  2000-2005 by Imaginova Canada, Inc., All rights reserved.
//				2-star alignment procedures adapted from David Ek's DSC ASCOM driver with permission of the author
//
//	File:		Controller.cpp
//
//	Contains:	Default telescope controller for Starry Night Pro X (4.0).
//
//				Starry Night communicates with the plug-in through the DoOperation
//				function (see description below). The plugin communicates with Starry
//				night through a callback API, whose functions are defined in 
//				StarryightPlugins.h
//
//				Starry Night provides a "Telescope" panel by default. Our plug-in
//				is called with one of the following op codes whenever the user selects 
//				one of the Telescope panel items (e.g. a button or checkbox is clicked):
//
//					Telescope Tab
//					------------
//					Configure Connection...		op_TelescopeConfigure
//					------------
//					Sync						op_TelescopeSync
//					------------
//					Connect/Disconnect			op_TelescopeOpen/op_TelescopeClose
//					------------
//					Slew						op_TelescopeSlew
//					------------
//					Follow	(checkbox)			op_TelescopeTrack
//
//				This plug-in also adds one (or two if the scope can synch) menu items to
//				Starry Night's contextual popup menus that appear whenever a user right-clicks
//				on an item. These two menu items are "Slew to [object name]" and "Synch on [object name]".
//
//	Authors:	Dave Whipps, Ted Leckie, Tom Andersen
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	99/02/01	Tom			3.0			Version 3 offers callbacks, also x-platform plug-ins
//	01/11/23	DWW			4.0			New for Pro X
//  05/03/11	MK			5.02		Added support for 2-star alignment (see DoTelescopeAlign,
//											DoGetAlignedRaDec)
//----------------------------------------------------------------------

#include "ControllerPluginUtils.h"
#include "Transfrm.h"
#include "Scope.h"
#include "ScopeConfigBox.h"
#include "ListBox.h"

// These are errors that get sent back to SN after an op_something call has been made to the
// plugin and there was a problem executing the command. Don't confuse these with snErr's which are
// used with the sn_something API calls (of type long and < 0) and the internally by the plugin (with 
// values of type long and are > 0)
#define noError				0
#define undefinedError		1


// Some global variables for use throughout
//---------------------------------------------------------------------------------------
PCentreAction			gCentreAction;
PGazeAction 			gGazeAction;
TScope*					gScope 							= NULL;
TScopeConfigBox*		gScopeConfigureBox				= NULL;
TListBox*				gStarListBox					= NULL;
TSerialCommunicator*	gSerialCommunicator 			= NULL;
GSXSerialPortNameRecordHandle 	gPortNameRecordHandle 	= NULL;
UInt16					gNumberOfReadySerialPorts		= 0;
UInt32 					gSelectedSerialPortIndex		= 0;
UInt16					gListSelectionIndex				= 0;                   /*list box selection index*/
scopeType				gSelectedScopeType				= scope_NONE;
GSXSerialPortNameRecord gSelectedPortNameRecord			= {0,0,0};
GSSerialPortInfoRecord 	gDefaultPortInfoRecord			= {	9600,				/*BAUD RATE*/
															8, 					/*DATA BITS*/
															kGSSerialParityNone,/*PARITY*/
															kGSSerialStopBits10	/*STOP BITS*/
															};
pBoolean				gInitialSetupComplete					= false;
pBoolean 				gConnectionOpened 						= false;
pBoolean				gIsConfigurationComplete 				= false;
pBoolean 				gIsTracking 							= false;
pBoolean				gScopeCanSynch 							= false;
pBoolean				gScopeCanSlew							= false;
pBoolean				gCanDirSlew								= false;
pBoolean				gCanSetScopeTimeAndLocation				= false;
double					gLastPositionUpdate						= 0;
double					gAlignStartTime							= 0;             /*alignment start time(in seconds)*/
double					gAlignAltOffset							= 0;             /*alt offset calculated during alignment*/
pBoolean				gDecReverse								= false;		 /*is the alt encoder reversed?*/
pBoolean				gScopeAlignmentInProgress				= false;         /*alignment in progress flag*/
pBoolean				gAlignedInSoftware						= false;		 /*aligned using 2-star alignment flag*/

// New stuff added for Pro Plus
pBoolean				gScopeCanPark			= false;
pBoolean				gScopeCanUnpark			= false;
pBoolean				gScopeCanSetPark		= false;
pBoolean				gIsParked				= false;
pBoolean				gScopeCanGoHome			= false;


// Initial time location setup action to perform
/*
enum {	
		action_SetSNToScopeTimeLocationValues = 1,
		action_SetScopeToSNTimeLocationValues = 2,
		action_None = 3};
SInt32 					gSetTimeLocationAction 					= action_None;
		*/


// We (carefully) maintain the right ascension and declination internally and update these as often as possible
// without polling the scope too much since polling the scope too often can cause serious problems
double					gCurrentScopeRightAscension		= coordinate_NotAvailable;
double					gCurrentScopeDeclination		= coordinate_NotAvailable;
//---------------------------------------------------------------------------------------

// Function prototypes
// Note that the protype for DoOperation is supplied to us in StarryNightPlugins.h
snErr 	GetTelescopeRaDec(	double* outCurrentScopeRightAscension, 
							double* outCurrentScopeDeclination, 
							pBoolean scopeQueryRequired);
void 	DoMenuStatus(PMenuStatus* ioMenuRecord);
void 	DoInitialize(PInitialize* outInitRecord);
void 	DoTelescopeConfigure(void);
SInt16	DoShowAlert(const char* title, const char* text, const char* buttonOK, const char* buttonCancel);
void	SetTelescopeToCurrentSNTimeAndLocation(void);

void 	DoTelescopeTime(PTime* inTime);
void 	DoTelescopeLocation(PLocation* inLocation);
short 	DoTelescopeOpen(void);
short 	DoTelescopeClose(void);
void 	DoTelescopeTrack(void);
void	DoParkScope(void);
void	DoUnparkScope(void);
void	DoSetScopeParkPosition(void);
void	DoScopeGoHome(void);
snErr   DoTelescopeAlign(void);
snErr	DoGetAlignedRaDec(double *ra, double *dec);
			
void 	DoIdle(PEvent* ioEvent);
void 	DoDrawIndicator(PIndicator* outIndicator);
void 	DoBuildContextualPopup(const PBuildContext* inBuildContext);
void 	DoContextualPopup(const PDoContext* inContext);

void 	SlewTelescope(PGaze* slewGaze);
void 	SlewTelescope(const PSelection inObject);
void 	SlewTelescope(const PScopeMotion* inMotionParam);
void 	SynchTelescope(PGaze* synchGaze);
void 	SynchTelescope(const PSelection inObject);

#if TARGET_OS_WIN32
HINSTANCE gDLLInstance;
//----------------------------------------------------------------------
//	DLLMain
//
//	Purpose:	DLL initialization called first thing
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------
extern "C" BOOL WINAPI DllMain ( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpvReserved );

BOOL WINAPI DllMain( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpReserved )
{

	::MessageBox(::GetFocus(), "DLLMain Called", "Telescope Controller", MB_OK|MB_SYSTEMMODAL);

	// we need to get the HINSTANCE of this module
	switch(wDataSeg)
	{
			case DLL_PROCESS_ATTACH:
			gDLLInstance = hInst;
	  		break;
	}
	return 1;
}
#endif /*TARGET_OS_WIN32*/


//----------------------------------------------------------------------
//	GetTelescopeRaDec
//
//	Purpose:	Gets the Ra and Dec of the telescope if scopeQueryRequired is true
//				otherwise gets the cached values.
//					
//	Inputs:		outRightAscension	RA in epoch of current date in decimal radians.
//				outDeclination		Dec in epoch of current date in decimal radians.
//
//	Returns:	A starry night error code
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	02/07/08	DWW			4.0			New for Pro 4.0
//----------------------------------------------------------------------
snErr GetTelescopeRaDec(	double* outCurrentScopeRightAscension,
							double* outCurrentScopeDeclination, 
							pBoolean scopeQueryRequired)
{
	snErr error = snNoErr;

	if (gConnectionOpened)
	{
		if (scopeQueryRequired)
		{
			double returnedRa	= 0.0;
			double returnedDec	= 0.0;
			
			error = gScope->GetScopeRaDec(returnedRa, returnedDec);
			
			// Only reset the globals if the "get" was a success
			if (error == snNoErr)
			{
				gCurrentScopeRightAscension		= returnedRa;
				gCurrentScopeDeclination		= returnedDec;

				*outCurrentScopeRightAscension	= gCurrentScopeRightAscension;
				*outCurrentScopeDeclination		= gCurrentScopeDeclination;			
			}
		}
		else
		{
			error = snNoErr;

			*outCurrentScopeRightAscension 	= gCurrentScopeRightAscension;
			*outCurrentScopeDeclination 	= gCurrentScopeDeclination;			
		}
	}

	return error;
}


//----------------------------------------------------------------------
//	DoOperation
//
//	Purpose:	Starry Night communicates with your plug-in through one call, DoOperation.
//				(and hence DoOperation is exported using #pragma export).
//					
//	Inputs:		inOperation	the operation that Starry Night wishes your plug-in to perform
//							(StarryNightPlugins.h describes each operation). You need not
//							support every operation. This example responds to all the
//							operations that a telescope controller plug-in will need to
//							respond to. You may chose to respond to others as well, if you
//							wish.
//
//	I/O:		ioParams	a parameter block that passes various information back and forth
//							between Starry Night and the plug-in. Each operation uses
//							a different paramater block or none at all, in which case
//							ioParams will be NULL. The description of the various operations
//							(in StarryNightPlugins.h) describes what ioParams means for each
//							one.
//
//	Outputs:	short		Unused. You should always pass back noError here.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	02/07/08	DWW			4.0			New for Pro 4.0
/// 05/03/02	MK			5.02		Support for telescope handbox/homing/park/unpark
//----------------------------------------------------------------------
extern "C"{
SNCall short DoOperation(OperationT inOperation, void* ioParams)
{
	// When there is no connection to the telescope, we don't respond to many operations
	switch (inOperation)
	{
		case op_Initialize:
			::DoInitialize((PInitialize*) ioParams);
			break;
		case op_Terminate:
			::DoTelescopeClose(); // Just to keep things in order
			break;
		case op_MenuStatus:
			::DoMenuStatus((PMenuStatus*) ioParams);
			break;
		case op_TelescopeConfigure:
			::DoTelescopeConfigure();
			break;
		case op_TelescopeTime:
			if (gConnectionOpened && gCanSetScopeTimeAndLocation)
				::DoTelescopeTime((PTime*) ioParams);
			break;
		case op_TelescopeLocation:
			if (gConnectionOpened && gCanSetScopeTimeAndLocation)
				::DoTelescopeLocation((PLocation*) ioParams);
			break;
		case op_LocationChanged:
			break;
		case op_LatLongChanged:
			if (gConnectionOpened)
				::SetStatusText(&kMessage_SNLocationChanged, &kInformative_SNLocationChanged);
			break;		
		case op_TelescopeOpen:
			::DoTelescopeOpen();
			break;
		case op_TelescopeClose:
			::DoTelescopeClose();
			break;
		case op_TelescopeTrack:
			if (gConnectionOpened)
				::DoTelescopeTrack();
			break;
		case op_TelescopeSlew:
			if (gConnectionOpened)
				::SlewTelescope(((PGaze*) ioParams));
			break;
		case op_TelescopeMotion:
			if (gConnectionOpened)
				::SlewTelescope(((PScopeMotion*) ioParams));
			break;
		case op_TelescopeSync:
			if (gConnectionOpened && gScopeCanSynch)
				::SynchTelescope(((PGaze*) ioParams));
			break;
		case op_Idle:
			if (gConnectionOpened)
				::DoIdle((PEvent*) ioParams);
			break;
		case op_DrawIndicator:
			if (gConnectionOpened)
				::DoDrawIndicator((PIndicator*) ioParams);
			break;
		case op_BuildContextualPopup:
			if (gConnectionOpened)
				::DoBuildContextualPopup((PBuildContext*) ioParams);
			break;
		case op_DoContextualPopup:
			if (gConnectionOpened)
				::DoContextualPopup((PDoContext*) ioParams);
			break;
		case op_TelescopePark:
			if (gConnectionOpened)
				::DoParkScope();
			break;
		case op_TelescopeUnpark:
			if (gConnectionOpened)
				::DoUnparkScope();
			break;
		case op_TelescopeSetParkPosition:
			if (gConnectionOpened)
				::DoSetScopeParkPosition();
			break;
		case op_TelescopeGoHome:
			if (gConnectionOpened)
				::DoScopeGoHome();
			break;
		default:
			break;
	}
	return noError;
}

}


//----------------------------------------------------------------------
//	DoMenuStatus
//
//	Purpose:	Starry Night queries us to determine which Telescope panel items
//				should be enabled/disabled, checked/unchecked.
//					
//	Inputs:		ioMenuRecord	structure that informs us which command we are being
//								queried on. We also return status information in
//								this record.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	02/04/05	DWW			4.0			Changes for ProX
//	05/03/02	MK			5.02		Support for telescope homing/park/unpark
//----------------------------------------------------------------------
void DoMenuStatus(PMenuStatus* ioMenuRecord)
{
	// Most of the items don't use checkmarks (the only exception is the
	// "Follow" (op_TelescopeTrack) item.
	ioMenuRecord->fHasCheck = false;
	ioMenuRecord->fIsEnabled = false; 

	// We set fIsEnabled true for each menu item we support.
	switch (ioMenuRecord->fMenuCommand)
	{
		case cmd_TelescopeConfigure:
			// Always enable the configure button
			ioMenuRecord->fIsEnabled = true; 
			break;
		case cmd_TelescopeOpen:
			// if the connection to the telescope is already open then we disable the "Open Connection" menu item.
			ioMenuRecord->fIsEnabled = (gIsConfigurationComplete && !gConnectionOpened);
			break;
		case cmd_TelescopeClose:
			// We only enable the "Close Connection" menu item if we already have an open connection.
			ioMenuRecord->fIsEnabled = gConnectionOpened;
			break;
		case cmd_TelescopeTrack:
			// "check" this menu item when tracking (follow) is in effect, and uncheck it when it isn't.
			ioMenuRecord->fIsEnabled = (gIsConfigurationComplete && gConnectionOpened && !gIsParked);
			ioMenuRecord->fHasCheck = gIsTracking;
			break;
		case cmd_TelescopeSlew:
			// Disable this menu command when we are tracking.
			ioMenuRecord->fIsEnabled = (gIsConfigurationComplete && gScopeCanSlew && gConnectionOpened && !gIsTracking && !gIsParked);
			break;
		case cmd_TelescopeSync:
			ioMenuRecord->fIsEnabled = (gIsConfigurationComplete && gConnectionOpened && gScopeCanSynch);
			break;
		case cmd_TelescopePark:
			if (gIsConfigurationComplete && gConnectionOpened && gScopeCanPark)
				ioMenuRecord->fIsEnabled = !gIsParked;
			else
				ioMenuRecord->fIsEnabled = false;
			break;
		case cmd_TelescopeUnpark:
			if (gIsConfigurationComplete && gConnectionOpened && gScopeCanUnpark)
				ioMenuRecord->fIsEnabled = gIsParked;
			else
				ioMenuRecord->fIsEnabled = false;
			break;
		case cmd_TelescopeGoHome:
			ioMenuRecord->fIsEnabled = (gIsConfigurationComplete && gConnectionOpened && gScopeCanGoHome);
			break;
		case cmd_TelescopeSetParkPosition:
			ioMenuRecord->fIsEnabled = (gIsConfigurationComplete && gConnectionOpened && gScopeCanSetPark);
			break;
		case cmd_TelescopeDirectionalSlew:
			ioMenuRecord->fIsEnabled = (gIsConfigurationComplete && gConnectionOpened && gCanDirSlew);
			break;
		default:
			ioMenuRecord->fIsEnabled = false;
			break;
	}	
}

//----------------------------------------------------------------------
//	DoInitialize
//
//	Purpose:	Starry Night calls our plug-in with op_Initialize at when it is
//				starting up. We inform Starry Night about our needs and wants.
//				We could also perform one time memory allocations, etc here.
//					
//	Inputs:		ioParams	we must fill out this parameter block to inform
//							Starry Night of our needs.
//							If for any reason our initialization fails, we need
//							to set ioParams->fSuccess false.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/04/05	DWW			4.0			Changes for ProX
//	98/07/29	ETL			1.0			New
//----------------------------------------------------------------------
void DoInitialize(PInitialize* outInitRecord)
{	
	outInitRecord->fSuccess 				= true;				// has to be true for our plugin to load

	if (outInitRecord->fPluginAPI < kVersion3Plugin)
	{
		outInitRecord->fSuccess = false;
		return;
	}

	outInitRecord->fPluginAPI 				= kLatestVersion;	// this is the API we use
	outInitRecord->fWantsMenuItem 			= false;			// wants a "Settings" menu item (no longer supported, set to false)
	outInitRecord->fOptionsMenuItem[0] 		= 0;				// text of menu item, (no longer supported, set to 0)
	outInitRecord->fWantsWindow 			= false;			// 
	outInitRecord->fWindowMenuItem[0] 		= 0;				// if we had a window, name of menu item for opening and closing it
	outInitRecord->fLayer 					= Depth_Telrad;		// pass back Depth_None if you don't do any drawing
	outInitRecord->fIsTelescopeController 	= true;				//
	outInitRecord->fAboutMenuItem[0] 		= 0;				// we don't bother with an About Box
	outInitRecord->fWantsIdleTime			= true;				//
}

//----------------------------------------------------------------------
//	DoTelescopeConfigure
//
//	Purpose:	Opens the configuration dialog box allowing the user to
//				select the serial port and scope type for use.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/03/01	DWW			4.0			New for ProX
//----------------------------------------------------------------------
void DoTelescopeConfigure(void)
{
	gScopeConfigureBox = TScopeConfigBox::CreateNewConfigBox();
	
	if (gScopeConfigureBox)
		gScopeConfigureBox->DisplayAndRunConfigBox();	

	if (gSerialCommunicator)
	{
		gIsConfigurationComplete = true;
		::SetStatusText(&kMessage_ConnectionClosed, &kInformative_ConnectionClosed);
	}

}

//----------------------------------------------------------------------
//	DoShowAlert
//
//	Purpose:	Displays an alert box with two buttons. Returns the number of the
//				button pressed or 0 if problems are encountered.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/14	MK			5.02		Added to support telescope alignment
//----------------------------------------------------------------------
SInt16 DoShowAlert(const char* title, const char* text, const char* buttonOK, const char* buttonCancel)
{
	SInt16 buttonpressed;
	AlertStdAlertParamRec windowProperties;
	LStr255 buttonone(buttonOK);
	LStr255 buttontwo(buttonCancel);
	LStr255 warningtitle(title);
	LStr255 warningtext(text);
	
	windowProperties.movable = false;
	windowProperties.helpButton = false;
	windowProperties.filterProc = NULL;
	windowProperties.defaultText = StringPtr(buttonone);
	windowProperties.cancelText = StringPtr(buttontwo);
	windowProperties.otherText = NULL;
	windowProperties.defaultButton = kAlertStdAlertOKButton;
	windowProperties.cancelButton = kAlertStdAlertCancelButton;
	windowProperties.position = kWindowDefaultPosition;
	
	if (StandardAlert(kAlertNoteAlert, StringPtr(warningtitle), StringPtr(warningtext), &windowProperties, &buttonpressed) != noErr)
		return 0;
	else	
		return buttonpressed;
}

//----------------------------------------------------------------------
//	DoTelescopeTime
//
//	Purpose:	The user has requested to set the telescope time on connect
//					
//	Inputs:		inTime		a record describing the current time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	02/07/08	DWW			4.0			New for Pro 4.0
//----------------------------------------------------------------------
void DoTelescopeTime(PTime* inTime)
{
	snErr	error = snNoErr;
		
	PluginAssert(gCanSetScopeTimeAndLocation);

	if (gCanSetScopeTimeAndLocation) 
	{	
		// Need the timezone offset since a PTime is in Julian date
		long timeZoneOffsetInSeconds = 0;
		pBoolean isDSTInEffect = false;
		
		error = ::sn_GetTimeZoneSeconds(kFrontWindow, &timeZoneOffsetInSeconds, &isDSTInEffect);

		gScope->SetScopeTime(inTime->fJulianDate, timeZoneOffsetInSeconds, isDSTInEffect);
	}
}

//----------------------------------------------------------------------
//	DoTelescopeLocation
//
//	Purpose:	The user has requested to set the telescope location on connect
//					
//	Inputs:		inLocation	record describing the current location.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	02/07/08	DWW			4.0			New for Pro 4.0
//----------------------------------------------------------------------
void DoTelescopeLocation(PLocation* inLocation)
{
	PluginAssert(gCanSetScopeTimeAndLocation);

	if (gCanSetScopeTimeAndLocation)
		gScope->SetScopeLocation(inLocation->fEarthLatitude, inLocation->fEarthLongitude);
}

//----------------------------------------------------------------------
//	DoTelescopeOpen
//
//	Purpose:	The user selects "Connect" to initiate the connection
//				between the computer and the telescope.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	02/07/08	DWW			4.0			New for Pro 4.0
//	05/03/02	MK			5.02		Added support for telescopes that need 2-star alignment
//										done in sofware
//----------------------------------------------------------------------
short DoTelescopeOpen(void)
{
	snErr error = snNoErr;

	if (gIsConfigurationComplete && !gConnectionOpened)
	{
		pBoolean isConnectionInitialized = false;
		
		PluginAssert(gSerialCommunicator);
		isConnectionInitialized = gSerialCommunicator->GetIsSerialConnectionInitialized();

		// It should already be initialized, but if not, initialize it here...
		if(isConnectionInitialized == false)
		{
			gSerialCommunicator->InitSerialComm();
			isConnectionInitialized = gSerialCommunicator->GetIsSerialConnectionInitialized();
			PluginAssert(isConnectionInitialized);
		}

		PluginAssert(gSelectedScopeType != scope_NONE);
		gScope = TScope::MakeNewTScope(gSerialCommunicator, gSelectedScopeType);
		PluginAssert(gScope);
		
		// Open the scope connection
		error = gScope->OpenScopeConnection();
		::HandleError(error);
		
		gConnectionOpened = gScope->GetIsConnected();

		if (gScope && gConnectionOpened)
		{	
			// Setup the global variables for the scope to control the menu items
			gScopeCanSynch				= gScope->GetCanSynch();
			gScopeCanSlew				= gScope->GetCanSlew();
			gCanSetScopeTimeAndLocation = (gScope->GetCanSetScopeTime() && gScope->GetCanSetScopeLocation());
			gCanDirSlew					= gScope->GetCanDirSlew();
			
			gScopeCanPark				= gScope->GetCanPark();
			gScopeCanUnpark				= gScope->GetCanUnpark();
			gScopeCanSetPark			= gScope->GetCanSetPark();
			gIsParked					= gScope->GetIsParked();
			gScopeCanGoHome				= gScope->GetCanGoHome();
			
			gLastPositionUpdate			= 0;
			gAlignedInSoftware			= false;			
						
			//perform alignment
			// Note: keep the alignment procedure in a loop in case of problems
			if (gScope->GetMustAlignInSoftware() ) {
				error = snNoErr;
				gScopeAlignmentInProgress = true;
				do {
					if (error != snNoErr) {
						int result = DoShowAlert("Alignment procedure experienced an error","Please check communication with the telescope and make sure that you are pointing at the appropriate stars.","Try aligning again","Cancel alignment");
		
						if (result == kAlertStdAlertCancelButton) {
							error = pluginErr_UserCancelled;
							break;
						}
						else if (!result)
							return pluginErr_Failed;
					}
					error = DoTelescopeAlign();
				} while ((error != snNoErr) && (error != pluginErr_UserCancelled));
				
				if (error == snNoErr)
					gAlignedInSoftware = true;

				gScopeAlignmentInProgress = false;
			}			
									
			// Initialize the plugin internal Ra and Dec to current scope Ra and Dec
			error = ::GetTelescopeRaDec(&gCurrentScopeRightAscension, &gCurrentScopeDeclination, true);
			
			// Set the scope status and pan to the initial scope coords	
			if (error == snNoErr)
				::SetStatusText(&kMessage_Tracking, &kInformative_Tracking, gCurrentScopeRightAscension, gCurrentScopeDeclination);
			else
			{
				gConnectionOpened = false;
				::HandleError(error);
			}
		}
	}
	
	return error;
}

//----------------------------------------------------------------------
//	DoTelescopeClose
//
//	Purpose:	The user selects "Close Connection" to terminate the connection
//				between the computer and the telescope. When you get called here
//				you should find the telescope and close the connection. We
//				disable the telescope menu items when the connection is closed.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//----------------------------------------------------------------------
short DoTelescopeClose(void)
{
	snErr	error = snNoErr;

	if (gScope)
	{
		gScope->CloseScopeConnection();
		gConnectionOpened = gScope->GetIsConnected();
		PluginAssert(!gConnectionOpened);
		delete gScope;
		gScope = NULL;
	}

	gCurrentScopeRightAscension	= coordinate_NotAvailable;
	gCurrentScopeDeclination	= coordinate_NotAvailable;

	// reset the global variables to false
	gIsTracking							= false;
	gInitialSetupComplete	 			= false;
	gScopeCanSynch						= false;
	gScopeCanSlew						= false;
	gCanSetScopeTimeAndLocation			= false;
	gCanDirSlew							= false;

	gScopeCanPark					= false;
	gScopeCanUnpark					= false;
	gScopeCanSetPark				= false;
	gIsParked						= false;
	gScopeCanGoHome					= false;
					
	::SetStatusText(&kMessage_ConnectionClosed, &kInformative_ConnectionClosed, coordinate_NotAvailable, coordinate_NotAvailable);
	
	::sn_ImmediateUpdateWindow(kFrontWindow);

	return error;
}

//----------------------------------------------------------------------
//	DoTelescopeTrack
//
//	Purpose:	When the user puts our telescope controller into tracking mode then
//				it becomes our respondibility to keep Starry Night's view aligned with
//				the telescope. We do that when we receive an op_Idle event. See DoIdle
//				for more details.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------
void DoTelescopeTrack(void)
{
	gIsTracking = !gIsTracking;
}

//----------------------------------------------------------------------
//	DoParkScope
//
//	Purpose:	Parks the telescope
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for pro plus advanced scope control
//
//----------------------------------------------------------------------
void DoParkScope(void)
{
	gScope->ParkScope();
	gIsParked = gScope->GetIsParked();
}

//----------------------------------------------------------------------
//	DoUnparkScope
//
//	Purpose:	Un-Parks the telescope
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for pro plus advanced scope control
//
//----------------------------------------------------------------------
void DoUnparkScope(void)
{
	gScope->UnparkScope();
	gIsParked = gScope->GetIsParked();
}

//----------------------------------------------------------------------
//	DoSetScopeParkPosition
//
//	Purpose:	Sets the park position of the telescope to whereever it currently is.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for pro plus advanced scope control
//
//----------------------------------------------------------------------
void DoSetScopeParkPosition(void)
{
	gScope->SetParkPosition();
}

//----------------------------------------------------------------------
//	DoScopeGoHome
//
//	Purpose:	Moves the scope to its predefined 'Home' position.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for pro plus advanced scope control
//
//----------------------------------------------------------------------
void DoScopeGoHome(void)
{
	gScope->MoveScopeToHomePosition();
}


//----------------------------------------------------------------------
//	DoTelescopeAlign
//
//	Purpose:	Prompt the user for needed information and then align the telescope
//				using 2-star alignment
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/11	MK			5.0.2		New for telescope 2-star alignment
//
//----------------------------------------------------------------------

snErr DoTelescopeAlign(void) {

	dircos_t scopeCosines[ 3 ]; // the 3x3 matrix of direction cosines in scope coordinates
	dircos_t skyCosines[ 3 ];   // the 3x3 matrix of direction cosines in sky coordinates

	double ra1; // RA of star1 (radians)
	double dec1; // DEC of star1 (radians)
	double scopeAz1; // scope AZ of star1 (radians)
	double scopeAlt1; //scope ALT of star1 (radians)
	double ra2; // same quantities for star 2
	double dec2;
	double scopeAz2;
	double scopeAlt2;
	double timenow; //current time as julian date
	int result; // the result from various calls-- for fnctns in Transfrrm.cpp, 1 is good, 0 is bad	

	///
	/// TELL THE USER THAT ALIGNMENT IS ABOUT TO BE INITIATED
	///
	
	result = DoShowAlert("Two-star alignment is about to begin","The alignment procedure requires that you point your telescope at two stars of your choice when prompted. If you wish to repeat this procedure or perform it later on you will have to disconnect and then reconnect.","Align","Don't align");
		
	if (result == kAlertStdAlertCancelButton)
		return pluginErr_UserCancelled;
	else if (!result)
		return pluginErr_Failed;
						
	///
	/// GET THE USER TO MOVE THE SCOPE TO 90 degrees altitude (or 90 dec for equatorials). This is needed to check for alt encoder reversals
	///	
	result = DoShowAlert("Move telescope to ninety degrees altitude/declination","For a dobsonian, this means pointing at the zenith. For an equatorial mount, this means pointing at Polaris (assuming you are roughly polar-aligned).\nNote: Extreme accuracy is not required - within five or ten degrees is adequate.","Telescope is in position","Cancel alignment");
		
	if (result == kAlertStdAlertCancelButton)
		return pluginErr_UserCancelled;
	else if (!result)
		return pluginErr_Failed;

	if ( gScope->GetScopeAzAlt(scopeAz1, scopeAlt1) != snNoErr )
		return pluginErr_CommunicationError;

	if ( scopeAlt1 > kPiDefine )
		gDecReverse = true;
	else
		gDecReverse = false;
					
	///
	/// GET THE USER TO POINT THE SCOPE AT THE FIRST STAR
	///	
	gStarListBox = new TMacListBox("Two-star alignment: First star","Please choose a star and point your telescope to it.");

	if (gStarListBox) {
		gStarListBox->DisplayAndRunListBox();
		if (!gStarListBox->m_OK)
			return pluginErr_UserCancelled;
	}
	else
		return pluginErr_Failed;
	
	sn_GetCurrentTime(kFrontWindow,&gAlignStartTime);

	ra1 = starList[gListSelectionIndex].ra;
	dec1 = starList[gListSelectionIndex].dec;
	if ( gScope->GetScopeAzAlt(scopeAz1, scopeAlt1) != snNoErr )
		return pluginErr_Failed;
		
	delete(gStarListBox);
	

	///
	/// GET THE USER TO POINT THE SCOPE AT THE SECOND STAR
	///	
	gStarListBox = new TMacListBox("Two-star alignment: Second star","Please choose a star and point your telescope to it.");

	if (gStarListBox) {
		gStarListBox->DisplayAndRunListBox();
		if (!gStarListBox->m_OK)
			return pluginErr_UserCancelled;
	}
	else
		return pluginErr_Failed;
	
	sn_GetCurrentTime(kFrontWindow,&timenow);
	
	double deltaTinHrs = (timenow - gAlignStartTime) * 24.0;

	ra2 = starList[gListSelectionIndex].ra - deltaTinHrs*kRadiansPerHour*kSiderealPerSolar;
	dec2 = starList[gListSelectionIndex].dec;
	if ( gScope->GetScopeAzAlt(scopeAz2, scopeAlt2) != snNoErr )
		return pluginErr_CommunicationError;
		
	delete(gStarListBox);


	///
	/// DO NECESSARY alt/az adjustements and look for alt offset if there should be one
	///

	scopeAz1 = kTwicePi - scopeAz1;
	scopeAz2 = kTwicePi - scopeAz2;
	
	if ( gDecReverse )
	{
		scopeAlt1 = kTwicePi - scopeAlt1;
		scopeAlt2 = kTwicePi - scopeAlt2;
	}


	// now find declination offset from true zero and correct.

	double diff;
	double lastDiff = 0;
	double cosAngSep;
	double offset;
	int i;
	double firstOffset = 0;
	double secondOffset = 0;
	pBoolean closing = false;
	pBoolean firstFound = false;
	pBoolean secondFound = false;

	double M = sin( dec1 ) * sin( dec2 ) + cos( dec1 ) * cos( dec2 ) * cos( ra1 - ra2 );

	for ( i = -5400;i < 5400;i++ ) {
		offset = ( double ) i * kPiDefine / 10800.0;
		cosAngSep = sin( scopeAlt1 + offset ) * sin( scopeAlt2 + offset ) +
		            cos( scopeAlt1 + offset ) * cos( scopeAlt2 + offset ) * cos( scopeAz1 - scopeAz2 );
		diff = fabs( cosAngSep - M );
		if ( diff < lastDiff )
			closing = true;
		else if ( closing ) {
			if ( firstFound ) {
				secondOffset = offset;
				secondFound = true;
				break;
			}
			else {
				firstOffset = offset;
				firstFound = true;
			}
			closing = false;
		}
		lastDiff = diff;
	}

	CoordsToCosines( ra1, dec1, &skyCosines[ 0 ] );
	CoordsToCosines( ra2, dec2, &skyCosines[ 1 ] );
	CrossProduct( skyCosines[ 0 ], skyCosines[ 1 ], &skyCosines[ 2 ] );
	Normalize(&skyCosines[2]);

	double ra3, dec3;
	CosinesToCoords( &skyCosines[ 2 ], &ra3, &dec3 );

	CoordsToCosines( scopeAz1, scopeAlt1, &scopeCosines[ 0 ] );
	CoordsToCosines( scopeAz2, scopeAlt2, &scopeCosines[ 1 ] );
	CrossProduct( scopeCosines[ 0 ], scopeCosines[ 1 ], &scopeCosines[ 2 ] );
	Normalize(&scopeCosines[2]);

	double scopeAz3, scopeAlt3;
	CosinesToCoords( &scopeCosines[ 2 ], &scopeAz3, &scopeAlt3 );

	double angsep1 = sin( scopeAlt1 + firstOffset ) * sin( scopeAlt3 + firstOffset )
	                 + cos( scopeAlt1 + firstOffset ) * cos( scopeAlt3 + firstOffset )
	                 * cos( scopeAz1 - scopeAz3 );

	double angsep2 = sin( scopeAlt1 + secondOffset ) * sin( scopeAlt3 + secondOffset )
	                 + cos( scopeAlt1 + secondOffset ) * cos( scopeAlt3 + secondOffset )
	                 * cos( scopeAz1 - scopeAz3 );

	// the smaller of the two solutions is the correct one, provided we knew the initial
	// angle of the telescope axis well enough (5 or ten degrees, perhaps).
	if ( fabs( angsep1 ) < fabs( angsep2 ) ) {
		gAlignAltOffset = firstOffset;
	}
	else {
		gAlignAltOffset = secondOffset;
	}

	// now correct all the altitude values and the computed offset.
	scopeAlt1 += gAlignAltOffset;
	scopeAlt2 += gAlignAltOffset;


	///
	/// BUILD TRANSFORMATION MATRIX
	///
	
	// convert the coordinates of the two stars to direction cosines and build the
	// two matrices which are used to determine the transformation matrix.

	CoordsToCosines( ra1, dec1, &skyCosines[ 0 ] );
	CoordsToCosines( scopeAz1, scopeAlt1, &scopeCosines[ 0 ] );

	CoordsToCosines( ra2, dec2, &skyCosines[ 1 ] );
	CoordsToCosines( scopeAz2, scopeAlt2, &scopeCosines[ 1 ] );

	// take the cross product of the direction cosines of the stars in each coordinate
	// system to get the third vector.

	CrossProduct( skyCosines[ 0 ], skyCosines[ 1 ], &skyCosines[ 2 ] );
	Normalize(&skyCosines[2]);
	CrossProduct( scopeCosines[ 0 ], scopeCosines[ 1 ], &scopeCosines[ 2 ] );
	Normalize(&scopeCosines[2]);

	// now pass the matrices to InitMatrix, which builds the transformation matrix and
	// stores it.

	result = InitMatrix( scopeCosines, skyCosines ); //trans matrix is stored as a static fwdMatrix and invMatrix
													// defined in Transfrm.h
	
	// InitMatrix returns 0 if it is unsuccessful
	if ( !result ) {
		return pluginErr_MatrixInitError;
	}

	gAlignedInSoftware = true;

	return snNoErr;
}


//----------------------------------------------------------------------
//	DoGetAlignedRaDec
//
//	Purpose:	Calculate Ra/Dec of the telescope using transformation matrix
//				resulting from DoTelescopeAlign
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/11	MK			5.0.2		New for telescope 2-star alignment
//
//----------------------------------------------------------------------

snErr DoGetAlignedRaDec(double *ra, double *dec) {
	double alt, az;
	double timenow;
	int mathfnctresult;

	dircos_t scopeCosines, skyCosines;

	//retrieve az/alt perceived by the scope
	if (gScope->GetScopeAzAlt( az, alt ) != snNoErr )
		return pluginErr_CommunicationError;

	sn_GetCurrentTime(kFrontWindow,&timenow);

	double deltaTinHrs = (timenow - gAlignStartTime) * 24.0;

	//Orion returns coordinates in clockwise orientation but the calculations need them in counterclockwise
	az = kTwicePi - az;

	if ( gDecReverse )
		alt = kTwicePi - alt;

	alt = alt + gAlignAltOffset;

	//math functions in Transfrm.cpp return 1 on success
	mathfnctresult = CoordsToCosines(az, alt, &scopeCosines) && ScopeToSky(&scopeCosines, &skyCosines) && CosinesToCoords(&skyCosines, ra, dec);

	if (!mathfnctresult)
		return pluginErr_Failed;
		
	*ra = *ra + deltaTinHrs * kSiderealPerSolar * kRadiansPerHour;
	*ra = kTwicePi/ gScope->GetScopeEncoderResolution() * floor( *ra * gScope->GetScopeEncoderResolution()/kTwicePi );
	*ra = fmod( *ra, kTwicePi );
	
	return snNoErr;
}


//----------------------------------------------------------------------
//	DoIdle
//
//	Purpose:	Starry Night calls our plug-in with op_Idle during idle moments. If we
//				are "tracking" the telescope (ie. Starry Night is acting as a slave to
//				the telescope) then we use this idle time to re-centre the Starry Night
//				view to the same position as the telescope.
//				Since op_Idle events come fast and furious we only re-centre Starry
//				Night's view once a second.
//					
//	Inputs:		ioEvent		we use this paramater block to tell Starry Night to
//							recentre its view when we are tracking the telescope.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	02/03/01	DWW			4.0			new features for Pro X
//	05/03/02	MK			5.02		Added support for getting RA/Dec from scopes aligned in software
//----------------------------------------------------------------------
void DoIdle(PEvent* ioEvent)
{
	const	double	kSecondsBetweenUpdates = gScope->GetScopeWaitTime()/500.0; //fScopeWaitTime is in ms but we want twice the waiting period
	const   double  currentTime = ::ClockSec();
	
	snErr error = snNoErr;
	// Check if enough time has passed, then read out the current location of the scope.
	// Centre on the scope coordinates if we are tracking.
	if (gConnectionOpened && !gScopeAlignmentInProgress && ((currentTime - gLastPositionUpdate) > kSecondsBetweenUpdates))
	{	
		gLastPositionUpdate = currentTime;
		
		if (!gInitialSetupComplete)
		{		
			double currentScopeRightAscension	= coordinate_NotAvailable;
			double currentScopeDeclination		= coordinate_NotAvailable;
			
			if (gAlignedInSoftware)
				error = DoGetAlignedRaDec (&currentScopeRightAscension, &currentScopeDeclination );
			else
				error = ::GetTelescopeRaDec(&currentScopeRightAscension, &currentScopeDeclination, true);
					
			if (error == snNoErr) {
				// sn_CentreWindowAtCoords callback method
				PXYZCoordinates scopeCoordsXYZ;
				PSphericalCoordinates raDecCoordsSpherical;
				raDecCoordsSpherical.ra = currentScopeRightAscension;
				raDecCoordsSpherical.dec = currentScopeDeclination;
				raDecCoordsSpherical.radius = 1.0;
			
				if ((currentScopeRightAscension != coordinate_NotAvailable) && (currentScopeDeclination != coordinate_NotAvailable))
				{
					error = sn_SphericalToXYZ(raDecCoordsSpherical, &scopeCoordsXYZ);
					//error = sn_CentreWindowAtCoords(kFrontWindow, kSNCelestialJNowSystem, scopeCoordsXYZ, true, false, 100.0, false);
				}

				gInitialSetupComplete = true;
			
				if (gSelectedScopeType == scope_ORIONSKYQUEST)
					::SetStatusText(&kMessage_Reporting, &kInformative_Reporting, gCurrentScopeRightAscension, gCurrentScopeDeclination);
				else
					::SetStatusText(&kMessage_Tracking, &kInformative_Tracking, gCurrentScopeRightAscension, gCurrentScopeDeclination);
				
				return;
			}
			else {
				if (error == pluginErr_CommunicationError)
					::SetStatusText(&kMessage_CommunicationError, &kInformative_CommunicationError);
				else
					::SetStatusText(&kMessage_GenericError, &kInformative_GenericError);
					
				gCurrentScopeRightAscension = coordinate_NotAvailable;
				gCurrentScopeDeclination = coordinate_NotAvailable;
			}
		}
		else
		{
			if (gAlignedInSoftware)
				error = DoGetAlignedRaDec (&gCurrentScopeRightAscension, &gCurrentScopeDeclination );
			else
				error = ::GetTelescopeRaDec(&gCurrentScopeRightAscension, &gCurrentScopeDeclination, true);

			if (error == snNoErr)
			{
				if (gIsTracking)
				{
					ioEvent->fOutOperation			= out_CentrePoint;
					gCentreAction.fUseCustomPan		= true;
					gCentreAction.fPan				= true;
					gCentreAction.fRightAscension	= gCurrentScopeRightAscension;
					gCentreAction.fDeclination		= gCurrentScopeDeclination;	
					gCentreAction.fAdjustFOV		= false;
					
					ioEvent->fOutParams				= &gCentreAction;
		
					::SetStatusText(&kMessage_Following, &kInformative_Following, gCurrentScopeRightAscension, gCurrentScopeDeclination);
					return;
				}
				else
				{
					if (gSelectedScopeType == scope_ORIONSKYQUEST)
						::SetStatusText(&kMessage_Reporting, &kInformative_Reporting, gCurrentScopeRightAscension, gCurrentScopeDeclination);
					else
						::SetStatusText(&kMessage_Tracking, &kInformative_Tracking, gCurrentScopeRightAscension, gCurrentScopeDeclination);
					return;
				}
			}
			else
			{
				if (error == pluginErr_CommunicationError)
					::SetStatusText(&kMessage_CommunicationError, &kInformative_CommunicationError);
				else
					::SetStatusText(&kMessage_GenericError, &kInformative_GenericError);
					
				gCurrentScopeRightAscension = coordinate_NotAvailable;
				gCurrentScopeDeclination = coordinate_NotAvailable;
				
				// we should recommend (force?) a close and reopen of the scope?
				// Try to recover from a bad read?
				//ioEvent->fOutOperation = out_DoNothing;
			}
		}
	}
}

//----------------------------------------------------------------------
//	DoDrawIndicator
//
//	Purpose:	Since we didn't pass back fLayer as Depth_None, we get called everytime
//				Starry Night's view is updated. We then pass back in PIndicator
//				a description of a field of view indicator we would like to have
//				drawn. If you don't want an indicator drawn at the current scope
//				location then pass back fLayer as Depth_None in DoInitialize.
//					
//	Inputs:		outIndicator	description of indicator
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//----------------------------------------------------------------------
void DoDrawIndicator(PIndicator* outIndicator)
{
	if ((gCurrentScopeRightAscension == coordinate_NotAvailable) || (gCurrentScopeDeclination == coordinate_NotAvailable))
		return;

	if (gIsConfigurationComplete && gConnectionOpened)
	{
		outIndicator->fDrawIndicator			= true;						// true, we want an indicator drawn
		outIndicator->fIndicatorStyle			= Indicator_RigelQuickfinder;	// type of indicator
		outIndicator->fCoordSystemOfIndicator	= kSNCelestialJNowSystem;
		
		::GetTelescopeRaDec(&(outIndicator->fCentreRA), &(outIndicator->fCentreDec), false);
		
		outIndicator->fVertical 	= 0.0524;				// A reasonable number
		outIndicator->fHorizontal 	= 0.0;					// not used for circular indicators
		outIndicator->fRollAngle 	= 0.0;					// not used for circular indicator
		
		// Colour
		outIndicator->fRed			= 65535;
		outIndicator->fGreen		= 65535;
		outIndicator->fBlue			= 65535;
		
		if (gScope)
			::PStringAssign(outIndicator->fString, gScope->GetScopePrettyNamePtr());

		outIndicator->fDoneDrawing = true;					// this is our only indicator
	}
				
}

//----------------------------------------------------------------------
//	DoBuildContextualPopup
//
//	Purpose:	If the user right-clicks on an object then Starry Night builds a
//				contextual popup menu. You can choose to add items to this menu if
//				you wish. Our plug-in adds two items: Slew and Synch.
//					
//	Inputs:		inBuildContext	record describing object for which we are building a
//								menu, as well as information about the menu itself.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	05/09/15	MK			5.7			Slew to and Synch on now show the name of the object
//----------------------------------------------------------------------

// Win32 needs these for the contextual menu.
const unsigned long	Command_Slew	= do_SlewTelescope;
const unsigned long	Command_Synch	= do_SynchTelescope;

void DoBuildContextualPopup(const PBuildContext* inBuildContext)
{
	// on the Mac we are passed a MenuHandle
#if TARGET_OS_MAC
	PSelection o = inBuildContext->fObject;
	LStr255 objName;
	LStr255 slewInstruction;
	LStr255 syncInstruction;
	short menuitemposition = 0;
	
	//If we are in constellation selector then don't slew
	if (LStr255(o.fObjectName) == LStr255(o.fConstellationName))
		return;
		
	if (gScopeCanSlew || gScopeCanSynch)
	{
		if (o.fObjectName[0] != 0)
			objName.Assign(o.fUserFormattedName);
		else
			objName.Assign("this location");
		// add the seperator
		::InsertMenuItem(inBuildContext->fMenuHandle, "\p-(", 0);
	}

	if(gScopeCanSlew)
	{
		slewInstruction.Assign("Slew to ");
		slewInstruction += objName;
		::InsertMenuItem(inBuildContext->fMenuHandle, slewInstruction, menuitemposition);
		menuitemposition++; 
//		long synchMenuItem = ::CountMenuItems(inBuildContext->fMenuHandle);
		::SetMenuItemCommandID(inBuildContext->fMenuHandle, menuitemposition, do_SlewTelescope);
	}
	
	if(gScopeCanSynch && (o.fObjectName[0] != 0))
	{
		syncInstruction.Assign("Sync on ");
		syncInstruction += objName;
		::InsertMenuItem(inBuildContext->fMenuHandle, syncInstruction, menuitemposition); // better to set text with SetMenuItemText
		menuitemposition++;
//		long synchMenuItem = ::CountMenuItems(inBuildContext->fMenuHandle);
		::SetMenuItemCommandID(inBuildContext->fMenuHandle, menuitemposition, do_SynchTelescope);
	}
#endif /*TARGET_OS_MAC*/

	// For the PC we are passed an HMENU, 
#if TARGET_OS_WIN32
	char	itemText[255];
	// Add divider line to divide our telescope controller items from the others
	unsigned long flags = MF_SEPARATOR;

	if (gScopeCanSynch || gScopeCanSlew)
		bool success = ::AppendMenu(inBuildContext->fMenuHandle, flags, 0, "-");

	if (gScopeCanSlew)
	{
		// Add Slew item using a command of Command_Slew
		flags = 0;
		strcpy(itemText, "Slew Telescope");
		success = ::AppendMenu(inBuildContext->fMenuHandle, flags, Command_Slew, itemText);
	}

	if(gScopeCanSynch)
	{
		// Add Synch item using a command of Command_Synch only if the scope has synch capabilities
		strcpy(itemText, "Synch Telescope");
		success = ::AppendMenu(inBuildContext->fMenuHandle, flags, Command_Synch, itemText);
	}

#endif /*TARGET_OS_WIN32*/
}

//----------------------------------------------------------------------
//	DoContextualPopup
//
//	Purpose:	We get called whenever the user selects an item from the contextual
//				popup menu. We check whether it is one of the items that we added and
//				handle it appropriately.
//					
//	Inputs:		inContext	record describing object for which we are building a
//							menu, as well as information about the menu item that
//							was selected.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
////----------------------------------------------------------------------
void DoContextualPopup(const PDoContext* inContext)
{
	unsigned long command = 0;
	command = inContext->fMenuCommand;
	
	PGaze target;
	target.fGazeRa = inContext->fObject.fRightAscension;
	target.fGazeDec = inContext->fObject.fDeclination;
	
	target.fGazeAltitude = inContext->fObject.fAlt;
	target.fGazeAzimuth = inContext->fObject.fAz;
	
	pBoolean allowSlew = true;
	snErr error = sn_GetAreCoordinatesWithinUserSlewLimits(target, true, &allowSlew);

	if (allowSlew && (error == snNoErr))
	{
		// Look for one of the menu commands that we added
		if (command == Command_Slew)
			SlewTelescope(inContext->fObject);
		else if ((command == Command_Synch) && gScopeCanSynch)
			SynchTelescope(inContext->fObject);	
	}

}

//----------------------------------------------------------------------
//	SlewTelescope
//
//	Purpose:	Slew the telescope to the passed gaze vector whenever you receive
//				op_TelescopeSlew. When the user clicks "Slew" in
//				the telescope panel they want the attached telescope to slew to the
//				same spot as the centre of the Starry Night screen. This is the gaze
//				vector that is passed with op_TelescopeSlew.
//					
//	Inputs:		slewGaze	pointer to a SN PGaze
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//----------------------------------------------------------------------
void SlewTelescope(PGaze* slewGaze)
{
	snErr error = pluginErr_CommunicationError;
	error = gScope->SlewScope(slewGaze);
	
	if (error != snNoErr)
		::HandleError(error);
}


//----------------------------------------------------------------------
//	SlewTelescope
//
//	Purpose:	Slew the telescope to the passed object. Overloaded to support the 
//				PSelection type in DoContextualPopup.
//					
//	Inputs:		inObject	pointer to the object to slew to (contains coords)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//----------------------------------------------------------------------
void SlewTelescope(const PSelection inObject)
{
	snErr error;	
	error = gScope->SlewScope(inObject);
	
	if (error != snNoErr)
		::HandleError(error);
}

//----------------------------------------------------------------------
//	SlewTelescope
//
//	Purpose:	Overloaded to allow control of telescope motion. (i.e simulate
//				handbox controls up/down/left/right also stop.
//					
//	Inputs:		inMotionParam	pointer to a PScopeMotion parameter containing the direction and slew rate.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/09/01	DWW			5.0.1		Added for advanced telescope control in Mac
//----------------------------------------------------------------------
void SlewTelescope(const PScopeMotion* inMotionParam)
{
	snErr error;	
	error = gScope->SlewScope(inMotionParam);
	
	if (error != snNoErr)
		::HandleError(error);
}

//----------------------------------------------------------------------
//	SynchTelescope
//
//	Purpose:	Synch the telescope to the passed gaze vector whenever you receive
//				op_TelescopeSync. When the user selects "Sync to Current Gaze" from
//				the telescope menu they want the attached telescope to synch to the
//				same spot as the centre of the Starry Night screen. This is the gaze
//				vector that is passed with op_TelescopeSync.
//					
//				For example, the user may centre on Altair, locate it in their scope and
//				then select "Sync to Current Gaze". It is our responsibility to inform
//				our telescope where it is currently pointed. This is referred to as
//				synch-ing the telescope.
//					
//	Inputs:		synchGaze	pointer to a SN PGaze
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//----------------------------------------------------------------------
void SynchTelescope(PGaze* synchGaze)
{
	snErr error = pluginErr_CommunicationError;

	if(gScopeCanSynch)
	{
		error = gScope->SynchScope(synchGaze);

		::GetTelescopeRaDec(&gCurrentScopeRightAscension, 
							&gCurrentScopeDeclination, 
							true);
								
	if (error != snNoErr)
		::HandleError(error);
	}
}

//----------------------------------------------------------------------
//	SynchTelescope
//
//	Purpose:	Synch the telescope to the passed object. Overloaded to support the 
//				PSelection* type in DoContextualPopup.
//					
//	Inputs:		inObject	pointer to the object to synch on (contains coords)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	-------------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------
void SynchTelescope(const PSelection inObject)
{
	snErr error = pluginErr_CommunicationError;

	if(gScopeCanSynch)
	{
		error = gScope->SynchScope(inObject);

		::GetTelescopeRaDec(&gCurrentScopeRightAscension, 
							&gCurrentScopeDeclination, 
							true);
		
		if (error != snNoErr)
			::HandleError(error);
	}
}


