//----------------------------------------------------------------------
//	Copyright © 1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		CelestronScope.cpp
//
//	Contains:	Class for controlling Celestron type telescopes
//				
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/05	MK			5.02		GetValidResponse now sends the last character expected
//										to serial communication methods in order to speed up communication
//  04/06/15	MK			5.0			Bugfixes and optimization
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//
//----------------------------------------------------------------------

#include "Scope.h"

// These are the default celestron telescope port communication settings
GSSerialPortInfoRecord 	portInfo_CelestronDefault =	{	
														9600,				/*BAUD RATE*/
														8, 					/*DATA BITS*/
														kGSSerialParityNone,/*PARITY*/ 
														kGSSerialStopBits10	/*STOP BITS*/
													};
												
#pragma mark ---TCelestronScope---
//----------------------------------------------------------------------
//	TCelestronScope::TCelestronScope()
//
//	Purpose:	Default TCelestronScope constructor. Setup defaults.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//
//----------------------------------------------------------------------
TCelestronScope::TCelestronScope(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType) : 	TScope(inSerialCommunicator, inScopeType),
																										fUses32BitCoords(false)
{
	fCanSynch				= false;
	fCanSlew				= true;
	fCanSetScopeTime		= false;
	fCanSetScopeLocation 	= false;
	fCanDirSlew				= false;
	fCanSetSlewRate			= false;
	fCanSetSlewDirRate		= false;
	fScopeWaitTime			= kNS11U2KWaitTime;

	fCanPark				= false;					
	fCanUnpark				= false;
	fCanSetPark				= false;
	fCanGoHome				= false;	

	switch (inScopeType) {
		case scope_NEXSTAR5i8i:
			fCanDirSlew = true;
			fCanSetSlewDirRate = true;
			fScopePrettyName.Assign("NexStar 5i/8i");
			break;
		case scope_NEXSTAR811GPS:
			fCanSynch = true;
			fCanDirSlew = true;
			fCanSetSlewDirRate = true;
			fScopePrettyName.Assign("NexStar 8/11 GPS");
			break;
		case scope_CELESTCGE:
			fScopePrettyName.Assign("Celestron CGE");
			break;
		case scope_CELESTASC:
			fScopePrettyName.Assign("Celestron ASC");
			break;
		case scope_NEXSTAR6080114:
			fScopePrettyName.Assign("NexStar 60/80/114");
			break;
		case scope_NEXSTAR4GT:
			fScopePrettyName.Assign("NexStar 4 GT");
			break;
/*		case scope_U2K:
			fScopePrettyName.Assign("Ultima 2000");
			break;
		case scope_U2KCOMPATIBLE:
			fScopePrettyName.Assign("Ultima 2000 Compatible");
			break;
*/
		default:
			break;
	}
	
	if (inSerialCommunicator)
	{
		// First get the name of the port that is in use
		GSXSerialPortNameRecord		nameOfPortInUse; // This we need in the next step to set the values
		fScopeSerialCommunicator->GetPortOptions(NULL, &nameOfPortInUse);
		
		// Now set the port info for the scope in question
		fScopeSerialCommunicator->SetPortOptions(&portInfo_CelestronDefault, &nameOfPortInUse);
	}
	else
	{
		// We passed in a NULL pointer as the inSerialCommunicator since we only want to check if the 
		// scope type can set time and location.
	}
}

//----------------------------------------------------------------------
//	TCelestronScope::~TCelestronScope()
//
//	Purpose:	Destruct
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//
//----------------------------------------------------------------------

TCelestronScope::~TCelestronScope()	
{
}

//----------------------------------------------------------------------
//	TCelestronScope::CheckAlignmentComplete
//
//	Purpose:	Checks to see if the scope is connected and aligned
//									
//	Returns:	true if the scope is connected and communicating proplerly
//				false if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0		changed response checking, removed delay
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//----------------------------------------------------------------------

pBoolean TCelestronScope::CheckAlignmentComplete(void)
{
	LStr255 alignCompleteString('J');

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo(alignCompleteString);
	
	/* TScope::SleepDelay(kNS11U2KWaitTime); */
	
	LStr255 scopeResponse;
	scopeResponse.Assign("\p");

/* - no echoing was observed in June, 2004 so this is unnecessary
	UInt32  scopeResponseLength = 3;
	this->GetScopeResponse(scopeResponse, scopeResponseLength, kNS11U2KWaitTime);

	// Subtract one since we want the second to last character (i.e. not the terminating '#')
	// why does this echo the J command?
	UInt32 scopeResponseIndex = scopeResponse.Length()-1;
*/
	
	//valid format here is 0# or 1# - but # doesn't count
	if (this->GetValidResponse(scopeResponse,1))
		if (scopeResponse[1] == 1)
			return true;
		
	return false;
}

//----------------------------------------------------------------------
//	TCelestronScope::OpenScopeConnection
//
//	Purpose:	"Opens" the connection to the scope by making sure there is a scope connected
//				to the fScopeSerialCommunicator
//									
//	Returns:	snNoErr if the scope is connected and communicating proplerly
//				pluginErr_CommunicationError if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TCelestronScope::OpenScopeConnection(void)
{
	snErr error = pluginErr_CommunicationError;
	
	if (this->CheckAlignmentComplete())
	{
		fIsConnected = true;
		fUses32BitCoords = this->GetUses32BitCoordinates();
		error = snNoErr;
	}
	else
		fIsConnected = false;
	
	return error;
}

//----------------------------------------------------------------------
//	TCelestronScope::SynchScope
//
//	Purpose:	Synch the telescope to the passed Alt and Az, that is,
//				tells the scope where it is pointing. Useful for alignment
//									
//	Inputs:		inAz	The azimuth in decimal radians to synch on
//				inAlt	The altitude in decimal radians to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0		Unnecessary delay
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TCelestronScope::SynchScope(double inAzimuth, double inAltitude)
{
	snErr error = snNoErr;

	if (fCanSynch)
	{
		if (this->IsReadyForCommand())
		{
			fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
			/* TScope::SleepDelay(kNS11U2KWaitTime); */
		
			LStr255 synchStringToSend;
			synchStringToSend.Assign("\p");
			this->MakeAltAzSynchString(inAzimuth, inAltitude, synchStringToSend);
			fScopeSerialCommunicator->SendSerialInfo(synchStringToSend);
			
			error = snNoErr;
		}
		else
			error = pluginErr_CommunicationError;
	}
	else 
		error = pluginErr_FunctionNotSupported;

	return error;
}

/* - slewing to Alt/Az is just not needed here - Ra/Dec works fine

//----------------------------------------------------------------------
//	TCelestronScope::SlewScope
//
//	Purpose:	Slew the telescope to the passed Alt and Az.
//									
//	Inputs:		slewGaze	gaze struct containing the Alt and Az
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TCelestronScope::SlewScope(PGaze* slewGaze)
{
	double inAzimuth 	= slewGaze->fGazeAzimuth;
	double inAltitude	= slewGaze->fGazeAltitude;

	return this->SlewScope(inAltitude, inAzimuth);
}

//----------------------------------------------------------------------
//	TCelestronScope::SlewScope
//
//	Purpose:	Slew the telescope to the passed Alt and Az.
//									
//	Inputs:		inObject	The object to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TCelestronScope::SlewScope(const PSelection inObject)
{
	double inAzimuth	= inObject.fAz;
	double inAltitude	= inObject.fAlt;
		
	return this->SlewScope(inAltitude, inAzimuth);
}
*/


//----------------------------------------------------------------------
//	TCelestronScope::SlewScope
//
//	Purpose:	Slew the telescope to the passed RA and Dec.
//									
//	Inputs:		inRA	The right ascension in decimal radians to synch on
//				inDec	The declination in decimal radians to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0		Optimized and removed delay
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TCelestronScope::SlewScope(double inRA, double inDec)
{

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	// TScope::SleepDelay(kNS11U2KWaitTime);
		
	LStr255 slewStringToSend;
	slewStringToSend.Assign("\p");
	this->MakeRaDecSlewString(inRA, inDec, slewStringToSend);
	fScopeSerialCommunicator->SendSerialInfo(slewStringToSend);
	
	LStr255 response;
	// GetValidResponse does not count # which is what we are waiting for
	if (this->GetValidResponse(response,0))
		return snNoErr;
	else
		return pluginErr_CommunicationError;

}

//----------------------------------------------------------------------
//	TCelestronScope::SlewScope
//
//	Purpose:	Slew the telescope in the passed Direction
//									
//	Inputs:		dir		Direction in which to slew
//				speed   Rate of slewing
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/07/13	MK			5.0			Finalized
//	04/09/01	DWW			5.0.1		Updated for advanced telescope control in Mac
//----------------------------------------------------------------------
snErr TCelestronScope::SlewScope(const PScopeMotion* inMotionParam)
{
	
	LStr255 inString,outString;
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	
	ScopeSlewRateT theSlewRate = inMotionParam->fRelativeSlewRate;
	ScopeDirectionT theDirection = inMotionParam->fMotionDirection;
	
	//calculate appropriate rate for celestron in the range 0-9
	// 2 - FIRST
	// 4 - SECOND
	// 7 - THIRD
	// 9 - FOURTH
	int celestronSlewRate = 8; // a reasonable default
	
	switch (theSlewRate)
	{
		case rate_1:
			celestronSlewRate = 5;
			break;
		case rate_2:
			celestronSlewRate = 6;
			break;
		case rate_3:
			celestronSlewRate = 8;
			break;
		case rate_4:
			celestronSlewRate = 9;
			break;
		default:
			celestronSlewRate = 8; // default to rate_3
			break;
	}
	
	//char(80) is a signal to the handbox to send the rest of data directly to the scope (no processing except for checksum)
	//char(3) signifies that data is going to the scope
	//char(17) is for altitude encoder, char(18) for az
	//char(36) is for right/down, char(37) for left/up
	switch (theDirection)
	{
		case direction_UP:
			outString=LStr255(char(80))+LStr255(char(3))+LStr255(char(17))+LStr255(char(36))+LStr255(char(celestronSlewRate))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0));
			break;
		case direction_DOWN:
			outString=LStr255(char(80))+LStr255(char(3))+LStr255(char(17))+LStr255(char(37))+LStr255(char(celestronSlewRate))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0));
			break;
		case direction_LEFT:
			outString=LStr255(char(80))+LStr255(char(3))+LStr255(char(16))+LStr255(char(37))+LStr255(char(celestronSlewRate))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0));
			break;
		case direction_RIGHT:
			outString=LStr255(char(80))+LStr255(char(3))+LStr255(char(16))+LStr255(char(36))+LStr255(char(celestronSlewRate))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0));
			break;
		case direction_NONE_STOP:
			//To stop slewing set slewing speed to 0 for both az and alt encoders
			outString=LStr255(char(80))+LStr255(char(3))+LStr255(char(17))+LStr255(char(36))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0));
			fScopeSerialCommunicator->SendSerialInfo(outString);
			
			//make sure that # is returned
			if (!this->GetValidResponse(inString,0))
				return pluginErr_CommunicationError;
			outString=LStr255(char(80))+LStr255(char(3))+LStr255(char(16))+LStr255(char(36))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0))+LStr255(char(0));
		default:
			break;
	}
	fScopeSerialCommunicator->SendSerialInfo(outString);
	if (!this->GetValidResponse(inString,0))
		return pluginErr_CommunicationError;
	return snNoErr;
}

//----------------------------------------------------------------------
//	TCelestronScope::GetScopeRaDec()
//
//	Purpose:	Determine the RA and Dec that the scope is currently pointing at.
//				
//	Inputs:		outRA		Ra of the scope
//				outDec		Dec of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TCelestronScope::GetScopeRaDec(double& outRA, double& outDec)
{
	LStr255 raDecStringToParse;
	raDecStringToParse.Assign("");

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	
	if (fUses32BitCoords) {
		fScopeSerialCommunicator->SendSerialInfo("e",1);
		//valid response here is in the format: 34AB0500,12CE0500 and #
		if (!this->GetValidResponse(raDecStringToParse,17))
			return pluginErr_CommunicationError;
		}
	else {
		fScopeSerialCommunicator->SendSerialInfo("E",1);
		//valid response here is in the format: 34AB,12CE and #
		if (!this->GetValidResponse(raDecStringToParse,9))
			return pluginErr_CommunicationError;
		}
		
	this->MakeRadianCoordsFromHexString(raDecStringToParse, outRA, outDec);
	return snNoErr;
}

//----------------------------------------------------------------------
//	TCelestronScope::GetScopeAzAlt()
//
//	Purpose:	Provides a method alternative to RA/Dec for getting scope position.
//				
//	Inputs:		outAz	Az of the scope
//				outAlt	Alt of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  05/03/09	MK			5.02		currently not supported
//----------------------------------------------------------------------
snErr TCelestronScope::GetScopeAzAlt(double& outAz, double& outAlt)
{
	return pluginErr_FunctionNotSupported;
}

#pragma mark ---NexStarHelperFunctions---
//----------------------------------------------------------------------
//	TCelestronScope::IsReadyForCommand()
//
//	Purpose:	Determine if the scope is ready to receive a command. 
//				Returns true if the scope is ready. Uses the "Echo" command.			
//									
//	Inputs:		inScope 	a pointer to the scope to initialize
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0		not used in this class except for SynchScope
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
pBoolean TCelestronScope::IsReadyForCommand(void)
{
	pBoolean isReady = false;
	LStr255 scopeResponse("");
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);

	const char echoChar = 'x';
	LStr255 echoString('K');
	echoString.Append(echoChar);
	fScopeSerialCommunicator->SendSerialInfo(echoString);

	// x# should be returned but # doesn't count
	this->GetValidResponse(scopeResponse, 1);
	
	if (scopeResponse[1] == echoChar)
		isReady = true;
	else
		isReady = false;
			
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);		
		
	return isReady;
}


//----------------------------------------------------------------------
//	TCelestronScope::GetValidResponse()
//
//	Purpose:	Get response from the scope and detemine if it is valid. 
//				Returns true if the response is OK.
//									
//	Inputs:     outString - returns the string obtained from the buffer
//				numberOfCharsToRead - number of chars before # is expected
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/05	MK			5.02		GetValidResponse now sends the last character expected
//										to serial communication methods in order to speed up communication
//	04/06/15	MK			5.0			Introduced in order to replace
//										IsReadyForCommand mechanism
//----------------------------------------------------------------------
pBoolean TCelestronScope::GetValidResponse(LStr255& outString, UInt32 numberOfCharsToRead)
{
	//Celestron's responses are terminated with '#' so read until time expires or # is returned
	this->GetScopeResponse(outString, 255, fScopeWaitTime, '#');
	
	if ((outString.Length() > numberOfCharsToRead) && (outString(outString.Length(),1) == LStr255('#')))
		{
			//keep only the number of expected characters plus the termination character (gets rid of potential echo)
			outString.Assign(outString, outString.Length()-numberOfCharsToRead, numberOfCharsToRead+1);
			return true;
		}
	else
		return false;
}

//----------------------------------------------------------------------
//	TCelestronScope::GetUses32BitCoordinates()
//
//	Purpose:	Determine if the scope can receive/transmit coords as 32 bit			
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0			Removed delays + optimized
//	03/05/05	DWW			4.5.0		New for 4.5
//
//----------------------------------------------------------------------
pBoolean TCelestronScope::GetUses32BitCoordinates(void)
{
	pBoolean uses32Bit = false;

	switch (fScopeType) {
		//case scope_U2K:
		//case scope_U2KCOMPATIBLE:
		case scope_NEXSTAR4GT:
		case scope_NEXSTAR6080114:
			uses32Bit = false;
			break;
		case scope_NEXSTAR5i8i:
		case scope_NEXSTAR811GPS:
		case scope_CELESTCGE:
		case scope_CELESTASC:
			{
				LStr255 scopeResponse("");
				
				fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
			
				LStr255 getVersionCommand('V');
				fScopeSerialCommunicator->SendSerialInfo(getVersionCommand);

				this->GetValidResponse(scopeResponse,2);
				
				UInt8 versionCharOne = scopeResponse[1];
				UInt8 versionCharTwo = scopeResponse[2];
				
				if ((versionCharOne > 9) || (versionCharTwo > 9))
					return false; // means returned garbage

				// If firmware version is above 1.6 then scope uses 32bit coordinates
				if (versionCharOne >= 1)
				{
					if (versionCharOne > 1) 
						uses32Bit = true;
					else
					{
						if (versionCharTwo >= 6)
							uses32Bit = true;
					}
				}
			}
			break;
		default:
			PluginAssert(false); // I should have a case here for each type
			break;
	}
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);		
		
	return uses32Bit;
}


//----------------------------------------------------------------------
//	TCelestronScope::MakeAltAzSynchString()
//
//	Purpose:	Prepare a string to be sent to the scope to synch to the 
//				desired coords.	NOTE that this IS NOT IMPLEMENTED for the NexStar 5/8 scopes			
//									
//	Inputs:		inAz is the Azimuth in decimal radians
//				inAlt is the Altitude in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::MakeAltAzSynchString(double inAz, double inAlt, LStr255& outString)
{
	PluginAssert(&outString);
	
	LStr255 azString;
	azString.Assign("");
	LStr255 altString;
	altString.Assign("");

	if (fUses32BitCoords)
	{
		this->Make32BitHexStringFromRadianCoord(inAz, azString);
		this->Make32BitHexStringFromRadianCoord(inAlt, altString);
	}
	else
	{
		this->Make16BitHexStringFromRadianCoord(inAz, azString);
		this->Make16BitHexStringFromRadianCoord(inAlt, altString);
	}

	// Note. Order is important. Aziumuth THEN altitude.
	outString.Assign('S');
	outString.Append(azString);
	outString.Append(',');
	outString.Append(altString);
}


//----------------------------------------------------------------------
//	TCelestronScope::MakeRaDecSlewString()
//
//	Purpose:	Prepare a string to be sent to the scope to slew to the 
//				desired coords.				
//									
//	Inputs:		inRa is the Right Ascension in decimal radians
//				inDec is the Declination in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::MakeRaDecSlewString(double inRa, double inDec, LStr255& outString)
{
	PluginAssert(&outString);
	
	LStr255 raString;
	raString.Assign("");
	LStr255 decString;
	decString.Assign("");

	if (fUses32BitCoords)
	{
		outString.Assign('r');
		this->Make32BitHexStringFromRadianCoord(inRa, raString);
		this->Make32BitHexStringFromRadianCoord(inDec, decString);
	}
	else
	{
		outString.Assign('R');
		this->Make16BitHexStringFromRadianCoord(inRa, raString);
		this->Make16BitHexStringFromRadianCoord(inDec, decString);
	}

	outString.Append(raString);
	outString.Append(',');
	outString.Append(decString);	
}


//----------------------------------------------------------------------
//	TCelestronScope::Make16BitHexStringFromRadianCoord()
//
//	Purpose:	Convert input coordinate into a 16 bit hex number as a string.
//									
//	Inputs:		outString is the coordinate string returned form the scope
//				inCoord is the decimal radian coordinate to be converted to 16 bit hex
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::Make16BitHexStringFromRadianCoord(double inCoord, LStr255& outString)
{
	PluginAssert(&outString);

	if (inCoord < 0.0)
		inCoord = inCoord + kTwicePi;
	else
		inCoord = TScope::ModPi2(inCoord);

	double coordAs16BitDouble = (inCoord/(kTwicePi) * kCelestron16BitResolution);
	coordAs16BitDouble += 0.5; // add 0.5 to round properly
	UInt16 coordAs16Bit = (UInt16)(coordAs16BitDouble);
	
	char tempCString[4];
	sprintf(tempCString, "%04X", coordAs16Bit);
	
	outString.Assign(tempCString);
	
}


//----------------------------------------------------------------------
//	TCelestronScope::Make32BitHexStringFromRadianCoord()
//
//	Purpose:	Convert input coordinate into a 32 bit hex number as a string.
//									
//	Inputs:		outString is the coordinate string returned form the scope
//				inCoord is the decimal radian coordinate to be converted to 16 bit hex
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::Make32BitHexStringFromRadianCoord(double inCoord, LStr255& outString)
{
	PluginAssert(&outString);

	if (inCoord < 0.0)
		inCoord = inCoord + kTwicePi;
	else
		inCoord = TScope::ModPi2(inCoord);

	double coordAs32BitDouble = (inCoord/(kTwicePi) * kCelestron32BitResolution);
	coordAs32BitDouble += 0.5; // add 0.5 to round properly
	UInt32 coordAs32Bit = (UInt32)(coordAs32BitDouble);
	
	char tempCString[6];
	sprintf(tempCString, "%06X", coordAs32Bit);

	outString.Assign(tempCString);
	// last two characters will always be zeros
	outString.Append("00");
	
//	PluginAssert(outString.Length() == 8);
}


//----------------------------------------------------------------------
//	TCelestronScope::MakeRadianCoordsFromHexString()
//
//	Purpose:	Convert the coordinate string sent from  scope to
//				decimal radian coordinates (can be used for RA Dec or Alt Az.)
//									
//	Inputs:		inString is the coordinate string returned form the scope
//				outCoordOne is the first coord (RA or Azimuth)
//				outCoordTwo  is the the second coord (Dec or Alt)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0			Cosmetic fixes
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::MakeRadianCoordsFromHexString(LStr255& inString, 
													double& outCoordOne /*RA or Az*/, 
													double& outCoordTwo /*Dec or Alt*/)
{
	PluginAssert(&inString);
	PluginAssert((inString.Length() == 18) || (inString.Length() == 10));
	
	LStr255 coordOneString;
	LStr255 coordTwoString;
	
	if (fUses32BitCoords)
	{
		
		coordOneString.Assign(inString, 1, 8);
		coordTwoString.Assign(inString, 10, 8);
		
		this->Convert32BitHexStringToRadianDouble(coordOneString, outCoordOne);
		this->Convert32BitHexStringToRadianDouble(coordTwoString, outCoordTwo);
	}
	else
	{
		coordOneString.Assign(inString, 1, 4);
		coordTwoString.Assign(inString, 6, 4);
		
		this->Convert16BitHexStringToRadianDouble(coordOneString, outCoordOne);
		this->Convert16BitHexStringToRadianDouble(coordTwoString, outCoordTwo);
	}
}

//----------------------------------------------------------------------
//	TCelestronScope::Convert16BitHexStringToRadianDouble()
//
//	Purpose:	Convert a hexadecimal string sent from scope to
//				decimal radian coordinates 
//									
//	Inputs:		inString is the coordinate string (as a 16 bit hex munber) to be  converted 
//				outCoord is the coord in decimal decimal radian.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0			Modified function to use strtod, not homegrown function
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::Convert16BitHexStringToRadianDouble(LStr255& inString, double& outCoord)
{
	PluginAssert(inString.Length() == 4);
	
	char* remainder;
	inString.Insert(LStr255("0X"),1);
	
	//convert to double from hex
	outCoord=strtod(LStr255::PToCStr(inString),&remainder);
	
	//invalid data received if there is a remainder but there is no need to halt everything
	//PluginAssert(*remainder == '\0');
	
	outCoord = (outCoord/kCelestron16BitResolution)*kTwicePi;

	//this must never happen
	PluginAssert(outCoord <= kTwicePi);
}

//----------------------------------------------------------------------
//	TCelestronScope::Convert32BitHexStringToRadianDouble()
//
//	Purpose:	Convert a hexadecimal string sent from scope to
//				decimal radian coordinates 
//									
//	Inputs:		inString is the coordinate string (as a 32 bit hex munber) to be  converted 
//				outCoord is the coord in decimal decimal radian.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0			Modified function to use strtod, not homegrown function
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::Convert32BitHexStringToRadianDouble(LStr255 inString, double& outCoord)
{
	PluginAssert(inString.Length() == 8);
	
	LStr255 toconvert;
	toconvert.Assign("0X");
	
	//we are not interested in the last two characters since they are always 0
	// this is really 24bit resolution
	toconvert += inString(1,6);
		
	char* remainder;
	
	outCoord=strtod(LStr255::PToCStr(toconvert),&remainder);
	
	//invalid data received if there is a remainder but there is no need to halt everything
	//PluginAssert(*remainder == '\0');

	// No need to convert to the software resolution since we are only interested in the fraction of the circle
	outCoord = (outCoord/kCelestron32BitResolution)*kTwicePi;

	//this must never happen
	PluginAssert(outCoord <= kTwicePi);
	
}

//----------------------------------------------------------------------
//	TCelestronScope::WaitForSlewFinishedCharacterOrTimeoutPassed()
//
//	Purpose:	Returns when either the scope has sent the "slew finished" character
//
//									
//	Inputs:		NONE
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0			Removed a delay
//	01/11/16	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TCelestronScope::WaitForSlewFinishedCharacterOrTimeoutPassed(void)
{
	LStr255 scopeResponseChar("");	
	UInt32 	timeoutInMilliseconds = (UInt32)kNS11U2KSettleTime; // Time after zero distance bars are detected
														// we wait before slew considered complete
	
	for (UInt32 i = 0; (scopeResponseChar[1] == '1'); i++)
	{
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		
		fScopeSerialCommunicator->SendSerialInfo("L",1);
		/* TScope::SleepDelay(k100MilliSeconds); */
		this->GetScopeResponse(scopeResponseChar, 1, timeoutInMilliseconds); 
	}
}

#pragma mark ---NexStar58---
//----------------------------------------------------------------------
//	TNexStar58::TNexStar58()
//
//	Purpose:	Default TNexStar58 constructor. Setup defaults.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/11/20	MK			5.7			Added Orion Atlas EQ-G as supported scope
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TNexStar58::TNexStar58(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType) : TScope(	inSerialCommunicator,
																									inScopeType)
{	
	fCanSynch				= false;	
	fCanSlew				= true;			
	fCanSetScopeTime		= false;
	fCanSetScopeLocation 	= false;
	fCanDirSlew				= false;
	fCanSetSlewRate			= false;
	fCanSetSlewDirRate		= false;
	fScopeWaitTime			= kNS58WaitTime;

	fCanPark				= false;					
	fCanUnpark				= false;
	fCanSetPark				= false;
	fCanGoHome				= false;

	switch(inScopeType)
	{
		case scope_NEXSTAR58:
			fScopePrettyName.Assign("Celestron NexStar 5/8");
			break;
		case scope_ORIONATLAS:
			fScopePrettyName.Assign("Orion Atlas EQ-G");
			break;
		default:
			fScopePrettyName.Assign("UNSUPPORTED Scope");
			break;
	}	
	
	if (fScopeSerialCommunicator)
	{
		// First get the name of the port that is in use
		GSXSerialPortNameRecord	nameOfPortInUse; // This we need in the next step to set the values
		fScopeSerialCommunicator->GetPortOptions(NULL, &nameOfPortInUse);
		
		// Now set the port info for the scope in question
		fScopeSerialCommunicator->SetPortOptions(&portInfo_CelestronDefault, &nameOfPortInUse);
	}
	else
	{
		// We pass in a NULL pointer as the inSerialCommunicator when we want to check if the 
		// scope type can set time and location
	}
	
}

//----------------------------------------------------------------------
//	TNexStar58::~TNexStar58()
//
//	Purpose:	Destruct
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TNexStar58::~TNexStar58()	
{
}

//----------------------------------------------------------------------
//	TNexStar58::OpenScopeConnection
//
//	Purpose:	"Opens" the connection to the scope by making sure there is a scope connected
//				to the fScopeSerialCommunicator
//									
//	Returns:	snNoErr if the scope is connected and communicating proplerly
//				pluginErr_CommunicationError if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TNexStar58::OpenScopeConnection(void)
{
	snErr error = pluginErr_CommunicationError;
	
	double unusedRA = 0.0;
	double unusedDec = 0.0;
	
	error = this->GetScopeRaDec(unusedRA, unusedDec);
	
	if (error == snNoErr)
		fIsConnected = true;
	else
		fIsConnected = false;
		
	return error;
}

//----------------------------------------------------------------------
//	TNexStar58::SlewScope
//
//	Purpose:	Slew the telescope to the passed RA and Dec.
//									
//	Inputs:		inRA	RA in decimal radians
//				inDec	Dec in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/03	MK			5.0		Modified to use RA and Dec
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SlewScope(double inRA, double inDec)
{
	snErr error = pluginErr_CommunicationError;
		
	if (this->NexStar58CommandInitialization())
	{
		error = snNoErr;
		LStr255 slewStringToSend("");
		this->MakeNexStar58SlewString(inRA, inDec, slewStringToSend);
	
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		fScopeSerialCommunicator->SendSerialInfo(slewStringToSend);

		if (!this->WaitForSlewFinishedCharacterOrTimeoutPassed())
			error = pluginErr_CommunicationError;
	}
	
	return error;
}


//----------------------------------------------------------------------
//	TNexStar58::GetScopeRaDec()
//
//	Purpose:	Determine the RA and Dec that the scope is currently pointing at.
//				
//	Inputs:		outRA		Ra of the scope
//				outDec		Dec of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0		optimized and removed delay
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::GetScopeRaDec(double& outRA, double& outDec)
{
	snErr error = pluginErr_CommunicationError;
	
	if (this->NexStar58CommandInitialization())
	{
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		fScopeSerialCommunicator->SendSerialInfo("E",1);
		
		// we seem to need a bit of a rest here...
		//TScope::SleepDelay(k200MilliSeconds);
			
		LStr255 raDecStringToParse("");
		
		//6 chars for GTs, 4 chars for Nexstar 5/8
		if (fSerialCommandEchoed) {
			this->GetScopeResponse(raDecStringToParse, 5, fScopeWaitTime);
			raDecStringToParse.Assign(raDecStringToParse,2,raDecStringToParse.Length()-1);
		}
		else
			this->GetScopeResponse(raDecStringToParse, 4, fScopeWaitTime);
		
		// 2 bytes for RA and 2 bytes for Dec expected
		if (raDecStringToParse.Length() == 4)
		{
			this->MakeRadianCoordsFromNexStar58HighLowByteString(raDecStringToParse, outRA, outDec);
			error = snNoErr;
		}
		else
			error = pluginErr_CommunicationError;
	}
	else
		error = pluginErr_CommunicationError;

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	return error;
}

//----------------------------------------------------------------------
//	TNexStar58::GetScopeAzAlt()
//
//	Purpose:	Provides a method alternative to RA/Dec for getting scope position.
//				
//	Inputs:		outAz	Az of the scope
//				outAlt	Alt of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  05/03/09	MK			5.02		currently not supported
//----------------------------------------------------------------------
snErr TNexStar58::GetScopeAzAlt(double& outAz, double& outAlt)
{
	return pluginErr_FunctionNotSupported;
}


#pragma mark ---NexStar5/8HelperFunctions---
//----------------------------------------------------------------------
//	TNexStar58::NexStar58CommandInitialization()
//
//	Purpose:	Prepare the scope to receive a command. Returns true if the scope is ready. 				
//									
//	Inputs:		inScope 	a pointer to the scope to initialize
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
pBoolean TNexStar58::NexStar58CommandInitialization(void)
{
	pBoolean result = false;

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo("?",1);
	
	LStr255 scopeResponse;
	UInt32 maxScopeResponseLength = 2;
	this->GetScopeResponse(scopeResponse, maxScopeResponseLength, fScopeWaitTime);

	//take note if commands are echoed
	if (scopeResponse[1] == '?')
		fSerialCommandEchoed = true;
	else
		fSerialCommandEchoed = false;
	
	// This takes into account the possibility of any extra characters 
	// being in the buffer when the response is read (ie. echo)
	UInt32 scopeResponseIndex = scopeResponse.Length();
	
	if (scopeResponse[scopeResponseIndex] == '#')
		result = true;
	else
		result = false;

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);		
	return result;
}


//----------------------------------------------------------------------
//	TNexStar58::MakeNexStar58SlewString()
//
//	Purpose:	Prepare a string to be sent to the scope to slew to the 
//				desired coords.				
//									
//	Inputs:		inRA is the RightAscension in decimal radians
//				inDec is the Declination in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar58::MakeNexStar58SlewString(double inRA, double inDec, LStr255& outString)
{
	assert(&outString);
	
	UInt8 raHighByte	= 0;
	UInt8 raLowByte		= 0;
	UInt8 decHighByte	= 0;
	UInt8 decLowByte	= 0;
	
	this->MakeNexStar58HighLowByteStringsFromRadianCoord(inRA, raHighByte, raLowByte);
	this->MakeNexStar58HighLowByteStringsFromRadianCoord(inDec, decHighByte, decLowByte);

	outString.Assign("R");
	outString.Append(raHighByte);
	outString.Append(raLowByte);
	outString.Append(decHighByte);
	outString.Append(decLowByte);
}


//----------------------------------------------------------------------
//	TNexStar58::MakeNexStar58HighLowByteStringsFromRadianCoord()
//
//	Purpose:	Convert the coordinate (either RA, Dec, Alt or Az) to the high and low bytes of a 16 bit
//				number 				
//									
//	Inputs:		inCoord is the coordinate in decimal radians to be parsed
//				outHigh is the the "high" byte of the coordinate as a 16 bit number
//				outLow  is the the "low" byte of the coordinate as a 16 bit number
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0		Modified to use bitwise operations
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

void TNexStar58::MakeNexStar58HighLowByteStringsFromRadianCoord(double inCoord, UInt8& outHigh, UInt8& outLow)
{
	if ((&outHigh == NULL) || (&outLow == NULL))
		assert(false);
	
	// Celestron takes negatives as their complement. e.g. (-29.5 degrees -> 330.5 degrees) (or e.g. -Pi/4 -> 7Pi/4)
	// This will only occur if a negative declination or altitude is passed as inCoord
	if (inCoord < 0.0)
		inCoord = inCoord + kTwicePi;
	
	// Ensure that the input is in the correct range.
	//inCoord = TScope::ModPi2(inCoord);
		
	// The pointing resolution of the NexStar 5 and 8 is 64800 per 2Pi radians so we must round
	// the the nearest 64800/2Pi then scale up to 65536. Add the 0.5 to round to the nearest int properly
	double inCoordAs16Bit = 0.0;
	
	//inCoordAs16Bit = floor((inCoord/kTwicePi * kNS58PointingRes) + 0.5) * 65536.0 / kNS58PointingRes;
	inCoordAs16Bit = inCoord/kTwicePi * 65536.0;

	// Add the 0.5 to round to the nearest int properly.
	outHigh = (UInt8)((UInt16)(inCoordAs16Bit+0.5) >> 8);
	outLow = (UInt8)((UInt16)(inCoordAs16Bit+0.5) &  (UInt16)255);
}


//----------------------------------------------------------------------
//	TNexStar58::MakeRadianCoordsFromNexStar58HighLowByteString()
//
//	Purpose:	Convert the high, low byte strings sent from a Celestron scope to
//				decimal radian coordinates (can be used for RA Dec or Alt Az.)
//				Be sure to check with manual which is which!
//									
//	Inputs:		inString is the coordinate string returned from the scope
//				outCoordOne is the the combo of the first "high" and "low" bytes of inString in decimal radians
//				outCoordTwo  is the the combo of the second "high" and "low" bytes of inString in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0		Modified to use bitwise operations
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar58::MakeRadianCoordsFromNexStar58HighLowByteString(LStr255& inString, 
													double& outCoordOne, /*RA or Az*/
													double& outCoordTwo) /*Dec or Alt*/
{	
	if ((&inString == NULL) || (&outCoordOne == NULL) || (&outCoordTwo == NULL))
		assert(false);
	
	UInt8 highOne = 0;
	UInt8 lowOne  = 0;
	UInt8 highTwo = 0;
	UInt8 lowTwo  = 0;

	UInt32 startPositionIndex = 1;
	
	highOne = inString[startPositionIndex];
	lowOne 	= inString[startPositionIndex +1];
	highTwo	= inString[startPositionIndex +2];
	lowTwo  = inString[startPositionIndex +3];
		
	double tempCoordTwoPosNeg;
//	tempCoordTwoPosNeg = ((highTwoAsDouble*256.0 + lowTwoAsDouble)/65535.0) * kTwicePi;
	tempCoordTwoPosNeg = ((highTwo << 8) + lowTwo)/65535.0 * kTwicePi;
	
	if (tempCoordTwoPosNeg > kHalfPi)
	{
		if (tempCoordTwoPosNeg < kThreeHalfPi)
		{
			assert(tempCoordTwoPosNeg > kThreeHalfPi);
			// We know there is an error since neither Dec nor Alt should come back from celestron
			// in the range (kHalfPi, kThreeHalfPi). Deal with this...?
		}
		else
		{
			// Turn the value into a negative number by subtracting 2 Pi.
			tempCoordTwoPosNeg = tempCoordTwoPosNeg - kTwicePi;			
		}
	}

//	outCoordOne = ((highOneAsDouble*256.0) + lowOneAsDouble)/65535.0 * kTwicePi;	
	outCoordOne = ((highOne << 8) + lowOne)/65535.0 * kTwicePi;
	outCoordTwo = tempCoordTwoPosNeg;
}

//----------------------------------------------------------------------
//	TNexStar58::WaitForSlewFinishedCharacterOrTimeoutPassed()
//
//	Purpose:	Returns when either the scope has sent the "slew finished" character
//				(i.e. '@') or the timeout has passed.
//
//				***NOTE*** 	Sometimes the scope does not return the @ character at all when
//							the slew is complete, hence the timeout.
//									
//	Inputs:		NONE
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/11/16	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
pBoolean TNexStar58::WaitForSlewFinishedCharacterOrTimeoutPassed(void)
{
	pBoolean success = true; // default this to true until celestron fixes their own problems

	LStr255	scopeResponseChar("");
	UInt32	maxNumberOfCharsToGet = 1;
	UInt32	timeoutInMilliseconds = (UInt32)kNS58MaxSlewTime;
	
	for(UInt32 i = 0; (i < maxNumberOfCharsToGet) && (scopeResponseChar[1] != '@'); i++)
	{
		this->GetScopeResponse(scopeResponseChar, 1, timeoutInMilliseconds);
	}
	
	return success;	
}
