//----------------------------------------------------------------------
//	Copyright  1990-2002 by SPACE.com Canada, Inc., All rights reserved.
//
//	File:		LX200.cpp
//
//	Contains:	Class for controlling LX200 compatible and Autostar type telescopes
//				
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/11/11	DWW			4.0.1		changes to error reporting for set time/location etc.
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------

#include "Scope.h"

// These are the standard LX200 port communication settings
GSSerialPortInfoRecord 	portInfo_LX200 = 	{	
											9600,					/*BAUD RATE*/
											8, 						/*DATA BITS*/
											kGSSerialParityNone,	/*PARITY*/ 
											kGSSerialStopBits10		/*STOP BITS*/
											};

#pragma mark ---LX200---
//----------------------------------------------------------------------
//	TLX200::TLX200()
//
//	Purpose:	Default TLX200 constructor. Setup defaults.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TLX200::TLX200(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType) : 	TScope(	true /*inCanSynch*/,
																							true /*inCanSetScopeTime*/,
																							true /*inCanSetScopeLocation*/,
																							inScopeType,
																							inSerialCommunicator),
																					isHighPrecision(false)
{	
	switch(inScopeType)
	{
		case scope_LX200:
			fScopePrettyName.Assign("\pMeade LX200");
			break;
		case scope_LX200GPS:
			fCanSynch = false; // We want to disable these since a GPS scope knows its location a lot 				
			fCanSetScopeTime = false; // better than Starry Night does!
			fCanSetScopeLocation = false;
			fScopePrettyName.Assign("\pMeade LX200 GPS");
			break;
		case scope_AUTOSTAR:
			fScopePrettyName.Assign("\pMeade Autostar");
			break;
		case scope_AUTOSTAR22:
			fScopePrettyName.Assign("\pMeade Autostar");
			break;
		case scope_GENERICLX200:
			fScopePrettyName.Assign("\pLX200 Compatible");
			break;	
		default:
			fScopePrettyName.Assign("\pUNSUPPORTED Scope");
			break;
	}				
	
	if(fScopeSerialCommunicator)
	{
		// Want to set the port options for this scope.
		// First get the name of the port that is in use so that we can set its options
		GSXSerialPortNameRecord		currentPortName; // This we need in the next step to set the values
		fScopeSerialCommunicator->GetPortOptions(NULL, &currentPortName);
		
		// Set the port info for the scope in question
		fScopeSerialCommunicator->SetPortOptions(&portInfo_LX200, &currentPortName);
	}
	else
	{
		// We pass in a NULL pointer as the inSerialCommunicator when we want to check if the 
		// scope type can set time and location so do nothing here...
	}
}

//----------------------------------------------------------------------
//	TLX200::~TLX200()
//
//	Purpose:	Destruct
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TLX200::~TLX200()
{
}

//----------------------------------------------------------------------
//	TLX200::CheckConnection
//
//	Purpose:	Checks to see if there is an LX200-like scope on the other side
//				of the fScopeSerialCommunicator. If so, sets the isHighPrecision
//				flag depending on the results.
//									
//	Returns:	true if the scope is connected and communicating proplerly
//				false if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
pBoolean TLX200::CheckConnection(void)
{
	// Kludge this by checking the RA (which should work no matter what state the scope is in)
	pBoolean success = false;

	LStr255 rightAscensionString;
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo("#:GR#",5);
	this->SleepDelay(200);
	this->GetScopeResponse(rightAscensionString, 9, kMeadeWaitTime);
	
	if (rightAscensionString.Length() == 8)
	{
		isHighPrecision = false;
		success = true;
	}
	else if (rightAscensionString.Length() == 9)
	{
		isHighPrecision = true;
		success = true;
	}
	else
		success = false;
	
	return success;
}

//----------------------------------------------------------------------
//	TLX200::OpenScopeConnection
//
//	Purpose:	"Opens" the connection to the scope by making sure there is a scope connected
//				to the fScopeSerialCommunicator
//									
//	Returns:	snNoErr if the scope is connected and communicating proplerly
//				pluginErr_CommunicationError if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::OpenScopeConnection(void)
{
	fIsConnected = false;
	snErr error = pluginErr_CommunicationError;
	
	if (this->CheckConnection())
	{
		fIsConnected = true;
		error = snNoErr;
	}
	else
	{
		fIsConnected = false;
		error = pluginErr_CommunicationError;
	}
	
	return error;
}

//----------------------------------------------------------------------
//	TLX200::CheckIsSlewInProgress
//
//	Purpose:	Checks if the Meade scope is currently performing a slew
//									
//	Returns:	true if the scope is currently slewing, false otherwise
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/01/23	DWW			4.0.4		Fixing some LX200 GPS problems
//----------------------------------------------------------------------
pBoolean TLX200::IsSlewInProgress(void)
{

	if (fScopeType == scope_LX200GPS)
	{
		fScopeSerialCommunicator->SendSerialInfo("#:D#",4);
			
		LStr255 scopeResponse;
		this->GetScopeResponse(scopeResponse, 6, kMeadeWaitTime);
	
		if (scopeResponse.Length() > 1)
			return true;
		else
			return false;
	}
	else
		return false;
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
}

//----------------------------------------------------------------------
//	TLX200::SynchScope
//
//	Purpose:	Synch the telescope to the passed Ra and Dec, that is,
//				"tells" the scope where it is pointing. Useful for alignment.
//									
//	Inputs:		synchGaze	the gaze vector to synch to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::SynchScope(double inRA, double inDec)
{
	snErr error = pluginErr_Failed;

	if (fCanSynch)
	{
		error = this->SetLX200ObjectRaDec(inRA, inDec);
		
		if (error == snNoErr)
		{
			fScopeSerialCommunicator->SendSerialInfo("#:CM#",5);
					
			LStr255 scopeResponse;
			this->GetScopeResponse(scopeResponse, 2, kMeadeWaitTime);
			if (scopeResponse.Length() == 0)
				error = pluginErr_SynchError;
			else
				error = snNoErr;
		}
	}

	return error;
}

//----------------------------------------------------------------------
//	TLX200::SlewScope
//
//	Purpose:	Slew the scope to the passed gaze. LX200 takes Ra and Dec OK.
//									
//	Inputs:		inRa,inDec	Ra an Decto slew to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::SlewScope(double inRA, double inDec)
{
	snErr error = snNoErr;

	if (this->IsSlewInProgress() == false)
	{
		this->SetLX200ObjectRaDec(inRA,inDec);
		fScopeSerialCommunicator->SendSerialInfo("#:MS#",5);
			
		LStr255 scopeResponse;
		this->GetScopeResponse(scopeResponse, 20, kMeadeWaitTime);
		
		PluginAssert(scopeResponse.Length() > 0);
		
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);

		unsigned char responseCharacter = scopeResponse[1];
		switch (responseCharacter){
			case '0':
				error = snNoErr;
				break;
			case '1':
				error = pluginErr_ObjectBelowHorizon;
				break;
			case '2':
				error = pluginErr_LX200SpecificError;
				break;
			case '4':
				error = pluginErr_LX200SpecificError;
				break;
			default:
				error = pluginErr_CommunicationError;
				break;
		}
	}
	else
		error = pluginErr_SlewAlreadyInProgress;
	
	return error;
}


//----------------------------------------------------------------------
//	TLX200::GetScopeRaDec()
//
//	Purpose:	Determine the RA and Dec that the scope is pointing at.
//				
//	Inputs:		outRA	Ra of the scope
//				outDec	Dec of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::GetScopeRaDec(double& outRA, double& outDec)
{
	snErr error = pluginErr_CommunicationError;

	pBoolean success 	= false;
	pBoolean decSuccess = false;
	pBoolean raSuccess 	= false;

	LStr255 rightAscensionString;
	LStr255 declinationString;
	
	// Put all this in a for loop since autostar will occasionally have trouble
	// returning both of the strings on the first try.
	for (SInt16 i = 0; i < 4; i++)
	{	
		// Get the Right Ascension
		if (raSuccess == false)
		{
			rightAscensionString.Assign("\p"); // To ensure the length is zero
			fScopeSerialCommunicator->SendSerialInfo("#:GR#",5);
			TScope::SleepDelay(k100MilliSeconds);
			this->GetScopeResponse(rightAscensionString, 9, kMeadeWaitTime);
			
			if ((rightAscensionString.Length() == 8) || (rightAscensionString.Length() == 9))
				raSuccess = true;
			
			fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		}
		

		// Get the Declination
		if (decSuccess == false)
		{
			declinationString.Assign("\p");
			fScopeSerialCommunicator->SendSerialInfo("#:GD#",5);
			TScope::SleepDelay(k100MilliSeconds);
			this->GetScopeResponse(declinationString, 10, kMeadeWaitTime);
			
			if ((declinationString.Length() == 7) || (declinationString.Length() == 10))
				decSuccess = true;
			
			fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		}
				
		success = (raSuccess && decSuccess);
		
		if (success)
		{
			snErr convertRAError	= this->ConvertLX200RAToDouble(rightAscensionString, outRA);
			snErr convertDecError	= this->ConvertLX200DecToDouble(declinationString, outDec);

			if ((convertRAError == snNoErr) && (convertDecError == snNoErr))
				error = snNoErr;
			else
				error = pluginErr_CommunicationError;
		}
		else
			error = pluginErr_CommunicationError;
	}
	
	return error;
}

//----------------------------------------------------------------------
//	TLX200::SetScopeLocation()
//
//	Purpose:	Sets the location of the scope in the scope memory
//				
//	Inputs:		inlocation	pointer to a SN PLocation.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::SetScopeLocation(const PLocation* inLocation)
{
	if (fCanSetScopeLocation)
		return this->SetLX200Location(inLocation->fEarthLatitude, inLocation->fEarthLongitude);
	else
		return pluginErr_SetLocationError;
}

//----------------------------------------------------------------------
//	TLX200::SetScopeTime()
//
//	Purpose:	Sets the time of the scope in its memory
//				
//	Inputs:		inTime	pointer to a SN PTime.
//				timeZoneOffsetInSeconds number of seconds offset from GMT
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::SetScopeTime(const PTime* inTime, SInt32 timeZoneOffsetInSeconds )
{
	if (fCanSetScopeTime)
		return this->SetLX200Time((inTime->fJulianDate), timeZoneOffsetInSeconds);
	else
		return pluginErr_SetTimeError;
}

#pragma mark --LX200HelperFunctions--
//----------------------------------------------------------------------
//	TLX200::MakeLX200SetRAString()
//
//	Purpose:	Format the outstring to set Meade RA coords				
//									
//	Inputs:		inRA is a the Right Ascension in decimal radians.
//				outString will be assigned the proper pascal string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/01/15	DWW			4.0.4		Fixes for LX200 GPS
//----------------------------------------------------------------------

void TLX200::MakeLX200SetRAString(double inRA, LStr255& outString)
{	
	PluginAssert(&outString);
	PluginAssert((inRA >= 0.0) && (inRA <= 2*kPiDefine)); // Ensure that the Ra is in the Correct range
		
	inRA = TScope::ModPi2(inRA);
	
	// These are used for constructing the string output as Meade likes them:
	// HH:MM.T for low precision
	// HH:MM:SS for high precision
	SInt32 HH = 0;
	SInt32 MM = 0;
	SInt32 T  = 0;
	SInt32 SS = 0;
	
	if (!isHighPrecision)
		TScope::ConvertRadiansToHoursMinutesTenths(inRA, HH, MM, T);
	else
		TScope::ConvertRadiansToHoursMinutesSeconds(inRA, HH, MM, SS);
	
	// make the string
	outString.Assign("\p#:Sr ");
	
	if (HH < 10)
	{
		outString.Append('0');
		outString.Append(HH);
	}
	else 
		outString.Append(HH);
		
	outString.Append(':');
	
	if (MM < 10)
	{
		outString.Append('0');
		outString.Append(MM);
	}
	else
		outString.Append(MM);
	
	if (!isHighPrecision)
	{
		outString.Append('.');
		outString.Append(T);
	}
	else
	{
		outString.Append('.');
		
		if (SS < 10)
		{
			outString.Append('0');
			outString.Append(SS);
		}
		else
			outString.Append(SS);
	}
	
	outString.Append('#');				// The Meade termination character.
}
	
//----------------------------------------------------------------------
//	TLX200::MakeLX200SetDecString()
//
//	Purpose:	Returns a string properly formatted for setting Meade Dec coords				
//									
//	Inputs:		inDec is a the Declination in decimal radians. (-Pi, Pi)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/01/15	DWW			4.0.4		Fixes for LX200 GPS
//----------------------------------------------------------------------

void TLX200::MakeLX200SetDecString(double inDec, LStr255& outString)
{
	PluginAssert(&outString);
	PluginAssert((inDec >= -90.0) && (inDec <= 90.0)); // Ensure that the Dec is in the Correct range
	
	outString.Assign("\p#:Sd ");
	
	// Add the sign to the string, + or -. Do this before changing inDec below.
	if (inDec > 0.0)
		outString.Append('+');
	else
		outString.Append('-');
	
	// To deal with the negative side of the scale we just take the abs value and put a negative sign
	// on the string.
	if (inDec < 0.0)
		inDec = fabs(inDec);

	// Meade likes their declinations in the form:
	// sDD*MM		for low precision AUTOSTAR
	// sDD*MM.T		for low precision LX200 ?
	// sDD*MM:SS	for high precision (all)
	SInt32  DD = 0;
	SInt32  MM = 0;
	SInt32	T  = 0;
	SInt32  SS = 0;
	
	if (isHighPrecision)
		TScope::ConvertRadiansToDegreesMinutesSeconds(inDec, DD, MM, SS);
	else
		TScope::ConvertRadiansToDegreesMinutes(inDec, DD, MM);
	
	PluginAssert((DD >= 0) && (MM >= 0) && (SS >= 0) && (T >= 0));
	PluginAssert((DD < 90) && (MM < 60) && (SS < 60) && (T < 10));
	
	// Build the string.
	// Append a zero if necess and add the number of degrees to the string	
	if (DD < 10)
		outString.Append('0');
	outString.Append(DD);
	
	outString.Append('*');
	
	// Append a zero if neces and add the number of minutes to the string	
	if (MM < 10)
		outString.Append('0');
	outString.Append(MM);
		
	if (isHighPrecision)
	{
		outString.Append(':');
		if (SS < 10)
			outString.Append('0');
		outString.Append(SS);		
	}
	else
	{
		/*
		if (fScopeType != scope_AUTOSTAR)
		{
			outString.Append('.');
			outString.Append(T);
		}
		*/
	}

	outString.Append('#');
}

//----------------------------------------------------------------------
//	TLX200::SetLX200ObjectRaDec()
//
//	Purpose:	Sends the necessary strings to a Meade scope to set its 
//				object RA and DEC. (We can then Synch or Slew to the 'object')				
//									
//	Inputs:		inRA is a the Right Ascension in decimal radians.
//				inDec is the Declination in decimal radians
//				inScope is the telescope object to send the messages to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::SetLX200ObjectRaDec(double inRA, double inDec)
{
	LStr255 setRaString;
	LStr255 setDecString;
	
	this->MakeLX200SetRAString(inRA, setRaString);
	this->MakeLX200SetDecString(inDec, setDecString);
	
	fScopeSerialCommunicator->SendSerialInfo(setRaString);

	// Wait a bit to give scope time to respond
	 TScope::SleepDelay(k100MilliSeconds);
	
	// check if the scope is ready to receive the Dec string		
	LStr255 statusMessageFromScope;
	statusMessageFromScope.Assign("\p");
	this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);
	
	if (statusMessageFromScope[1] == '1')
	{
		fScopeSerialCommunicator->SendSerialInfo(setDecString);
		
		// reset the status to false.
		statusMessageFromScope.Assign("\p");
		this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);
		
		if (statusMessageFromScope[1] == '1')
			return snNoErr;
		else
			return pluginErr_SynchError;
	}
	else
		return pluginErr_SynchError;
}

//----------------------------------------------------------------------
//	TLX200::MakeLX200SetLatString()
//
//	Purpose:	Format the outString to set Meade Latitude coords				
//									
//	Inputs:		inLat is the Latitude in decimal radians (-Pi, Pi)
//				outString will be assigned the formatted pascal string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
void TLX200::MakeLX200SetLatString(double inLat, LStr255& outString)
{	
	PluginAssert(&outString);
	
	// Meade wants Lat in the form sDD*MM
	outString.Assign("#:St ");
	
	// First add the sign to the string, + or -.
	if ( inLat > 0.0)
		outString.Append('+');
	else
		outString.Append('-');
	
	// To Deal with the negative side of the scale
	if ( inLat < 0.0)
		inLat = fabs(inLat);
	
	// Ensure that the input is in the range
	inLat = TScope::ModPi2(inLat);
	
	SInt32 	DD = 0;
	SInt32	MM = 0;
	
	TScope::ConvertRadiansToDegreesMinutes(inLat, DD, MM);
					
	// Build the rest of the string.
	// Append a zero if neces and add the number of degrees to the string	
	if (DD < 10)
	{
		outString.Append('0');
		outString.Append(DD);
	}
	else
		outString.Append(DD);

	outString.Append('*');

	// Append a zero if neces and add the number of minutes to the string	
	if (MM <10)
	{
		outString.Append('0');
		outString.Append(MM);
	}
	else
		outString.Append(MM);

	outString.Append('#');
}

//----------------------------------------------------------------------
//	TLX200::MakeLX200SetLongString()
//
//	Purpose:	Format the outString to set Meade Latitude coords				
//									
//	Inputs:		inLong is the Longitude in decimal radians.
//				outString will be assigned the proper pascal string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
void TLX200::MakeLX200SetLongString(double inLong, LStr255& outString)
{	
	PluginAssert(&outString);
	
	// Meade likes the Longitude in DDD*MM
	SInt32 DDD 	= 0;
	SInt32 MM	= 0;
	
	// SN passes west as -ve and east as +ve but Meade wants 0-360 westward from 
	// Greenwich so need to do a bit of twiddling first...
	if(inLong < 0)
		inLong = fabs(inLong);
	else
		inLong = kTwicePi - inLong;
	
	TScope::ConvertRadiansToDegreesMinutes(inLong, DDD, MM);
	
	outString.Assign("#:Sg ");
	
	if (DDD < 100)
	{
		outString.Append('0');
		if (DDD < 10)
		{
			outString.Append('0');
			outString.Append(DDD);
		}
		outString.Append(DDD);
	}
	else
		outString.Append(DDD);
	
	outString.Append('*');
	
	if (MM < 10)
	{
		outString.Append('0');
		outString.Append(MM);
	}
	else
		outString.Append(MM);
	
	outString.Append('#');
}

//----------------------------------------------------------------------
//	TLX200::SetLX200Location()
//
//	Purpose:	Sends the necessary strings to a Meade scope to set its 
//				Lat and Long			
//									
//	Inputs:		inLat is a the Latitude in decimal radians.
//				inLong is the Longitude in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TLX200::SetLX200Location(double inLat, double inLong)
{
	LStr255 setMeadeLatitudeString;
	LStr255 setMeadeLongitudeString;
	
	this->MakeLX200SetLatString(inLat, setMeadeLatitudeString);
	this->MakeLX200SetLongString(inLong, setMeadeLongitudeString);

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_OUT);
	fScopeSerialCommunicator->SendSerialInfo(setMeadeLatitudeString);

	// check if the scope is ready to receive the Longitude string		
	LStr255 statusMessageFromScope;
	this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);
	if (statusMessageFromScope[1] == '1')
	{
		statusMessageFromScope[1] = '0';
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_OUT);
		fScopeSerialCommunicator->SendSerialInfo(setMeadeLongitudeString);
		this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);

		if (statusMessageFromScope[1] == '1')
			return snNoErr;
		else
			return pluginErr_SetLocationError;
	}
	else
		return pluginErr_CommunicationError;
}


//----------------------------------------------------------------------
//	TLX200::MakeLX200SetTimeString()
//
//	Purpose:	Creates a string suitable to be passed to a Meade scope
//				to set its internal time.			
//									
//	Inputs:		inJulianDate is the universal time (note that we only use the 
//				fractional part of this as to only get the time of day and not the date)
//				outString is the formatted string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------

void TLX200::MakeLX200SetTimeString(double inJulianDate, SInt32 inSecondsOffGMT, LStr255& outString)
{
	PluginAssert(&outString);
	
	SInt32	sYear		= 0; // Used as placeholder only
	SInt32	sMonth		= 0; // Used as placeholder only 
	SInt32	sDay		= 0; // Used as placeholder only
	SInt32	sHours		= 0;
	SInt32	sMinutes	= 0;
	SInt32	sSeconds	= 0;
	
	sn_Julian2CivilTime(inJulianDate, &sYear, &sMonth, &sDay, &sHours, &sMinutes, &sSeconds);
	
	SInt32 hoursOffGMT = (inSecondsOffGMT/ 3600);
	sHours = sHours + hoursOffGMT;
	if(sHours < 0)
		sHours = sHours + 24;
	
	outString.Assign("#:SL ");
	
	if (sHours < 10)
	{
		outString.Append('0');
		outString.Append(sHours);
	}
	else
		outString.Append(sHours);
	
	outString.Append(':');
	
	if (sMinutes < 10)
	{
		outString.Append('0');
		outString.Append(sMinutes);
	}
	else
		outString.Append(sMinutes);
	
	outString.Append(':');
	
	if (sSeconds < 10)
	{
		outString.Append('0');
		outString.Append(sSeconds);
	}
	else
		outString.Append(sSeconds);

	outString.Append('#');
}

//----------------------------------------------------------------------
//	TLX200::SetLX200Time()
//
//	Purpose:	Sends the necessary strings to a Meade scope to set its 
//				object RA and DEC				
//									
//	Inputs:		inLat is a the Latitude in decimal radians.
//				inLong is the Longitude in decimal radians
//				inScope is the telescope object to send the messages to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TLX200::SetLX200Time(double inJulianDate, SInt32 inSecondsOffUT)
{
	LStr255 setTimeString;
	this->MakeLX200SetTimeString(inJulianDate, inSecondsOffUT, setTimeString);
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_OUT);
	fScopeSerialCommunicator->SendSerialInfo(setTimeString);
	
	// check if the scope is ready to receive the Dec string		
	LStr255 statusMessageFromScope;
	this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);
	
	if (statusMessageFromScope[1] == '1')
		return snNoErr; // everything A OK.
	else
		return pluginErr_SetTimeError; // there was some problem
}

//----------------------------------------------------------------------
//	TLX200::ConvertLX200RAToDouble()
//
//	Purpose:	Converts the RA string sent by a Meade scope to the format
//				desired by Starry Night.				
//									
//	Inputs:		inRAString is the string sent by the scope
//				outRA is the Right Ascension in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/01/15	DWW			4.0.4		Fixes for LX200 GPS
//----------------------------------------------------------------------
snErr TLX200::ConvertLX200RAToDouble(LStr255 inRAString, double& outRA)
{
	PluginAssert(&outRA);
	PluginAssert(inRAString.Length() > 0);
	
	if (inRAString.Length() == 9)
		isHighPrecision = true;
	else if (inRAString.Length() == 8)
		isHighPrecision = false;
	else
	{
		PluginAssert(false); // This is a serious problem...
		return pluginErr_ReturnedBadValue;
	}
	
	SInt32 hours		= 0;
	SInt32 minutes		= 0;
	SInt32 tenthsOfMins	= 0;
	SInt32 seconds		= 0;
	double hrsInDecimal	= 0.0;
	
	LStr255 hoursString;
	LStr255 minsString;
	LStr255 tenthsMinsString;
	LStr255 secondsString;
	
	hoursString.Assign(inRAString, 1, 2);
	minsString.Assign(inRAString, 4, 2);
	hours	= hoursString;
	minutes	= minsString;
				
	if (isHighPrecision)
	{
		secondsString.Assign(inRAString, 7, 2);
		seconds = secondsString;
	
		hrsInDecimal = (hours + ((double)minutes/60) + ((double)seconds/3600));
		outRA = hrsInDecimal * kRadiansPerHour;
	}
	else
	{
		tenthsMinsString.Assign(inRAString, 7, 1);
		tenthsOfMins = tenthsMinsString;
	
		hrsInDecimal = (hours + ((double)minutes/60) + ((double)tenthsOfMins/600));
		outRA = hrsInDecimal * kRadiansPerHour;
	}
	
	if ((outRA >= 0.0) && (outRA <= 2*kPiDefine))
		return snNoErr;
	else
		return pluginErr_ReturnedBadValue;
}

//----------------------------------------------------------------------
//	TLX200::ConvertLX200DecToDouble()
//
//	Purpose:	Converts the RA string sent by a Meade scope to the format
//				desired by Starry Night.				
//									
//	Inputs:		inDecString is the string sent by the scope
//				outDec is the Declination in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

snErr TLX200::ConvertLX200DecToDouble(LStr255 inDecString, double& outDec)
{
	PluginAssert(&outDec);
	PluginAssert(inDecString.Length() > 0);
	
	if (inDecString.Length() == 10)
		isHighPrecision = true;
	else if (inDecString.Length() == 7)
		isHighPrecision = false;
	else
	{
		PluginAssert(false); // This is a serious problem...
		return pluginErr_ReturnedBadValue;
	}	
	
	char 	sign = inDecString[1]; // The sign of the declination
	
	SInt32 	degrees	= 0;
	SInt32 	minutes	= 0;
	SInt32 	seconds	= 0;
	double 	degreesInDecimal = 0.0;
	
	LStr255 degreesString;
	LStr255 minsString;
	LStr255 secondsString;
	
	degreesString.Assign(inDecString,2,2);
	minsString.Assign(inDecString,5,2);
	
	degrees = degreesString;
	minutes = minsString;
	
	if (isHighPrecision)
	{
		secondsString.Assign(inDecString,8,2);
		seconds = secondsString;
	}
	
	degreesInDecimal = degrees + ((double)minutes/60.0) + ((double)seconds/3600.0);

	if (sign == '-')
		outDec = -1.0*(degreesInDecimal * kDegreesToRadians);
	else if (sign == '+')
		outDec = degreesInDecimal * kDegreesToRadians;
	else
		return pluginErr_ReturnedBadValue;
	
	
	if ((outDec <= 90.0) && (outDec >= -90.0))
		return snNoErr;
	else
		return pluginErr_ReturnedBadValue;
}


//----------------------------------------------------------------------
//	TLX200::WaitForSlewFinishedCharacterOrTimeoutPassed()
//
//	Purpose:	Queries the scope to determine if a slew is complete			
//									
//	Inputs:		NONE
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TLX200::WaitForSlewFinishedCharacterOrTimeoutPassed(void)
{
	LStr255 scopeResponseChar;

	UInt32	distanceBarCharacterASCIIValue 	= 255; 	// LX200 seems to send the ASCII character 255
	UInt32	scopeResponseCharASCIIValue 	= 255;	// Initialize so the loop goes at least once
	
	if(fScopeType == scope_AUTOSTAR || fScopeType == scope_AUTOSTAR22)
	{
		distanceBarCharacterASCIIValue 	= 127; 	// Autostar sends the "Del" (ASCII 127) character
		scopeResponseCharASCIIValue 	= 127;	// Initialize so the loop goes at least once
	}

	TScope::SleepDelay(k500MilliSeconds);
				
	for(UInt32 i = 0; (scopeResponseCharASCIIValue == distanceBarCharacterASCIIValue); i++)
	{
		scopeResponseCharASCIIValue = 0;
		
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		fScopeSerialCommunicator->SendSerialInfo("#:D#",4);
		TScope::SleepDelay(k100MilliSeconds);
		this->GetScopeResponse(scopeResponseChar, 2, k1KMilliSeconds);
		scopeResponseCharASCIIValue = scopeResponseChar[1];
	}
	
	if(fScopeType == scope_AUTOSTAR || fScopeType == scope_AUTOSTAR22)
		TScope::SleepDelay(kAUTOSTARSettleTime); 	// The Autostars can take a lot longer to settle 
	else											// than the LX200
		TScope::SleepDelay(kLX200SettleTime);
	
	return;	
}








