//----------------------------------------------------------------------
//	Copyright © 1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		MeadeScope.cpp
//
//	Contains:	Class for controlling Meade, Autostar and compatible type telescopes
//				
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/11/15	MK			5.7			Removed forced sync on connection closure
//  04/06/24	MK			5.0			Removed delays and unnecessary code, added dir slewing 
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//----------------------------------------------------------------------

#include "Scope.h"
// marko is great 
// dave is super duper

// These are the standard Meade port communication settings
GSSerialPortInfoRecord 	portInfo_MeadeDefault = {	
													9600,					/*BAUD RATE*/
													8, 						/*DATA BITS*/
													kGSSerialParityNone,	/*PARITY*/ 
													kGSSerialStopBits10		/*STOP BITS*/
												};

#pragma mark ---TMeadeScope---
//----------------------------------------------------------------------
//	TMeadeScope::TMeadeScope()
//
//	Purpose:	Default Meade Telescope constructor. Setup defaults.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TMeadeScope::TMeadeScope(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType) :	TScope(inSerialCommunicator, inScopeType),
																								fIsHighPrecision(false)
{	
	fCanSynch				= true;
	fCanSlew				= true;
	fCanSetScopeTime		= true;
	fCanSetScopeLocation 	= true;
	fCanDirSlew				= true;
	fCanSetSlewRate			= true;
	fCanSetSlewDirRate		= false;
	fScopeWaitTime			= kMeadeWaitTime;

	fCanPark				= false;					
	fCanUnpark				= false;
	fCanSetPark				= false;

	switch(inScopeType)
	{
		case scope_LX200:
			fScopePrettyName.Assign("\pMeade LX200");
			break;
		case scope_LX200GPS:
			fCanPark	= true;					
			fCanSynch	= true;				
			fScopePrettyName.Assign("\pMeade LX200 GPS");
			break;
		case scope_AUTOSTAR:
			fCanPark	= true;					
			fCanSynch	= true;				
			fScopePrettyName.Assign("\pMeade Autostar");
			break;
		case scope_GENERICLX200:
			fScopePrettyName.Assign("\pLX200 Compatible");
			break;	
		default:
			fScopePrettyName.Assign("\pUNSUPPORTED Scope");
			break;
	}				
	
	if (inSerialCommunicator)
	{
		// Want to set the port options for this scope.
		// First get the name of the port that is in use so that we can set its options
		GSXSerialPortNameRecord		currentPortName; // This we need in the next step to set the values
		fScopeSerialCommunicator->GetPortOptions(NULL, &currentPortName);
		
		// Set the port info for the scope in question
		fScopeSerialCommunicator->SetPortOptions(&portInfo_MeadeDefault, &currentPortName);
	}
	else
	{
		// We passed in a NULL pointer as the inSerialCommunicator when we want to check if the 
		// scope type can set time and location so do nothing here...
	}
}

//----------------------------------------------------------------------
//	TMeadeScope::~TMeadeScope()
//
//	Purpose:	Destruct
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TMeadeScope::~TMeadeScope()
{
}

//----------------------------------------------------------------------
//	TMeadeScope::CheckConnection
//
//	Purpose:	Checks to see if there is an Meade-like scope on the other side
//				of the fScopeSerialCommunicator. If so, sets the fIsHighPrecision
//				flag depending on the results.
//									
//	Returns:	true if the scope is connected and communicating proplerly
//				false if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
pBoolean TMeadeScope::CheckConnection(void)
{
	// Kludge this by checking the RA (which should work no matter what state the scope is in)
	pBoolean success = false;

	LStr255 rightAscensionString("");
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo(LStr255(":GR#"));
//	TScope::SleepDelay(k200MilliSeconds);
	this->GetScopeResponse(rightAscensionString, 9, fScopeWaitTime);
	
	if (rightAscensionString.Length() == 8)
	{
		fIsHighPrecision = false;
		success = true;
	}
	else if (rightAscensionString.Length() == 9)
	{
		fIsHighPrecision = true;
		success = true;
	}
	else
		success = false;
	
	return success;
}

//----------------------------------------------------------------------
//	TMeadeScope::OpenScopeConnection
//
//	Purpose:	"Opens" the connection to the scope by making sure there is a scope connected
//				to the fScopeSerialCommunicator
//									
//	Returns:	snNoErr if the scope is connected and communicating proplerly
//				pluginErr_CommunicationError if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::OpenScopeConnection(void)
{
	fIsConnected = false;
	snErr error = pluginErr_CommunicationError;
	
	if (this->CheckConnection())
	{
		fIsConnected = true;
		error = snNoErr;
	}
	else
	{
		fIsConnected = false;
		error = pluginErr_CommunicationError;
	}
	
	return error;
}

//----------------------------------------------------------------------
//	TMeadeScope::CheckIsInProgress
//
//	Purpose:	Checks if the Meade scope is currently performing a slew
//									
//	Returns:	true if the scope is currently slewing, false otherwise
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/01/23	DWW			4.0.4		Fixing some LX200 GPS problems
//----------------------------------------------------------------------
pBoolean TMeadeScope::IsSlewInProgress(void)
{

	if (fScopeType == scope_LX200GPS)
	{
		fScopeSerialCommunicator->SendSerialInfo("#:D#",4);
			
		LStr255 scopeResponse;
		this->GetScopeResponse(scopeResponse, 6, fScopeWaitTime);
	
		if (scopeResponse.Length() > 1)
			return true;
		else
			return false;
	}
	else
		return false;
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
}

//----------------------------------------------------------------------
//	TMeadeScope::SynchScope
//
//	Purpose:	Synch the telescope to the passed Ra and Dec, that is,
//				"tells" the scope where it is pointing. Useful for alignment.
//									
//	Inputs:		synchGaze	the gaze vector to synch to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::SynchScope(double inRA, double inDec)
{
	snErr error = pluginErr_Failed;

	if (fCanSynch)
	{
		error = this->SetMeadeObjectRaDec(inRA, inDec);
		
		if (error == snNoErr)
		{
			fScopeSerialCommunicator->SendSerialInfo("#:CM#",5);
					
			LStr255 scopeResponse;
			this->GetScopeResponse(scopeResponse, 34, fScopeWaitTime);
			if (scopeResponse.Length() == 0)
				error = pluginErr_SynchError;
			else
				error = snNoErr;
			fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		}
	}

	return error;
}

//----------------------------------------------------------------------
//	TMeadeScope::SlewScope
//
//	Purpose:	Slew the scope to the passed gaze. Meade takes Ra and Dec OK.
//									
//	Inputs:		inRa,inDec	Ra an Decto slew to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::SlewScope(double inRA, double inDec)
{
	snErr error = snNoErr;

	this->SetMeadeObjectRaDec(inRA,inDec);
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo("#:MS#",5);
		
	LStr255 scopeResponse;
	this->GetScopeResponse(scopeResponse, 20, fScopeWaitTime);
	
	//PluginAssert(scopeResponse.Length() > 0);
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);

	unsigned char responseCharacter = scopeResponse[1];
	switch (responseCharacter)
	{
		case '0':
			error = snNoErr;
			break;
		case '1':
			error = pluginErr_ObjectBelowHorizon;
			break;
		case '2':
			error = pluginErr_LX200SpecificError;
			break;
		case '4':
			error = pluginErr_LX200SpecificError;
			break;
		default:
			error = pluginErr_CommunicationError;
			break;
	}
	
	return error;
}

//----------------------------------------------------------------------
//	TMeaseScope::SlewScope
//
//	Purpose:	Slew the telescope in the passed Direction
//									
//	Inputs:		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/07/13	MK			5.0			Finalized
//	04/09/01	DWW			5.0.1		Added for advanced telescope control in Mac
//----------------------------------------------------------------------
snErr TMeadeScope::SlewScope(const PScopeMotion* inMotionParam)
{

	LStr255 outString;
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);

	ScopeSlewRateT theSlewRate = inMotionParam->fRelativeSlewRate;
	ScopeDirectionT theDirection = inMotionParam->fMotionDirection;
	
	this->SetSlewRate(theSlewRate);
	
	switch (theDirection) {
		case direction_UP:
			outString=":Mn#";
			break;
		case direction_DOWN:
			outString=":Ms#";
			break;
		case direction_LEFT:
			outString=":Mw#";
			break;
		case direction_RIGHT:
			outString=":Me#";
			break;
		case direction_NONE_STOP:
			outString=":Qe#";
			fScopeSerialCommunicator->SendSerialInfo(outString);
			outString=":Qw#";
			fScopeSerialCommunicator->SendSerialInfo(outString);
			outString=":Qn#";
			fScopeSerialCommunicator->SendSerialInfo(outString);
			outString=":Qs#";
		default:
			break;
	}
	fScopeSerialCommunicator->SendSerialInfo(outString);
	return snNoErr;
}


//----------------------------------------------------------------------
//	TScope::SetSlewRate
//
//	Purpose:	Set scope's global slew rate.
//									
//	Inputs:		speed   rate of slew
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/07/16	MK			5.0			Finalized
//----------------------------------------------------------------------
snErr TMeadeScope::SetSlewRate(const ScopeSlewRateT speed)
{
	LStr255 outString;
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	
	switch (speed) {
		case rate_1:
			outString=":RG#";
			break;
		case rate_2:
			outString=":RC#";
			break;
		case rate_3:
			outString=":RM#";
			break;
		case rate_4:
			outString=":RS#";
			break;
		default:
			outString=":RM#"; // default to rate_3
			break;
	}
	fScopeSerialCommunicator->SendSerialInfo(outString);
	return snNoErr;
}
	
//----------------------------------------------------------------------
//	TMeadeScope::GetScopeRaDec()
//
//	Purpose:	Determine the RA and Dec that the scope is pointing at.
//				
//	Inputs:		outRA	Ra of the scope
//				outDec	Dec of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/13	MK			5.0			Removed unnecessary looping
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::GetScopeRaDec(double& outRA, double& outDec)
{
	snErr error = pluginErr_CommunicationError;

//GetScopeLocation produces long delays/unnecessary choppiness in the interface
/*	double jdate, lat, lon;
	long offset;

	GetScopeTime(jdate,offset);
	GetScopeLocation(lat,lon);
*/
		
	LStr255 rightAscensionString;
	LStr255 declinationString;
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	rightAscensionString.Assign(""); // To ensure the length is zero
	fScopeSerialCommunicator->SendSerialInfo(LStr255(":GR#"));
			
	//TScope::SleepDelay(k100MilliSeconds);
			
	if (fIsHighPrecision)
		this->GetScopeResponse(rightAscensionString, 9, fScopeWaitTime);
	else
		this->GetScopeResponse(rightAscensionString, 8, fScopeWaitTime);
			
	unsigned char debugRAStr[256];
	LString::CopyPStr(StringPtr(rightAscensionString), &debugRAStr[0], sizeof(debugRAStr));
			
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		
	
	declinationString.Assign("");
	fScopeSerialCommunicator->SendSerialInfo(LStr255(":GD#"));
	//TScope::SleepDelay(k100MilliSeconds);
		
	if (fIsHighPrecision)
		this->GetScopeResponse(declinationString, 10, fScopeWaitTime);
	else
		this->GetScopeResponse(declinationString, 7, fScopeWaitTime);
			
	unsigned char debugDecStr[256];
	LString::CopyPStr(StringPtr(declinationString), &debugDecStr[0], sizeof(debugDecStr));

	snErr convertRAError	= this->ConvertMeadeRAToDouble(rightAscensionString, outRA);
	snErr convertDecError	= this->ConvertMeadeDecToDouble(declinationString, outDec);

	if ((convertRAError == snNoErr) && (convertDecError == snNoErr))
		error = snNoErr;
	else
		error = pluginErr_CommunicationError;
	
	return error;
}

	
//----------------------------------------------------------------------
//	TMeadeScope::GetScopeAzAlt()
//
//	Purpose:	Provides a method alternative to RA/Dec for getting scope position.
//				
//	Inputs:		outAz	Az of the scope
//				outAlt	Alt of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  05/03/09	MK			5.02		currently not supported
//----------------------------------------------------------------------
snErr TMeadeScope::GetScopeAzAlt(double& outAz, double& outAlt)
{
	return pluginErr_FunctionNotSupported;
}


//----------------------------------------------------------------------
//	TMeadeScope::GetScopeTime()
//
//	Purpose:	Determine the time as defined by the scope (or GPS if applicable)
//				
//	Inputs:		outJulianDate	The current Julian Date	
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/13	DWW			4.5.0		New for Pro 4.5
//----------------------------------------------------------------------
snErr TMeadeScope::GetScopeTime(double& outJulianDate, long& outScopeTimeZoneOffsetSeconds)
{
	snErr error = snNoErr;
	
	long year	= 0;
	long month	= 0;
	long day	= 0;
	long hour	= 0;
	long minute	= 0;
	long second	= 0;
	
	double offsetFromGMT = 0.0;

	// Get and parse the date string
	LStr255 dateString("");
	fScopeSerialCommunicator->SendSerialInfo("#:GC#",5);
	//TScope::SleepDelay(k100MilliSeconds);
	this->GetScopeResponse(dateString, 9, fScopeWaitTime);

	if (dateString.Length() == 9)
	{
		LStr255 monthString("");
		LStr255 dayString("");
		LStr255 yearString("");
		
		monthString.Assign(dateString, 1, 2);
		dayString.Assign(dateString, 4, 2);
		yearString.Assign(dateString, 7, 2);
		
		month	= monthString;
		day		= dayString;
		year	= yearString;
		year	+= 2000; // This will work for the next 100 years or so, so I should be fine...
	}
	else
		error = pluginErr_CommunicationError;
	
	// Get and parse the time string
	LStr255 timeString("");
	fScopeSerialCommunicator->SendSerialInfo("#:GL#",5);
	//TScope::SleepDelay(k100MilliSeconds);
	this->GetScopeResponse(timeString, 9, fScopeWaitTime);

	if (timeString.Length() == 9)
	{
		LStr255 hourString("");
		LStr255 minuteString("");
		LStr255 secondString("");
		
		hourString.Assign(timeString, 1, 2);
		minuteString.Assign(timeString, 4, 2);
		secondString.Assign(timeString, 7, 2);
		
		hour	= hourString;
		minute	= minuteString;
		second	= secondString;
	}
	else
		error = pluginErr_CommunicationError;
	
	// Get and parse the GMT offset string
	LStr255 offsetFromGMTString("");
	fScopeSerialCommunicator->SendSerialInfo("#:GG#",5);
	//TScope::SleepDelay(k100MilliSeconds);
	this->GetScopeResponse(offsetFromGMTString, 4, fScopeWaitTime);

	char debugStr[256];
	CopyPascalStringToC(offsetFromGMTString, debugStr);
	
	long GMTOffsetHours = 0;
	long GMTOffsetTenthsHours = 0;
	if (offsetFromGMTString.Length() == 4 || offsetFromGMTString.Length() == 6)
	{
		char sign;
		LStr255 hourGMTString("");
		LStr255 tenthsHourGMTString("0");
		
		sign = offsetFromGMTString[1];
		hourGMTString.Assign(offsetFromGMTString, 2, 2);
		if (offsetFromGMTString.Length() == 6)
			tenthsHourGMTString.Assign(offsetFromGMTString, 5, 1);
		
		GMTOffsetHours			= hourGMTString;
		GMTOffsetTenthsHours	= tenthsHourGMTString;
		
		// Note that Meade has + for Westerly longitudes
		if (sign == '+')
			offsetFromGMT = GMTOffsetHours + GMTOffsetTenthsHours/10.0;
		else if (sign == '-')
			offsetFromGMT = -1.0*(GMTOffsetHours + (GMTOffsetTenthsHours)/10.0);
		else
			error = pluginErr_CommunicationError;
	}
	else
		error = pluginErr_CommunicationError;

	double localDate = 0.0;
	sn_Civil2JulianTime(&localDate, year, month, day, hour, minute, second);
	
	outJulianDate = localDate + (offsetFromGMT/24.0);

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);	
	return error;
}

//----------------------------------------------------------------------
//	TMeadeScope::GetScopeLocation()
//
//	Purpose:	Determine the location as defined by the scope (or GPS if applicable)
//				
//	Inputs:		outLat	scope latitude in radians from -Pi/2..+Pi/2
//				outLong	scope longitude in radians from -Pi..+Pi (+ve is westerly long)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/13	DWW			4.5.0		New for Pro 4.5
//----------------------------------------------------------------------
snErr TMeadeScope::GetScopeLocation(double& outLat, double& outLong )
{
	snErr error = snNoErr;

	LStr255 latitudeString("");
	fScopeSerialCommunicator->SendSerialInfo("#:Gt#",5);
//	TScope::SleepDelay(k100MilliSeconds);
	this->GetScopeResponse(latitudeString, 10, fScopeWaitTime);

	LStr255 longitudeString("");
	fScopeSerialCommunicator->SendSerialInfo("#:Gg#",5);
//	TScope::SleepDelay(k100MilliSeconds);
	this->GetScopeResponse(longitudeString, 10, fScopeWaitTime);
	
	if ((latitudeString.Length() >= 7) && (longitudeString.Length() >= 7))
	{
		// Latitude
		//***********
		char	sign;
		SInt32	degreesLat	= 0;
		SInt32	minutesLat	= 0;
		
		LStr255 degreesLatString("");
		LStr255 minsLatString("");
		
		sign = latitudeString[1];
		degreesLatString.Assign(latitudeString, 2, 2);
		minsLatString.Assign(latitudeString, 5, 2);
		
		degreesLat	= degreesLatString;
		minutesLat	= minsLatString;
		
		if (sign == '+')
			outLat = (degreesLat + minutesLat/60.0)/180.0*kPiDefine;
		else if (sign == '-')
			outLat = -1.0*(degreesLat + minutesLat/60.0)/180.0*kPiDefine;
		else
			return pluginErr_CommunicationError; // Bail here!
		
		PluginAssert(fabs(outLat) <= kPiDefine/2.0); // otherwise I've got a problem somewhere...

		// Longitude
		//***********
		SInt32	degreesLong	= 0;
		SInt32	minutesLong	= 0;
		
		LStr255 degreesLongString("");
		LStr255 minsLongString("");
		
		degreesLongString.Assign(longitudeString, 1, 3);
		minsLongString.Assign(longitudeString, 5, 2);
		
		degreesLong	= degreesLongString;
		minutesLong	= minsLongString;
		
		if (degreesLong < 180)
			outLong = -1.0*(degreesLong + minutesLong/60.0)/180.0*kPiDefine;
		else
			outLong = (degreesLong + minutesLong/60.0)/180.0*kPiDefine;
		
		PluginAssert((outLong < kPiDefine) && (outLong >= -1.0*kPiDefine)); // otherwise I've got a problem somewhere...
		
		if ((outLong < kPiDefine) && (outLong >= -1.0*kPiDefine) && (fabs(outLat) <= kPiDefine/2.0))
			error = snNoErr;
		else
			error = pluginErr_CommunicationError;
	}
	else
		error = pluginErr_CommunicationError;

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	return error;
}


//----------------------------------------------------------------------
//	TMeadeScope::SetScopeLocation()
//
//	Purpose:	Sets the location of the scope in the scope memory
//				
//	Inputs:		inlocation	pointer to a SN PLocation.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::SetScopeLocation(const double inLat, const double inLong)
{
	if (fCanSetScopeLocation)
	{
		LStr255 setMeadeLatitudeString("");
		LStr255 setMeadeLongitudeString("");
		
		this->MakeMeadeSetLatString(inLat, setMeadeLatitudeString);
		this->MakeMeadeSetLongString(inLong, setMeadeLongitudeString);
	
		fScopeSerialCommunicator->SendSerialInfo(setMeadeLatitudeString);
	
		// check if the scope is ready to receive the Longitude string		
		LStr255 statusMessageFromScope("");
		this->GetScopeResponse(statusMessageFromScope, 2, fScopeWaitTime);
		if (statusMessageFromScope[1] == '1')
		{
			statusMessageFromScope[1] = '0';
			fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_OUT);
			fScopeSerialCommunicator->SendSerialInfo(setMeadeLongitudeString);
			this->GetScopeResponse(statusMessageFromScope, 2, fScopeWaitTime);
	
			if (statusMessageFromScope[1] == '1')
				return snNoErr;
			else
				return pluginErr_SetLocationError;
		}
		else
			return pluginErr_CommunicationError;
	}
	else
		return pluginErr_SetLocationError;
}

//----------------------------------------------------------------------
//	TMeadeScope::SetScopeTime()
//
//	Purpose:	Sets the time of the scope in its memory
//				
//	Inputs:		inTime	pointer to a SN PTime.
//				timeZoneOffsetInSeconds number of seconds offset from GMT
//				(NOTE: timeZoneOffsetInSeconds should include a DST correction if any)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::SetScopeTime(const double inJulianDate, const SInt32 inTimeZoneOffsetSeconds, const pBoolean inIsDSTInEffect)
{
	if (fCanSetScopeTime)
	{
		double secondsOffUT = inTimeZoneOffsetSeconds;
		
		if (inIsDSTInEffect)
			secondsOffUT += 3600.0;
			
		long year, month, day, hour, minute, second;
		
		double localJulianDate = inJulianDate + (secondsOffUT/3600)/24.0;
		sn_Julian2CivilTime(localJulianDate, &year, &month, &day, &hour, &minute, &second);
		
		
		LStr255 setDateString("");
		this->MakeMeadeSetDateString(year, month, day, setDateString);
		fScopeSerialCommunicator->SendSerialInfo(setDateString);
		
		LStr255 setTimeString("");
		this->MakeMeadeSetTimeString(hour, minute, second, setTimeString);
		fScopeSerialCommunicator->SendSerialInfo(setTimeString);
		
		LStr255 statusMessageFromScope("");
		this->GetScopeResponse(statusMessageFromScope, 2, fScopeWaitTime);
		
		if (statusMessageFromScope[1] == '1')
			return snNoErr; // everything A OK.
		else
			return pluginErr_SetTimeError; // there was some problem
	}
	else
		return pluginErr_SetTimeError;
}
#pragma mark ---Parking and Home---

//----------------------------------------------------------------------
//	TMeadeScope::ParkScope
//
//	Purpose:	Parks the telescope
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TMeadeScope::ParkScope(void)
{
	fScopeSerialCommunicator->SendSerialInfo("#:hP#",5);
	fIsParked = true;
	return snNoErr;
}

//----------------------------------------------------------------------
//	TMeadeScope::UnparkScope
//
//	Purpose:	Un-Parks the telescope
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TMeadeScope::UnparkScope(void)
{
	return pluginErr_FunctionNotSupported;
}

//----------------------------------------------------------------------
//	TMeadeScope::SetParkPosition
//
//	Purpose:	Sets the park position of the telescope to its currennt position
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TMeadeScope::SetParkPosition(void)
{
	return pluginErr_FunctionNotSupported;
}

//----------------------------------------------------------------------
//	TMeadeScope::MoveScopeToHomePosition
//
//	Purpose:	Moves the scope to its home position
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TMeadeScope::MoveScopeToHomePosition(void)
{
	// this does not seem to give the desired result... leave it for now
	// fScopeSerialCommunicator->SendSerialInfo("#:hF#",5);	
	return pluginErr_FunctionNotSupported;
}

#pragma mark --MeadeHelperFunctions--
//----------------------------------------------------------------------
//	TMeadeScope::MakeMeadeSetRAString()
//
//	Purpose:	Format the outstring to set Meade RA coords				
//									
//	Inputs:		inRA is a the Right Ascension in decimal radians.
//				outString will be assigned the proper pascal string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/01/15	DWW			4.0.4		Fixes for LX200 GPS
//----------------------------------------------------------------------

void TMeadeScope::MakeMeadeSetRAString(double inRA, LStr255& outString)
{	
	PluginAssert(&outString);
	PluginAssert((inRA >= 0.0) && (inRA <= 2*kPiDefine)); // Ensure that the Ra is in the Correct range
		
	inRA = TScope::ModPi2(inRA);
	
	// These are used for constructing the string output as Meade likes them:
	// HH:MM.T for low precision
	// HH:MM:SS for high precision
	SInt32 HH = 0;
	SInt32 MM = 0;
	SInt32 T  = 0;
	SInt32 SS = 0;
	
	if (!fIsHighPrecision)
		TScope::ConvertRadiansToHoursMinutesTenths(inRA, HH, MM, T);
	else
		TScope::ConvertRadiansToHoursMinutesSeconds(inRA, HH, MM, SS);
	
	// make the string
	outString.Assign(":Sr ");
	
	if (HH < 10)
	{
		outString.Append('0');
		outString.Append(HH);
	}
	else 
		outString.Append(HH);
		
	outString.Append(':');
	
	if (MM < 10)
	{
		outString.Append('0');
		outString.Append(MM);
	}
	else
		outString.Append(MM);
	
	if (!fIsHighPrecision)
	{
		outString.Append('.');
		outString.Append(T);
	}
	else
	{
		outString.Append('.');
		
		if (SS < 10)
		{
			outString.Append('0');
			outString.Append(SS);
		}
		else
			outString.Append(SS);
	}
	
	outString.Append('#');				// The Meade termination character.
}
	
//----------------------------------------------------------------------
//	TMeadeScope::MakeMeadeSetDecString()
//
//	Purpose:	Returns a string properly formatted for setting Meade Dec coords				
//									
//	Inputs:		inDec is a the Declination in decimal radians. (-Pi, Pi)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/01/15	DWW			4.0.4		Fixes for LX200 GPS
//----------------------------------------------------------------------

void TMeadeScope::MakeMeadeSetDecString(double inDec, LStr255& outString)
{
	PluginAssert(&outString);
	PluginAssert((inDec >= -90.0) && (inDec <= 90.0)); // Ensure that the Dec is in the Correct range
	
	outString.Assign(":Sd ");
	
	// Add the sign to the string, + or -. Do this before changing inDec below.
	if (inDec > 0.0)
		outString.Append('+');
	else
		outString.Append('-');
	
	// To deal with the negative side of the scale we just take the abs value and put a negative sign
	// on the string.
	if (inDec < 0.0)
		inDec = fabs(inDec);

	// Meade likes their declinations in the form:
	// sDD*MM		for low precision AUTOSTAR
	// sDD*MM.T		for low precision LX200 ?
	// sDD*MM:SS	for high precision (all)
	SInt32  DD = 0;
	SInt32  MM = 0;
//	SInt32	T  = 0;
	SInt32  SS = 0;
	
	if (fIsHighPrecision)
		TScope::ConvertRadiansToDegreesMinutesSeconds(inDec, DD, MM, SS);
	else
		TScope::ConvertRadiansToDegreesMinutes(inDec, DD, MM);
	
	PluginAssert((DD >= 0) && (MM >= 0) && (SS >= 0)); // && (T >= 0));
	PluginAssert((DD < 90) && (MM < 60) && (SS < 60)); // && (T < 10));
	
	// Build the string.
	// Append a zero if necess and add the number of degrees to the string	
	if (DD < 10)
		outString.Append('0');
	outString.Append(DD);
	
	outString.Append('*');
	
	// Append a zero if neces and add the number of minutes to the string	
	if (MM < 10)
		outString.Append('0');
	outString.Append(MM);
		
	if (fIsHighPrecision)
	{
		outString.Append(':');
		if (SS < 10)
			outString.Append('0');
		outString.Append(SS);		
	}
	else
	{
		/*
		if (fScopeType != scope_AUTOSTAR)
		{
			outString.Append('.');
			outString.Append(T);
		}
		*/
	}

	outString.Append('#');
}

//----------------------------------------------------------------------
//	TMeadeScope::SetMeadeObjectRaDec()
//
//	Purpose:	Sends the necessary strings to a Meade scope to set its 
//				object RA and DEC. (We can then Synch or Slew to the 'object')				
//									
//	Inputs:		inRA is a the Right Ascension in decimal radians.
//				inDec is the Declination in decimal radians
//				inScope is the telescope object to send the messages to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::SetMeadeObjectRaDec(double inRA, double inDec)
{
	LStr255 setRaString;
	LStr255 setDecString;
	
	this->MakeMeadeSetRAString(inRA, setRaString);
	this->MakeMeadeSetDecString(inDec, setDecString);

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo(setRaString);

	// Wait a bit to give scope time to respond
//	 TScope::SleepDelay(k100MilliSeconds);
	
	// check if the scope is ready to receive the Dec string		
	LStr255 statusMessageFromScope;
	statusMessageFromScope.Assign("\p");
	this->GetScopeResponse(statusMessageFromScope, 1, fScopeWaitTime);
	
	if (statusMessageFromScope[1] == '1')
	{
//		TScope::SleepDelay(k100MilliSeconds);
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		fScopeSerialCommunicator->SendSerialInfo(setDecString);
		
		// reset the status to false.
		statusMessageFromScope.Assign("\p");
		this->GetScopeResponse(statusMessageFromScope, 2, fScopeWaitTime);
		
		if (statusMessageFromScope[1] == '1')
			return snNoErr;
		else
			return pluginErr_SynchError;
	}
	else
		return pluginErr_SynchError;
}

//----------------------------------------------------------------------
//	TMeadeScope::MakeMeadeSetLatString()
//
//	Purpose:	Format the outString to set Meade Latitude coords				
//									
//	Inputs:		inLat is the Latitude in decimal radians (-Pi, Pi)
//				outString will be assigned the formatted pascal string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
void TMeadeScope::MakeMeadeSetLatString(double inLat, LStr255& outString)
{	
	PluginAssert(&outString);
	
	// Meade wants Lat in the form sDD*MM
	outString.Assign("#:St ");

	// First add the sign to the string, + or -.
	if ( inLat > 0.0)
		outString.Append('+');
	else
		outString.Append('-');
	
	// To Deal with the negative side of the scale
	if ( inLat < 0.0)
		inLat = fabs(inLat);
	
	// Ensure that the input is in the range
	inLat = TScope::ModPi2(inLat);
	
	SInt32 	DD = 0;
	SInt32	MM = 0;
	
	TScope::ConvertRadiansToDegreesMinutes(inLat, DD, MM);
					
	// Build the rest of the string.
	// Append a zero if neces and add the number of degrees to the string	
	if (DD < 10)
	{
		outString.Append('0');
		outString.Append(DD);
	}
	else
		outString.Append(DD);

	outString.Append('*');

	// Append a zero if neces and add the number of minutes to the string	
	if (MM <10)
	{
		outString.Append('0');
		outString.Append(MM);
	}
	else
		outString.Append(MM);

	outString.Append('#');
}

//----------------------------------------------------------------------
//	TMeadeScope::MakeMeadeSetLongString()
//
//	Purpose:	Format the outString to set Meade Latitude coords				
//									
//	Inputs:		inLong is the Longitude in decimal radians.
//				outString will be assigned the proper pascal string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
void TMeadeScope::MakeMeadeSetLongString(double inLong, LStr255& outString)
{	
	PluginAssert(&outString);
	
	// Meade likes the Longitude in DDD*MM
	SInt32 DDD 	= 0;
	SInt32 MM	= 0;
	
	// SN passes west as -ve and east as +ve but Meade wants 0-360 westward from 
	// Greenwich so need to do a bit of twiddling first...
	if(inLong < 0)
		inLong = fabs(inLong);
	else
		inLong = kTwicePi - inLong;
	
	TScope::ConvertRadiansToDegreesMinutes(inLong, DDD, MM);
	
	outString.Assign("#:Sg ");
	
	if (DDD < 100)
	{
		outString.Append('0');
		if (DDD < 10)
		{
			outString.Append('0');
			outString.Append(DDD);
		}
		outString.Append(DDD);
	}
	else
		outString.Append(DDD);
	
	outString.Append('*');
	
	if (MM < 10)
	{
		outString.Append('0');
		outString.Append(MM);
	}
	else
		outString.Append(MM);
	
	outString.Append('#');
}

//----------------------------------------------------------------------
//	TMeadeScope::MakeMeadeSetTimeString()
//
//	Purpose:	Creates a string suitable to be passed to a Meade scope
//				to set its internal time.			
//									
//	Inputs:		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/26	DWW			4.5.0		Updated for Pro 4.5
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------

void TMeadeScope::MakeMeadeSetTimeString(long inHour, long inMinute, long inSecond, LStr255& outSetTimeString)
{
	PluginAssert(&outSetTimeString);
	
	SInt32	sHours		= inHour;
	SInt32	sMinutes	= inMinute;
	SInt32	sSeconds	= inSecond;
	
	outSetTimeString.Assign("#:SL ");
	
	if (sHours < 10)
	{
		outSetTimeString.Append('0');
		outSetTimeString.Append(sHours);
	}
	else
		outSetTimeString.Append(sHours);
	
	outSetTimeString.Append(':');
	
	if (sMinutes < 10)
	{
		outSetTimeString.Append('0');
		outSetTimeString.Append(sMinutes);
	}
	else
		outSetTimeString.Append(sMinutes);
	
	outSetTimeString.Append(':');
	
	if (sSeconds < 10)
	{
		outSetTimeString.Append('0');
		outSetTimeString.Append(sSeconds);
	}
	else
		outSetTimeString.Append(sSeconds);

	outSetTimeString.Append('#');
}

//----------------------------------------------------------------------
//	TMeadeScope::inSecond()
//
//	Purpose:	Creates a string suitable to be passed to a Meade scope
//				to set its internal date.			
//									
//	Inputs:		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/05/26	DWW			4.5.0		New for Pro 4.5
//----------------------------------------------------------------------

void TMeadeScope::MakeMeadeSetDateString(long inYear, long inMonth, long inDay, LStr255& outSetDateString)
{
	PluginAssert(&outSetDateString);
	
	SInt32	sYear	= inYear;
	SInt32	sMonth	= inMonth;
	SInt32	sDay	= inDay;
	
	outSetDateString.Assign("#:SC ");
	
	if (sMonth < 10)
	{
		outSetDateString.Append('0');
		outSetDateString.Append(sMonth);
	}
	else
		outSetDateString.Append(sMonth);
	
	outSetDateString.Append('/');
	
	if (sDay < 10)
	{
		outSetDateString.Append('0');
		outSetDateString.Append(sDay);
	}
	else
		outSetDateString.Append(sDay);
	
	outSetDateString.Append('/');
	
	
	LStr255 year(sYear);
	LStr255 lastTwoYearChars;
	PluginAssert(year.Length() == 4);
	if (year.Length() == 4)
		lastTwoYearChars.Assign(year, 3, 2);
	
	if (lastTwoYearChars.Length() == 2)
		outSetDateString.Append(lastTwoYearChars);
	
	PluginAssert(lastTwoYearChars.Length() == 2);

	outSetDateString.Append('#');
}

/*
//----------------------------------------------------------------------
//	TMeadeScope::SetMeadeLocation()
//
//	Purpose:	Sends the necessary strings to a Meade scope to set its 
//				Lat and Long			
//									
//	Inputs:		inLat is a the Latitude in decimal radians.
//				inLong is the Longitude in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TMeadeScope::SetMeadeLocation(double inLat, double inLong)
{
	LStr255 setMeadeLatitudeString;
	LStr255 setMeadeLongitudeString;
	
	this->MakeMeadeSetLatString(inLat, setMeadeLatitudeString);
	this->MakeMeadeSetLongString(inLong, setMeadeLongitudeString);

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_OUT);
	fScopeSerialCommunicator->SendSerialInfo(setMeadeLatitudeString);

	// check if the scope is ready to receive the Longitude string		
	LStr255 statusMessageFromScope;
	this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);
	if (statusMessageFromScope[1] == '1')
	{
		statusMessageFromScope[1] = '0';
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_OUT);
		fScopeSerialCommunicator->SendSerialInfo(setMeadeLongitudeString);
		this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);

		if (statusMessageFromScope[1] == '1')
			return snNoErr;
		else
			return pluginErr_SetLocationError;
	}
	else
		return pluginErr_CommunicationError;
}


//----------------------------------------------------------------------
//	TMeadeScope::SetMeadeTime()
//
//	Purpose:	Sends the necessary strings to a Meade scope to set its 
//				object RA and DEC				
//									
//	Inputs:		inLat is a the Latitude in decimal radians.
//				inLong is the Longitude in decimal radians
//				inScope is the telescope object to send the messages to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TMeadeScope::SetMeadeTime(double inJulianDate, SInt32 inSecondsOffUT)
{
	LStr255 setTimeString;
	this->MakeMeadeSetTimeString(inJulianDate, inSecondsOffUT, setTimeString);
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_OUT);
	fScopeSerialCommunicator->SendSerialInfo(setTimeString);
	
	// check if the scope is ready to receive the Dec string		
	LStr255 statusMessageFromScope;
	this->GetScopeResponse(statusMessageFromScope, 2, kMeadeWaitTime);
	
	if (statusMessageFromScope[1] == '1')
		return snNoErr; // everything A OK.
	else
		return pluginErr_SetTimeError; // there was some problem
}
*/

//----------------------------------------------------------------------
//	TMeadeScope::ConvertMeadeRAToDouble()
//
//	Purpose:	Converts the RA string sent by a Meade scope to the format
//				desired by Starry Night.				
//									
//	Inputs:		inRAString is the string sent by the scope
//				outRA is the Right Ascension in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/01/15	DWW			4.0.4		Fixes for Meade GPS
//----------------------------------------------------------------------
snErr TMeadeScope::ConvertMeadeRAToDouble(LStr255 inRAString, double& outRA)
{
	PluginAssert(&outRA);
//	PluginAssert(inRAString.Length() > 0);
	
	if (inRAString.Length() == 9)
		fIsHighPrecision = true;
	else if (inRAString.Length() == 8)
		fIsHighPrecision = false;
	else
	{
//		PluginAssert(false); // This is a serious problem...
		return pluginErr_ReturnedBadValue;
	}
	
	SInt32 hours		= 0;
	SInt32 minutes		= 0;
	SInt32 tenthsOfMins	= 0;
	SInt32 seconds		= 0;
	double hrsInDecimal	= 0.0;
	
	LStr255 hoursString;
	LStr255 minsString;
	LStr255 tenthsMinsString;
	LStr255 secondsString;
	
	hoursString.Assign(inRAString, 1, 2);
	minsString.Assign(inRAString, 4, 2);
	hours	= hoursString;
	minutes	= minsString;
				
	if (fIsHighPrecision)
	{
		secondsString.Assign(inRAString, 7, 2);
		seconds = secondsString;
	
		hrsInDecimal = (hours + ((double)minutes/60) + ((double)seconds/3600));
		outRA = hrsInDecimal * kRadiansPerHour;
	}
	else
	{
		tenthsMinsString.Assign(inRAString, 7, 1);
		tenthsOfMins = tenthsMinsString;
	
		hrsInDecimal = (hours + ((double)minutes/60) + ((double)tenthsOfMins/600));
		outRA = hrsInDecimal * kRadiansPerHour;
	}
	
	if ((outRA >= 0.0) && (outRA <= 2*kPiDefine))
		return snNoErr;
	else
		return pluginErr_ReturnedBadValue;
}

//----------------------------------------------------------------------
//	TMeadeScope::ConvertMeadeDecToDouble()
//
//	Purpose:	Converts the RA string sent by a Meade scope to the format
//				desired by Starry Night.				
//									
//	Inputs:		inDecString is the string sent by the scope
//				outDec is the Declination in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

snErr TMeadeScope::ConvertMeadeDecToDouble(LStr255 inDecString, double& outDec)
{
	PluginAssert(&outDec);
//	PluginAssert(inDecString.Length() > 0);
	
	if (inDecString.Length() == 10)
		fIsHighPrecision = true;
	else if (inDecString.Length() == 7)
		fIsHighPrecision = false;
	else
	{
//		PluginAssert(false); // This is a serious problem...
		return pluginErr_ReturnedBadValue;
	}	
	
	char 	sign = inDecString[1]; // The sign of the declination
	
	SInt32 	degrees	= 0;
	SInt32 	minutes	= 0;
	SInt32 	seconds	= 0;
	double 	degreesInDecimal = 0.0;
	
	LStr255 degreesString;
	LStr255 minsString;
	LStr255 secondsString;
	
	degreesString.Assign(inDecString,2,2);
	minsString.Assign(inDecString,5,2);
	
	degrees = degreesString;
	minutes = minsString;
	
	if (fIsHighPrecision)
	{
		secondsString.Assign(inDecString,8,2);
		seconds = secondsString;
	}
	
	degreesInDecimal = degrees + ((double)minutes/60.0) + ((double)seconds/3600.0);

	if (sign == '-')
		outDec = -1.0*(degreesInDecimal * kDegreesToRadians);
	else if (sign == '+')
		outDec = degreesInDecimal * kDegreesToRadians;
	else
		return pluginErr_ReturnedBadValue;
	
	
	if ((outDec <= 90.0) && (outDec >= -90.0))
		return snNoErr;
	else
		return pluginErr_ReturnedBadValue;
}


//----------------------------------------------------------------------
//	TMeadeScope::WaitForSlewFinishedCharacterOrTimeoutPassed()
//
//	Purpose:	Queries the scope to determine if a slew is complete			
//									
//	Inputs:		NONE
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TMeadeScope::WaitForSlewFinishedCharacterOrTimeoutPassed(void)
{
	LStr255 scopeResponseChar;

	UInt32	distanceBarCharacterASCIIValue 	= 255; 	// LX200 seems to send the ASCII character 255
	UInt32	scopeResponseCharASCIIValue 	= 255;	// Initialize so the loop goes at least once
	
	if (fScopeType == scope_AUTOSTAR)
	{
		distanceBarCharacterASCIIValue 	= 127; 	// Autostar sends the "Del" (ASCII 127) character
		scopeResponseCharASCIIValue 	= 127;	// Initialize so the loop goes at least once
	}

//	TScope::SleepDelay(k500MilliSeconds);
				
	for (UInt32 i = 0; (scopeResponseCharASCIIValue == distanceBarCharacterASCIIValue); i++)
	{
		scopeResponseCharASCIIValue = 0;
		
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		fScopeSerialCommunicator->SendSerialInfo("#:D#",4);
//		TScope::SleepDelay(k100MilliSeconds);
		this->GetScopeResponse(scopeResponseChar, 2, k1KMilliSeconds);
		scopeResponseCharASCIIValue = scopeResponseChar[1];
	}
	
/*	if (fScopeType == scope_AUTOSTAR)
		TScope::SleepDelay(kAUTOSTARSettleTime); 	// The Autostars can take a lot longer to settle 
	else											// than the LX200
		TScope::SleepDelay(kLX200SettleTime); */
	
	return;	
}


//----------------------------------------------------------------------
//	TMeadeScope::CloseScopeConnection
//
//	Purpose:	Close the telescope's serial port connection
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/11/15	MK			5.7			Removed forced sync
//	03/10/16	DWW			4.5.1		Added for Autostar, since gets out of sync sometimes
//
//----------------------------------------------------------------------
snErr TMeadeScope::CloseScopeConnection(void)
{
//	fScopeSerialCommunicator->SendSerialInfo(LStr255('#'),1); // just in case we are out of sync
	return TScope::CloseScopeConnection();
}





