//----------------------------------------------------------------------
//	Copyright  1990-2001 by SPACE.com Canada, Inc., All rights reserved.
//
//	File:		NexStar11U2K.cpp
//
//	Contains:	Class for controlling NexStar 11 compatible and Ultima 2000 type telescopes
//				
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

#include "Scope.h"

// These are the standard NexStar 11 GPS, 8GPS, 114GT and U2K port communication settings
GSSerialPortInfoRecord 	portInfo_NEXSTAR11U2K =	{	
													9600,				/*BAUD RATE*/
													8, 					/*DATA BITS*/
													kGSSerialParityNone,/*PARITY*/ 
													kGSSerialStopBits10	/*STOP BITS*/
												};
												
#pragma mark ---TNexStar11U2K---
//----------------------------------------------------------------------
//	TNexStar11U2K::TNexStar11U2K()
//
//	Purpose:	Default TNexStar11U2K constructor. Setup defaults.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TNexStar11U2K::TNexStar11U2K(	TSerialCommunicator *inSerialCommunicator,
								scopeType inScopeType) : 	TScope(	false  /*inCanSynch...cant get this to work yet*/,
																	false /*inCanSetScopeTime*/,
																	false /*inCanSetScopeLocation*/,
																	inScopeType,
																	inSerialCommunicator)
{	
	switch(inScopeType)
	{
		case scope_NEXSTAR114GT:
			fScopePrettyName.Assign("\pNexStar 114GT");
			break;
		case scope_NEXSTAR8GPS:
			fScopePrettyName.Assign("\pNexStar 8 GPS");
			break;
		case scope_NEXSTAR11GPS:
			fScopePrettyName.Assign("\pNexStar 11 GPS");
			break;
		case scope_U2K:
			fScopePrettyName.Assign("\pUltima 2000");
			break;
		case scope_U2KCOMPATIBLE:
			fScopePrettyName.Assign("\pUltima 2000 Compatible");
			break;
		default:
			PluginAssert(false);
			fScopePrettyName.Assign("\pUNSUPPORTED Scope");
			break;
	}
	
	if(fScopeSerialCommunicator)
	{
		// First get the name of the port that is in use
		GSXSerialPortNameRecord		nameOfPortInUse; // This we need in the next step to set the values
		fScopeSerialCommunicator->GetPortOptions(NULL, &nameOfPortInUse);
		
		// Now set the port info for the scope in question
		fScopeSerialCommunicator->SetPortOptions(&portInfo_NEXSTAR11U2K,&nameOfPortInUse);
	}
	else
	{
		// We pass in a NULL pointer as the inSerialCommunicator when we want to check if the 
		// scope type can set time and location
	}
}

//----------------------------------------------------------------------
//	TNexStar11U2K::~TNexStar11U2K()
//
//	Purpose:	Destruct
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TNexStar11U2K::~TNexStar11U2K()	
{
}

//----------------------------------------------------------------------
//	TNexStar11U2K::CheckAlignmentComplete
//
//	Purpose:	Checks to see if the scope is connected and aligned
//									
//	Returns:	true if the scope is connected and communicating proplerly
//				false if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
pBoolean TNexStar11U2K::CheckAlignmentComplete(void)
{
	pBoolean success = false;

	LStr255 alignCompleteString('J');

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo(alignCompleteString);
	
	TScope::SleepDelay(kNS11U2KWaitTime);
	
	LStr255 scopeResponse;
	scopeResponse.Assign("\p");
	UInt32  scopeResponseLength = 3;
	this->GetScopeResponse(scopeResponse, scopeResponseLength, kNS11U2KWaitTime);
	// Subtract one since we want the second to last character (i.e. not the terminating '#')
	// why does this echo the J command?
	UInt32 scopeResponseIndex = scopeResponse.Length() - 1;
	
	if (scopeResponse[scopeResponseIndex] == 1)
		success = true;
	else
		success = false;
		
	return success;
}

//----------------------------------------------------------------------
//	TNexStar11U2K::OpenScopeConnection
//
//	Purpose:	"Opens" the connection to the scope by making sure there is a scope connected
//				to the fScopeSerialCommunicator
//									
//	Returns:	snNoErr if the scope is connected and communicating proplerly
//				pluginErr_CommunicationError if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TNexStar11U2K::OpenScopeConnection(void)
{
	snErr error = pluginErr_CommunicationError;
	
	if (this->CheckAlignmentComplete())
	{
		fIsConnected = true;
		error = snNoErr;	
	}
	else
	{
		fIsConnected = false;
		error = pluginErr_CommunicationError;
	}
	
	return error;
}

//----------------------------------------------------------------------
//	TNexStar11U2K::SynchScope
//
//	Purpose:	Synch the telescope to the passed Alt and Az, that is,
//				tells the scope where it is pointing. Useful for alignment
//									
//	Inputs:		inAz	The azimuth in decimal radians to synch on
//				inAlt	The altitude in decimal radians to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar11U2K::SynchScope(double inAzimuth, double inAltitude)
{
	snErr error = pluginErr_CommunicationError;

	if (this->NexStar11U2KIsReadyForCommand())
	{
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		TScope::SleepDelay(kNS11U2KWaitTime);
	
		LStr255 synchStringToSend;
		synchStringToSend.Assign("\p");
		this->MakeNexStar11U2KAltAzSynchString(inAzimuth, inAltitude, synchStringToSend);
		fScopeSerialCommunicator->SendSerialInfo(synchStringToSend);
		
		error = snNoErr;
	}
	else
		error = pluginErr_CommunicationError;

	return error;
}


//----------------------------------------------------------------------
//	TNexStar11U2K::SlewScope
//
//	Purpose:	Slew the telescope to the passed RA and Dec.
//									
//	Inputs:		inRA	The right ascension in decimal radians to synch on
//				inDec	The declination in decimal radians to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar11U2K::SlewScope(double inRA, double inDec)
{
	snErr error = pluginErr_CommunicationError;

	if (this->NexStar11U2KIsReadyForCommand())
	{
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		TScope::SleepDelay(kNS11U2KWaitTime);
		
		LStr255 slewStringToSend;
		slewStringToSend.Assign("\p");
		this->MakeNexStar11U2KRaDecSlewString(inRA, inDec, slewStringToSend);
		fScopeSerialCommunicator->SendSerialInfo(slewStringToSend);

		error = snNoErr;
	}
	else
		error = pluginErr_CommunicationError;

	return error;
}


//----------------------------------------------------------------------
//	TNexStar11U2K::GetScopeRaDec()
//
//	Purpose:	Determine the RA and Dec that the scope is currently pointing at.
//				
//	Inputs:		outRA		Ra of the scope
//				outDec		Dec of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar11U2K::GetScopeRaDec(double& outRA, double& outDec)
{
	snErr error = pluginErr_CommunicationError;
	
	if (this->NexStar11U2KIsReadyForCommand())
	{
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		fScopeSerialCommunicator->SendSerialInfo("E",1);
		TScope::SleepDelay(kNS11U2KWaitTime);
		
		LStr255 raDecStringToParse;
		raDecStringToParse.Assign("\p");
		this->GetScopeResponse(raDecStringToParse, 11, kNS11U2KWaitTime);
		
		if (raDecStringToParse.Length() >= 11)
		{
			this->MakeRadianCoordsFromNexStar11U2KHexString(raDecStringToParse, outRA, outDec);
			error = snNoErr;
		}
		else
			error = pluginErr_CommunicationError;
	}
	else
		error = pluginErr_CommunicationError;
		
	return error;
}

//----------------------------------------------------------------------
//	TNexStar11U2K::SetScopeLocation()
//
//	Purpose:	Sets the location of the scope in its memory
//				
//	Inputs:		inlocation	pointer to a SN PLocation.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar11U2K::SetScopeLocation(const PLocation* inLocation)
{
#pragma unused(inLocation)
	
	PluginAssert(fCanSetScopeLocation);
	
	if (fCanSetScopeLocation)
		return pluginErr_Failed;
		
	return pluginErr_Failed;
}

//----------------------------------------------------------------------
//	TNexStar11U2K::SetScopeTime()
//
//	Purpose:	Sets the time of the scope in its memory
//				
//									
//	Inputs:		inTime	pointer to a SN PTime.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar11U2K::SetScopeTime(const PTime* inTime, SInt32 timeZoneOffsetInSeconds)
{
#pragma unused(inTime)
#pragma unused(timeZoneOffsetInSeconds)
	PluginAssert(fCanSetScopeTime);

	if (fCanSetScopeTime)
		return pluginErr_Failed;

	return pluginErr_Failed;
}

#pragma mark ---NexStar11U2KHelperFunctions---
//----------------------------------------------------------------------
//	TNexStar11U2K::NexStar11U2KIsReadyForCommand()
//
//	Purpose:	Determine if the scope is ready to receive a command. 
//				Returns true if the scope is ready. Uses the "Echo" command.			
//									
//	Inputs:		inScope 	a pointer to the scope to initialize
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
pBoolean TNexStar11U2K::NexStar11U2KIsReadyForCommand(void)
{
	pBoolean success = false;

	const char echoChar = 'x';
	LStr255 echoString('K');
	echoString.Append(echoChar);
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo(echoString);
	
	TScope::SleepDelay(kNS11U2KWaitTime);
	
	LStr255 scopeResponse;
	scopeResponse.Assign("\p");
	UInt32  scopeResponseLength = 3;
	this->GetScopeResponse(scopeResponse, scopeResponseLength, kNS11U2KWaitTime);
	// Subtract one since we want the second to last character (i.e. not the terminating '#')
	UInt32 scopeResponseIndex = scopeResponse.Length() - 1;
	
	if (scopeResponse[scopeResponseIndex] == echoChar)
		success = true;
	else
		success = false;
		
	return success;
}

//----------------------------------------------------------------------
//	TNexStar11U2K::MakeNexStar11U2KAltAzSlewString()
//
//	Purpose:	Prepare a string to be sent to the scope to slew to the 
//				desired coords.				
//									
//	Inputs:		inAz is the Azimuth in decimal radians
//				inAlt is the Altitude in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::MakeNexStar11U2KAltAzSlewString(double inAlt, double inAz, LStr255& outString)
{
	PluginAssert(&outString);
	
	LStr255 azString;
	azString.Assign("\p");
	LStr255 altString;
	altString.Assign("\p");

	this->MakeNexStar11U2KHexStringFromRadianCoord(inAz, azString);
	this->MakeNexStar11U2KHexStringFromRadianCoord(inAlt, altString);
	
	// Note. Order is important. Aziumuth THEN altitude.
	outString.Assign('B');
	outString.Append(azString);
	outString.Append(',');
	outString.Append(altString);
}

//----------------------------------------------------------------------
//	TNexStar11U2K::MakeNexStar11U2KAltAzSynchString()
//
//	Purpose:	Prepare a string to be sent to the scope to synch to the 
//				desired coords.	NOTE that this IS NOT IMPLEMENTED for the NexStar 5/8 scopes			
//									
//	Inputs:		inAz is the Azimuth in decimal radians
//				inAlt is the Altitude in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::MakeNexStar11U2KAltAzSynchString(double inAz, double inAlt, LStr255& outString)
{
	PluginAssert(&outString);
	
	LStr255 azString;
	azString.Assign("\p");
	LStr255 altString;
	altString.Assign("\p");

	this->MakeNexStar11U2KHexStringFromRadianCoord(inAz, azString);
	this->MakeNexStar11U2KHexStringFromRadianCoord(inAlt, altString);
	
	// Note. Order is important. Aziumuth THEN altitude.
	outString.Assign('S');
	outString.Append(azString);
	outString.Append(',');
	outString.Append(altString);
}

//----------------------------------------------------------------------
//	TNexStar11U2K::MakeNexStar11U2KRaDecSlewString()
//
//	Purpose:	Prepare a string to be sent to the scope to slew to the 
//				desired coords.				
//									
//	Inputs:		inRa is the Right Ascension in decimal radians
//				inDec is the Declination in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::MakeNexStar11U2KRaDecSlewString(double inRa, double inDec, LStr255& outString)
{
	PluginAssert(&outString);
	
	LStr255 raString;
	raString.Assign("\p");
	LStr255 decString;
	decString.Assign("\p");

	this->MakeNexStar11U2KHexStringFromRadianCoord(inRa, raString);
	this->MakeNexStar11U2KHexStringFromRadianCoord(inDec, decString);
	
	outString.Assign('R');
	outString.Append(raString);
	outString.Append(',');
	outString.Append(decString);
}

//----------------------------------------------------------------------
//	TNexStar11U2K::MakeNexStar11U2KHexStringFromRadianCoord()
//
//	Purpose:	Convert input coordinate into a 16 bit hex number as a string.
//									
//	Inputs:		outString is the coordinate string returned form the scope
//				inCoord is the decimal radian coordinate to be converted to 16 bit hex
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::MakeNexStar11U2KHexStringFromRadianCoord(double inCoord, LStr255& outString)
{
	PluginAssert(&outString);

	if (inCoord < 0.0)
		inCoord = inCoord + kTwicePi;
	else
		inCoord = TScope::ModPi2(inCoord);

	UInt16 coordAs16Bit = (UInt16)((inCoord/(kTwicePi) * 65535.0 + 0.5)); // add 0.5 to round properly
	
	char tempCString[5];
	sprintf(tempCString, "%X", coordAs16Bit);
	
	if (coordAs16Bit > 4096)
		outString.Assign(tempCString);
	else if (coordAs16Bit < 4096 && coordAs16Bit > 255)
	{
		outString.Assign('0');
		outString.Append(tempCString);
	}	
	else if(coordAs16Bit < 256 && coordAs16Bit > 15)
	{
		outString.Assign("00");
		outString.Append(tempCString);
	}
	else
	{
		PluginAssert(coordAs16Bit < 16);
		outString.Assign("000");
		outString.Append(tempCString);
	}
}

//----------------------------------------------------------------------
//	TNexStar11U2K::MakeRadianCoordsFromNexStar11U2KHexString()
//
//	Purpose:	Convert the coordinate string sent from a Nexstar 11 (or U2k) scope to
//				decimal radian coordinates (can be used for RA Dec or Alt Az.)
//									
//	Inputs:		inString is the coordinate string returned form the scope
//				outCoordOne is the first coord (RA or Azimuth)
//				outCoordTwo  is the the second coord (Dec or Alt)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::MakeRadianCoordsFromNexStar11U2KHexString(	LStr255& inString, 
																double& outCoordOne /*RA or Az*/, 
																double& outCoordTwo /*Dec or Alt*/)
{
	PluginAssert(&inString);

	UInt32 startPositionIndex = inString.Length() - 9;
	
	LStr255 coordOneString;
	LStr255 coordTwoString;
	
	coordOneString.Assign(inString, startPositionIndex, 4);
	coordTwoString.Assign(inString, startPositionIndex + 5, 4);
	
	this->Convert16BitHexStringToRadianDouble(coordOneString, outCoordOne);
	this->Convert16BitHexStringToRadianDouble(coordTwoString, outCoordTwo);
}

//----------------------------------------------------------------------
//	TNexStar11U2K::Convert16BitHexStringToRadianDouble()
//
//	Purpose:	Convert a hexadecimal string sent from a Nexstar 11 (or U2k) scope to
//				decimal radian coordinates 
//									
//	Inputs:		inString is the coordinate string (as a 16 bit hex munber) to be  converted 
//				outCoord is the coord in decimal decimal radian.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::Convert16BitHexStringToRadianDouble(LStr255& inString, double& outCoord)
{
	PluginAssert(inString.Length() == 4);
	
	UInt8 charOne 	= inString[1];
	UInt8 charTwo 	= inString[2];
	UInt8 charThree = inString[3];
	UInt8 charFour 	= inString[4];

	double charOneAsDouble	= 0.0;
	double charTwoAsDouble	= 0.0;
	double charThreeAsDouble= 0.0;
	double charFourAsDouble	= 0.0;
	
	this->ConvertHexCharToDouble(charOne, charOneAsDouble);
	this->ConvertHexCharToDouble(charTwo, charTwoAsDouble);
	this->ConvertHexCharToDouble(charThree, charThreeAsDouble);
	this->ConvertHexCharToDouble(charFour, charFourAsDouble);
	
	if((charOneAsDouble < 0.0) || (charTwoAsDouble < 0.0) || (charThreeAsDouble < 0.0) || (charFourAsDouble < 0.0))
		PluginAssert(false);
	
	outCoord = ((	 (charOneAsDouble*4096.0)
					+(charTwoAsDouble*256.0)
					+(charThreeAsDouble*16.0)
					+charFourAsDouble)/65535.0) * kTwicePi;
}

//----------------------------------------------------------------------
//	TNexStar11U2K::Convert16BitHexStringToRadianDouble()
//
//	Purpose:	Convert a hexadecimal character to a double
//									
//	Inputs:		inChar is the char to be converted 
//				outDouble is the coord in decimal.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::ConvertHexCharToDouble(unsigned char& inChar, double& outDouble)
{
	switch(inChar)
	{
		case '0':
			outDouble = 0.0;
			break;
		case '1':
			outDouble = 1.0;
			break;
		case '2':
			outDouble = 2.0;
			break;
		case '3':
			outDouble = 3.0;
			break;
		case '4':
			outDouble = 4.0;
			break;
		case '5':
			outDouble = 5.0;
			break;
		case '6':
			outDouble = 6.0;
			break;
		case '7':
			outDouble = 7.0;
			break;
		case '8':
			outDouble = 8.0;
			break;
		case '9':
			outDouble = 9.0;
			break;
		case 'A':
			outDouble = 10.0;
			break;
		case 'B':
			outDouble = 11.0;
			break;
		case 'C':
			outDouble = 12.0;
			break;
		case 'D':
			outDouble = 13.0;
			break;
		case 'E':
			outDouble = 14.0;
			break;
		case 'F':
			outDouble = 15.0;
			break;
		default:
			outDouble = -1.0;
			break;
	}
}

//----------------------------------------------------------------------
//	TNexStar11U2K::WaitForSlewFinishedCharacterOrTimeoutPassed()
//
//	Purpose:	Returns when either the scope has sent the "slew finished" character
//
//									
//	Inputs:		NONE
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/11/16	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar11U2K::WaitForSlewFinishedCharacterOrTimeoutPassed(void)
{
	LStr255 scopeResponseChar("");	
	UInt32 	timeoutInMilliseconds = kNS11U2KSettleTime; 	// Time after zero distance bars are detected
														// we wait before slew considered complete
	
	for(UInt32 i = 0; (scopeResponseChar[1] == '1'); i++)
	{
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		
		fScopeSerialCommunicator->SendSerialInfo("L",1);
		TScope::SleepDelay(k100MilliSeconds);
		this->GetScopeResponse(scopeResponseChar, 1, timeoutInMilliseconds); 
	}
}
