//----------------------------------------------------------------------
//	Copyright  1990-2001 by SPACE.com Canada, Inc., All rights reserved.
//
//	File:		nexstar58.cpp
//
//	Contains:	Class for controlling NexStar5 and NexStar8 type telescopes
//				
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

#include "Scope.h"

// These are the standard NexStar 5 and 8 port communication settings
GSSerialPortInfoRecord 	portInfo_NEXSTAR58= {	
											9600,				/*BAUD RATE*/
											8, 					/*DATA BITS*/
											kGSSerialParityNone,/*PARITY*/ 
											kGSSerialStopBits10	/*STOP BITS*/
											};
											
#pragma mark ---NexStar58---
//----------------------------------------------------------------------
//	TNexStar58::TNexStar58()
//
//	Purpose:	Default TNexStar58 constructor. Setup defaults.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TNexStar58::TNexStar58(	TSerialCommunicator *inSerialCommunicator,
						scopeType inScopeType):	TScope(	false /*inCanSynch*/,
														false /*inCanSetScopeTime*/,
														false /*inCanSetScopeLocation*/,
														inScopeType,
														inSerialCommunicator)										

{	
	switch(inScopeType)
	{
		case scope_NEXSTAR58:
			fScopePrettyName.Assign("\pCelestron NexStar 5/8");
			break;
		default:
			fScopePrettyName.Assign("\pUNSUPPORTED Scope");
			break;
	}	
	
	if(fScopeSerialCommunicator)
	{
	// First get the name of the port that is in use
	GSXSerialPortNameRecord	nameOfPortInUse; // This we need in the next step to set the values
	fScopeSerialCommunicator->GetPortOptions(NULL,&nameOfPortInUse);
	
	// Now set the port info for the scope in question
	fScopeSerialCommunicator->SetPortOptions(&portInfo_NEXSTAR58,&nameOfPortInUse);
	}
	else
	{
		// We pass in a NULL pointer as the inSerialCommunicator when we want to check if the 
		// scope type can set time and location
	}
}

//----------------------------------------------------------------------
//	TNexStar58::~TNexStar58()
//
//	Purpose:	Destruct
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TNexStar58::~TNexStar58()	
{
}

//----------------------------------------------------------------------
//	TNexStar58::OpenScopeConnection
//
//	Purpose:	"Opens" the connection to the scope by making sure there is a scope connected
//				to the fScopeSerialCommunicator
//									
//	Returns:	snNoErr if the scope is connected and communicating proplerly
//				pluginErr_CommunicationError if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/05/15	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
snErr TNexStar58::OpenScopeConnection(void)
{
	snErr error = pluginErr_CommunicationError;
	
	double unusedRA = 0.0;
	double unusedDec = 0.0;
	
	error = this->GetScopeRaDec(unusedRA, unusedDec);
	
	if (error == snNoErr)
		fIsConnected = true;
	else
		fIsConnected = false;
		
	return error;
}

//----------------------------------------------------------------------
//	TNexStar58::SynchScope
//
//	Purpose:	Synch the telescope to the passed Ra and Dec, that is,
//				tells the scope where it is pointing. Useful for 
//									
//	Inputs:		inCoordOne	first coord to synch on (Ra or Az)
//				inCoordTwo	second coord to synch on (Ra or Az)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SynchScope(double inCoordOne, double inCoordTwo)
{
#pragma unused(inCoordOne)
#pragma unused(inCoordTwo)
	PluginAssert(fCanSynch);

	if (fCanSynch)
		return pluginErr_Failed;//These scopes do not have the ability to synch yet.

	return pluginErr_Failed;
}


/*
//----------------------------------------------------------------------
//	TNexStar58::SlewScope
//
//	Purpose:	Slew the telescope to the passed Alt and Az.
//									
//	Inputs:		inAzimuth	azimuth in decimal radians
//				inAltitude	altitude in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SlewScope(double inRA, double inDec)
{
	snErr error = pluginErr_CommunicationError;

	if (this->NexStar58CommandInitialization())
	{
		LStr255 slewStringToSend("");
		this->MakeNexStar58RaDecSlewString(inRA, inDec, slewStringToSend);
		
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
		fScopeSerialCommunicator->SendSerialInfo(slewStringToSend);
		
		if (this->WaitForSlewFinishedCharacterOrTimeoutPassed())
			error = snNoErr;
	}
	else
		error = pluginErr_CommunicationError;
	
	return error;
}

//----------------------------------------------------------------------
//	TNexStar58::SlewScope
//
//	Purpose:	Slew the telescope to the passed Alt and Az.
//									
//	Inputs:		slewGaze	gaze struct containing the Alt and Az
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SlewScope(PGaze* slewGaze)
{
	double inRA		= slewGaze->fGazeRa;
	double inDec	= slewGaze->fGazeDec;
	
	return this->SlewScope(inRA, inDec);
}

//----------------------------------------------------------------------
//	TNexStar58::SlewScope
//
//	Purpose:	Slew the telescope to the passed Alt and Az.
//									
//	Inputs:		inObject	The object to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SlewScope(const PSelection inObject)
{
	double inRA		= inObject.fRightAscension;
	double inDec	= inObject.fDeclination;
		
	return this->SlewScope(inRA, inDec);
}
*/

//----------------------------------------------------------------------
//	TNexStar58::SlewScope
//
//	Purpose:	Slew the telescope to the passed Alt and Az.
//									
//	Inputs:		inAzimuth	azimuth in decimal radians
//				inAltitude	altitude in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SlewScope(double inAzimuth, double inAltitude)
{
	snErr error = pluginErr_CommunicationError;

	if (this->NexStar58CommandInitialization())
	{
		LStr255 slewStringToSend;
		this->MakeNexStar58AltAzSlewString(inAzimuth, inAltitude, slewStringToSend);
	
		fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_IN);
		fScopeSerialCommunicator->SendSerialInfo(slewStringToSend);

		if (this->WaitForSlewFinishedCharacterOrTimeoutPassed())
			error = snNoErr;
	}
	else
		error = pluginErr_CommunicationError;
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	return error;
}

//----------------------------------------------------------------------
//	TNexStar58::GetScopeRaDec()
//
//	Purpose:	Determine the RA and Dec that the scope is currently pointing at.
//				
//	Inputs:		outRA		Ra of the scope
//				outDec		Dec of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::GetScopeRaDec(double& outRA, double& outDec)
{
	snErr error = pluginErr_CommunicationError;
	
	if (this->NexStar58CommandInitialization())
	{
		//fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_IN);
		fScopeSerialCommunicator->SendSerialInfo("E",1);
		
		// we seem to need a bit of a rest here...
		TScope::SleepDelay(k500MilliSeconds);
			
		// This kludge (length 5 instead of 4) is in place to fix the problem when the mac 
		// sends a single character command to the scope, it returns the character with the 
		// subsequent response.
		LStr255 raDecStringToParse;
		this->GetScopeResponse(raDecStringToParse, 5, kNS58WaitTime);
		
		if (raDecStringToParse.Length() >= 4)
		{
			this->MakeRadianCoordsFromNexStar58HighLowByteString(raDecStringToParse, outRA, outDec);
			error = snNoErr;
		}
		else
			error = pluginErr_CommunicationError;
	}
	else
		error = pluginErr_CommunicationError;

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	return error;
}

//----------------------------------------------------------------------
//	TNexStar58::SetScopeLocation()
//
//	Purpose:	Sets the location of the scope in the scope memory
//				
//	Inputs:		inlocation	pointer to a SN PLocation.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SetScopeLocation(const PLocation* inLocation)
{
#pragma unused(inLocation)
	PluginAssert(fCanSetScopeLocation);

	if (fCanSetScopeLocation)
		return pluginErr_Failed;// cannot set the scope location for these types yet

	return pluginErr_Failed;// cannot set the scope location for these types yet
}

//----------------------------------------------------------------------
//	TNexStar58::SetScopeTime()
//
//	Purpose:	Sets the time of the scope in its memory
//				
//	Inputs:		inTime	pointer to a SN PTime.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TNexStar58::SetScopeTime(const PTime* inTime, SInt32 timeZoneOffsetInSeconds)
{
#pragma unused(inTime)
#pragma unused(timeZoneOffsetInSeconds)
	PluginAssert(false);

	if (fCanSetScopeTime)
		return pluginErr_Failed;// cannot set the scope location for these types yet
		
	return pluginErr_Failed;// cannot set the scope location for these types yet
}

#pragma mark ---NexStar5/8HelperFunctions---
//----------------------------------------------------------------------
//	TNexStar58::NexStar58CommandInitialization()
//
//	Purpose:	Prepare the scope to receive a command. Returns true if the scope is ready. 				
//									
//	Inputs:		inScope 	a pointer to the scope to initialize
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
pBoolean TNexStar58::NexStar58CommandInitialization(void)
{
	pBoolean result = false;

	//fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	fScopeSerialCommunicator->SendSerialInfo("?",1);
	
	// The NexStar Needs a little break...
	TScope::SleepDelay(k500MilliSeconds);

	LStr255 scopeResponse;
	UInt32 maxScopeResponseLength = 2;
	this->GetScopeResponse(scopeResponse, maxScopeResponseLength, kNS58WaitTime);
	// This takes into account the possibility of any extra characters 
	// being in the buffer when the response is read
	UInt32 scopeResponseIndex = scopeResponse.Length();
	
	if (scopeResponse[scopeResponseIndex] == '#')
		result = true;
	else
		result = false;

	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);		
	return result;
}

//----------------------------------------------------------------------
//	TNexStar58::MakeNexStar58AltAzSlewString()
//
//	Purpose:	Prepare a string to be sent to the scope to slew to the 
//				desired coords.				
//									
//	Inputs:		inAz is the Azimuth in decimal radians
//				inAlt is the Altitude in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar58::MakeNexStar58AltAzSlewString(double inAz, double inAlt, LStr255& outString)
{
	PluginAssert(&outString);
	
	UInt8 azHighByte	= 0;
	UInt8 azLowByte		= 0;
	UInt8 altHighByte	= 0;
	UInt8 altLowByte	= 0;
	
	this->MakeNexStar58HighLowByteStringsFromRadianCoord(inAz, azHighByte, azLowByte);
	this->MakeNexStar58HighLowByteStringsFromRadianCoord(inAlt, altHighByte, altLowByte);

	outString.Assign('A');
	outString.Append(azHighByte);
	outString.Append(azLowByte);
	outString.Append(altHighByte);
	outString.Append(altLowByte);
}

//----------------------------------------------------------------------
//	TNexStar58::MakeNexStar58RaDecSlewString()
//
//	Purpose:	Prepare a string to be sent to the scope to slew to the 
//				desired coords.				
//									
//	Inputs:		inRA is the RightAscension in decimal radians
//				inDec is the Declination in deimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar58::MakeNexStar58RaDecSlewString(double inRA, double inDec, LStr255& outString)
{
	PluginAssert(&outString);
	
	UInt8 raHighByte	= 0;
	UInt8 raLowByte		= 0;
	UInt8 decHighByte	= 0;
	UInt8 decLowByte	= 0;
	
	this->MakeNexStar58HighLowByteStringsFromRadianCoord(inRA, raHighByte, raLowByte);
	this->MakeNexStar58HighLowByteStringsFromRadianCoord(inDec, decHighByte, decLowByte);

	outString.Assign('R');
	outString.Append(raHighByte);
	outString.Append(raLowByte);
	outString.Append(decHighByte);
	outString.Append(decLowByte);
}


//----------------------------------------------------------------------
//	TNexStar58::MakeNexStar58HighLowByteStringsFromRadianCoord()
//
//	Purpose:	Convert the coordinate (either RA, Dec, Alt or Az) to the high and low bytes of a 16 bit
//				number 				
//									
//	Inputs:		inCoord is the coordinate in decimal radians to be parsed
//				outHigh is the the "high" byte of the coordinate as a 16 bit number
//				outLow  is the the "low" byte of the coordinate as a 16 bit number
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

void TNexStar58::MakeNexStar58HighLowByteStringsFromRadianCoord(double inCoord, UInt8& outHigh, UInt8& outLow)
{
	if ((&outHigh == NULL) || (&outLow == NULL))
		PluginAssert(false);
	
	// Celestron takes negatives as their complement. e.g. (-29.5 degrees -> 330.5 degrees) (or e.g. -Pi/4 -> 7Pi/4)
	// This will only occur if a negative declination or altitude is passed as inCoord
	if (inCoord < 0.0)
		inCoord = inCoord + kTwicePi;
	
	// Ensure that the input is in the correct range.
	//inCoord = TScope::ModPi2(inCoord);
		
	// The pointing resolution of the NexStar 5 and 8 is 64800 per 2Pi radians so we must round
	// the the nearest 64800/2Pi then scale up to 65536. Add the 0.5 to round to the nearest int properly
	double inCoordAs16Bit = 0.0;
	
	inCoordAs16Bit = floor((inCoord/(2.0*kPiDefine) * kNS58PointingRes) + 0.5) * (65536.0/kNS58PointingRes);

	// Add the 0.5 to round to the nearest int properly.
	outHigh = (UInt8) ((inCoordAs16Bit/256.0) + 0.5);
	outLow	= (UInt8) (fmod(inCoordAs16Bit,256.0) + 0.5);
}


//----------------------------------------------------------------------
//	TNexStar58::MakeRadianCoordsFromNexStar58HighLowByteString()
//
//	Purpose:	Convert the high, low byte strings sent from a Celestron scope to
//				decimal radian coordinates (can be used for RA Dec or Alt Az.)
//				Be sure to check with manual which is which!
//									
//	Inputs:		inString is the coordinate string returned from the scope
//				outCoordOne is the the combo of the first "high" and "low" bytes of inString in decimal radians
//				outCoordTwo  is the the combo of the second "high" and "low" bytes of inString in decimal radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TNexStar58::MakeRadianCoordsFromNexStar58HighLowByteString(LStr255& inString, 
													double& outCoordOne, /*RA or Az*/
													double& outCoordTwo) /*Dec or Alt*/
{	
	if ((&inString == NULL) || (&outCoordOne == NULL) || (&outCoordTwo == NULL))
		PluginAssert(false);
	
	UInt8 highOne = 0;
	UInt8 lowOne  = 0;
	UInt8 highTwo = 0;
	UInt8 lowTwo  = 0;

	// We know that the returned string should be four bytes long so the start position is four
	// bytes back from the end. (Avoids the Mac "extra-character" problem)
	// Note that it is -3 (NOT -4) since Length returns the length of the Pascal string (i.e. 1 less than
	// the length of the 'c' string (and since we want an index)
	UInt32 startPositionIndex = inString.Length() - 3;
	
	highOne = inString[startPositionIndex];
	lowOne 	= inString[startPositionIndex +1];
	highTwo	= inString[startPositionIndex +2];
	lowTwo  = inString[startPositionIndex +3];

	double highOneAsDouble 	= highOne;
	double lowOneAsDouble 	= lowOne;
	double highTwoAsDouble 	= highTwo;
	double lowTwoAsDouble 	= lowTwo;
		
	double tempCoordTwoPosNeg = ((highTwoAsDouble*256.0 + lowTwoAsDouble)/65536.0) * kTwicePi;
	if (tempCoordTwoPosNeg > kHalfPi)
	{
		if (tempCoordTwoPosNeg < kThreeHalfPi)
		{
			PluginAssert(tempCoordTwoPosNeg > kThreeHalfPi);
			// We know there is an error since neither Dec nor Alt should come back from celestron
			// in the range (kHalfPi, kThreeHalfPi). Deal with this...?
		}
		else
		{
			// Turn the value into a negative number by subtracting 2 Pi.
			tempCoordTwoPosNeg = tempCoordTwoPosNeg - kTwicePi;			
		}
	}
	
	outCoordOne = ((highOneAsDouble*256.0 + lowOneAsDouble)/65536.0) * kTwicePi;
	outCoordTwo = tempCoordTwoPosNeg;
}

//----------------------------------------------------------------------
//	TNexStar58::WaitForSlewFinishedCharacterOrTimeoutPassed()
//
//	Purpose:	Returns when either the scope has sent the "slew finished" character
//				(i.e. '@') or the timeout has passed.
//
//				***NOTE*** 	Sometimes the scope does not return the @ character at all when
//							the slew is complete, hence the timeout.
//									
//	Inputs:		NONE
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/11/16	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
pBoolean TNexStar58::WaitForSlewFinishedCharacterOrTimeoutPassed(void)
{
	pBoolean success = true; // default this to true until celestron fixes their own problems

	LStr255	scopeResponseChar;
	UInt32	maxNumberOfCharsToGet = 9;
	UInt32	timeoutInMilliseconds = kNS58MaxSlewTime;
	
	for(UInt32 i = 0; (i < maxNumberOfCharsToGet) && (scopeResponseChar[1] != '@'); i++)
	{
		this->GetScopeResponse(scopeResponseChar, 1, timeoutInMilliseconds);
	}
	
	return success;	
}