//----------------------------------------------------------------------
//	Copyright © 1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		Orion.cpp
//
//	Contains:	Class for controlling Orion type scopes
//				
//	Authors:	Marko Kudjerski
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/12	MK			5.7			Added support for scopes that use an interface
//											similar to Orion's
//	04/06/04	MK			5.0			Added support
//
//----------------------------------------------------------------------

#include "Scope.h"

// These are the default Orion telescope port communication settings
GSSerialPortInfoRecord 	portInfo_OrionDefault =	{	
														9600,				/*BAUD RATE*/
														8, 					/*DATA BITS*/
														kGSSerialParityNone,/*PARITY*/ 
														kGSSerialStopBits10	/*STOP BITS*/
													};
												
#pragma mark ---TOrionScope---
//----------------------------------------------------------------------
//	TOrionScope::TOrionScope()
//
//	Purpose:	Default TOrionScope constructor. Setup defaults.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/04	MK			5.0		Added support
//
//----------------------------------------------------------------------
TOrionScope::TOrionScope(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType) : TScope(inSerialCommunicator, inScopeType)
{
	fMustAlignInSoftware	= true;
	fScopeEncoderResolution = kOrionEncoderResolution;
	
	fCanSynch				= false;				
	fCanSlew				= false;
	fCanSetScopeTime		= false;
	fCanSetScopeLocation 	= false;
	fCanDirSlew				= false;
	fScopeWaitTime			= kNS11U2KWaitTime;

	fCanPark				= false;					
	fCanUnpark				= false;
	fCanSetPark				= false;
	fCanGoHome				= false;	

	switch (inScopeType) {
		case scope_NGCMAX:
			fScopePrettyName.Assign("NGCMAX");
			break;
		case scope_LUMICON:
			fScopePrettyName.Assign("Lumicon Sky Vector");
			break;
		case scope_DSTARFINDER:
			fScopePrettyName.Assign("Discovery Digital Star Finder");
			break;			
		case scope_ORIONSKYQUEST:
			fScopePrettyName.Assign("Orion Skyquest IntelliScope");
			break;
		default:
			break;
	}
	
	if (inSerialCommunicator)
	{
		// First get the name of the port that is in use
		GSXSerialPortNameRecord		nameOfPortInUse; // This we need in the next step to set the values
		fScopeSerialCommunicator->GetPortOptions(NULL, &nameOfPortInUse);
		
		// Now set the port info for the scope in question
		fScopeSerialCommunicator->SetPortOptions(&portInfo_OrionDefault, &nameOfPortInUse);
	}
	else
	{
		// We passed in a NULL pointer as the inSerialCommunicator since we only want to check if the 
		// scope type can set time and location.
	}
}

//----------------------------------------------------------------------
//	TOrionScope::~TOrionScope()
//
//	Purpose:	Destruct
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/04	MK			5.0		Added support
//
//----------------------------------------------------------------------

TOrionScope::~TOrionScope()	
{
}

//----------------------------------------------------------------------
//	TOrionScope::OpenScopeConnection
//
//	Purpose:	"Opens" the connection to the scope by making sure there is a scope connected
//				to the fScopeSerialCommunicator
//									
//	Returns:	snNoErr if the scope is connected and communicating proplerly
//				pluginErr_CommunicationError if there was an error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/04	MK			5.0			Added support
//----------------------------------------------------------------------
snErr TOrionScope::OpenScopeConnection(void)
{	
	if (this->IsReadyForCommand())
	{
		fIsConnected = true;
	}
	else
	{
		fIsConnected = false;
		return pluginErr_CommunicationError;
	}
		
	return snNoErr;
}

//----------------------------------------------------------------------
//	TOrionScope::SlewScope
//
//	Purpose:	Slew the telescope to the passed RA and Dec.
//				Orion Skyquest does not support this feature.
//									
//	Inputs:		inRA	The right ascension in decimal radians to synch on
//				inDec	The declination in decimal radians to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/04	MK			5.0			Added support
//
//----------------------------------------------------------------------
snErr TOrionScope::SlewScope(double inRA, double inDec)
{
	return pluginErr_FunctionNotSupported;
}

//----------------------------------------------------------------------
//	TOrionScope::GetScopeAzAlt()
//
//	Purpose:	Determine the RA and Dec that the scope is currently pointing at.
//				
//	Inputs:		outAz		Az of the scope
//				outAlt		Alt of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/04	MK			5.02		Added support for Software Alignment
//
//----------------------------------------------------------------------
snErr TOrionScope::GetScopeAzAlt(double& outAz, double& outAlt)
{
	char* remainder;
	LStr255 inString;
	inString.Assign("");
	UInt8 coordstart;
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);
	
	fScopeSerialCommunicator->SendSerialInfo("Q",1);
	
	//value retrieved should come in the format "Q+01234\t+01234\r" or "+01234\t+01234\r"
	this->GetScopeResponse(inString, 15, fScopeWaitTime);
	if (inString.Length() < 14)
		return pluginErr_CommunicationError;
	
	//However, sometimes \r shows up at the beginning and not at the end
	//so start counting from Q
	coordstart = inString.Find(LStr255("\r"));
	if (coordstart != 1)
		coordstart = 1;
	else
		coordstart = 2;
		
	//if Q is not there then roll back in position count
	if (inString(coordstart,1) != LStr255("Q"))
		coordstart--;
	
	//Extract Azimuth
	outAz=strtod(LStr255::PToCStr(inString(coordstart+1,6)),&remainder);	
	if (*remainder != '\0')
		return pluginErr_CommunicationError;
		
	//Extract altitude
	outAlt=strtod(LStr255::PToCStr(inString(coordstart+8,6)),&remainder);
	if (*remainder != '\0')
		return pluginErr_CommunicationError;
		
	outAlt *= kTwicePi/fScopeEncoderResolution;
	outAz *= kTwicePi/fScopeEncoderResolution;
	
	return snNoErr;
}


//----------------------------------------------------------------------
//	TOrionScope::GetScopeRaDec()
//
//	Purpose:	Determine the RA and Dec that the scope is currently pointing at.
//				
//	Inputs:		outRA		Ra of the scope
//				outDec		Dec of the scope
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/04	MK			5.0			Added support
//
//----------------------------------------------------------------------
snErr TOrionScope::GetScopeRaDec(double& outRA, double& outDec)
{
	double inAlt,inAz;

	if (GetScopeAzAlt(inAz, inAlt) != snNoErr)
		return pluginErr_CommunicationError;
	else
		//use this instead of sn_CoordinateConversion <- causes problems
		ConvertAltAzNowToRaDecJNow(inAlt,inAz,outRA, outDec);
	
	return snNoErr;
}

#pragma mark ---OrionHelperFunctions---
//----------------------------------------------------------------------
//	TOrionScope::IsReadyForCommand()
//
//	Purpose:	Determine if the scope is ready to receive a command. 
//				Returns true if the scope is ready. Uses the "P" command.			
//									
//	Inputs:		inScope 	a pointer to the scope to initialize
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/07	MK			5.0			Added support
//
//----------------------------------------------------------------------
pBoolean TOrionScope::IsReadyForCommand(void)
{
	pBoolean isReady = false;
	LStr255 scopeResponse("");
	
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);

	LStr255 echoString('P');
	fScopeSerialCommunicator->SendSerialInfo(echoString);
	
	this->GetScopeResponse(scopeResponse, 4, fScopeWaitTime);
	
	//Depending on the scope the response might be any of P001, 001 and ?
	if (scopeResponse == LStr255("P001"))
		isReady = true;
	else if (scopeResponse == LStr255("?"))
		isReady = true;
	else if (scopeResponse == LStr255("001"))
		isReady = true;
	else
		isReady = false;
			
	fScopeSerialCommunicator->PurgeFlushSerialBuffer(buffer_INandOUT);		
		
	return isReady;
}
