//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		Scope.cpp
//
//	Contains:	Abstract base class for controlling robotic telescopes
//				
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/09/01	MK			5.7			Added support for 3 more scopes based on Orion
//											interface
//  04/06/24	MK			5.0			Added Alt/Az to Ra/Dec conversion
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
 
#include "Scope.h"

//----------------------------------------------------------------------
//	TScope::MakeNewTScope
//
//	Purpose:	Create a telescope object based on the scope type
//				chosen by the user in the configuration.
//
//	Inputs:		inSerialCommunicator, the serial communicator already setup for communications
//				inScopeType, the scope type (as chosen from a list by the user)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TScope* TScope::MakeNewTScope( 	TSerialCommunicator *inSerialCommunicator,
								scopeType inScopeType)	
{
	switch (inScopeType)
	{
		case scope_LX200:
		case scope_LX200GPS:
		case scope_GENERICLX200:
		case scope_AUTOSTAR:
		case scope_ASTROPGERMAN:
			return new TMeadeScope(inSerialCommunicator, inScopeType);
			break;
		case scope_NEXSTAR58:
		case scope_ORIONATLAS:
			return new TNexStar58(inSerialCommunicator, inScopeType);
			break;
		case scope_NEXSTAR6080114:
		case scope_NEXSTAR4GT:
		case scope_NEXSTAR5i8i:
		case scope_NEXSTAR811GPS:
		case scope_CELESTCGE:
		case scope_CELESTASC:
		//case scope_U2K:
		//case scope_U2KCOMPATIBLE:
			return new TCelestronScope(inSerialCommunicator, inScopeType);
			break;
		case scope_NGCMAX:
		case scope_LUMICON:
		case scope_DSTARFINDER:
		case scope_ORIONSKYQUEST:
			return new TOrionScope(inSerialCommunicator, inScopeType);
			break;
		default:
			PluginAssert(false); // Scope type not handled or passed scope_NONE...which is bad too.
			return NULL;
			break;
	}
	
	return NULL;
}

//----------------------------------------------------------------------
//	TScope::GetNumberOfSupportedScopes
//
//	Purpose:	Returns the number of supported scope types to list in the
//				pulldown scope menu
//
//	Inputs:		none
//
//	Note:		kNumberOfScopeTypesSupported is defined in ScopeControllerConstants.h
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
const UInt32 TScope::GetNumberOfSupportedScopes(void)
{
	return kNumberOfScopeTypesSupported;
}

//----------------------------------------------------------------------
//	TScope::GetPointerToFirstScopeNameString
//
//	Purpose:	Returns a pointer to the first LString in the scope name
//				array. Should always be be kScopeName_NONE (== 0).
//
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
const LStr255* TScope::GetPointerToFirstScopeNameString(void)
{
	return &(kScopeNameArrary[scope_NONE]);
}

//----------------------------------------------------------------------
//	TScope::GetPointerToScopeNameString
//
//	Purpose:	Returns a pointer to the scope name corresponding to the
//				input scope type.
//
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
const LStr255* TScope::GetPointerToScopeNameString(scopeType inScopeType)
{
	PluginAssert(inScopeType);
	return &(kScopeNameArrary[inScopeType]);
}



//----------------------------------------------------------------------
//	TScope::DoesScopeTypeSupportSetTimeAndLocation
//
//	Inputs:		inScopeType the scope type to check.
//
//	Returns:	true if we can set the internal time and location
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
pBoolean TScope::DoesScopeTypeSupportSetTimeAndLocation(scopeType inScopeType)
{
	pBoolean result = false;
	TScope* tempScope = NULL;
	
	tempScope = TScope::MakeNewTScope(NULL, inScopeType);
	
	result = (tempScope->GetCanSetScopeTime() && tempScope->GetCanSetScopeLocation());
	
	delete tempScope;
	tempScope = NULL;
	
	return result;
}

#pragma mark ---GeneralHelperFunctions---
//----------------------------------------------------------------------
//	TScope::ModPi2()
//
//	Purpose:	Reduce an angle to the interval (0, 2Pi). 			
//									
//	Inputs:		x is the radian angle to be reduced
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		Nicked from SN "funmath.cpp"
//
//----------------------------------------------------------------------
long double  TScope::ModPi2(long double x)
{
	return (x - floor(x * kOneOverTwicePi) * kTwicePi);
}

//----------------------------------------------------------------------
//	TScope::SleepDelay
//
//	Purpose:	Will block the program for the passed number of milliseconds
//				use to wait in case something is going too fast, etc
//				Used mainly to give time for a scope to respond to a passed command.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/06/25	Tom			4.0.0		Pilfered from Tully by DW for Telescope
//
//----------------------------------------------------------------------
void TScope::SleepDelay(double inMillsecondsToSleepFor)
{
#if TARGET_OS_WIN32
	::Sleep(inMillsecondsToSleepFor);
#else
	UInt32 ticksToSleepFor = (UInt32) (inMillsecondsToSleepFor/1000.0*kTicksPerSecond);
	UInt32 finalTicks;
	::Delay(ticksToSleepFor, &finalTicks);
#endif
}

//----------------------------------------------------------------------
//	TScope::ConvertRadiansToHoursMinutesTenths()
//
//	Purpose:	Converts a radian ordinate into hours, minutes and tenths of minutes		
//									
//	Inputs:		inOrdinate 	is the ordinate in decimal radians to be parsed
//				outHours	is the hours portion of the ordinate
//				outMinutes	is the minutes portion of the ordinate
//				outTenthsOfMinutes	is the tenths of minutes portion of the ordinate
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
void TScope::ConvertRadiansToHoursMinutesTenths(double inOrdinate, 
												SInt32& outHours, 
												SInt32& outMinutes, 
												SInt32& outTenthsOfMinutes)
{
	double ordinateInDecimalHours	= 0.0;	// The input ordinate in decimal hours
	double ordinateHours 			= 0.0;	// Integer part of ordinateInDecimalHours
	double fractionalHours 			= 0.0;	// Fractional part of ordinateInDecimalHours
	
	double ordinateDecimalMinutes 	= 0.0;	// Minutes of ordinate in Decimal
	double ordinateMinutes 			= 0.0;	// Integer part of ordinateDecimalMinutes
	double fractionalMinutes		= 0.0;	// Fractional part of ordinateDecimalMinutes
	
	double ordinateTenthsOfMinutesDecimal = 0.0;	// Tenths of Minutes in decimal
	
	ordinateInDecimalHours = inOrdinate / kRadiansPerHour;
	fractionalHours = modf(ordinateInDecimalHours, &ordinateHours);
	outHours = (SInt32)ordinateHours;
	
	ordinateDecimalMinutes = fractionalHours * kMinsPerHour;
	fractionalMinutes = modf(ordinateDecimalMinutes, &ordinateMinutes);
	outMinutes = (SInt32)ordinateMinutes;

	ordinateTenthsOfMinutesDecimal = fractionalMinutes * 10;
	outTenthsOfMinutes = (SInt32)(ordinateTenthsOfMinutesDecimal + 0.5);// The integer assignment implicitly rounds 
																// down so we round up by adding 0.5
	
	// Take care of the case that rounding up takes the value past its max.
	if ( outTenthsOfMinutes == 10)
	{
		outTenthsOfMinutes = 0;
		outMinutes++;
		if (outMinutes == 60)
		{
			outMinutes = 0;
			outHours++;
		}
	}
	
}

//----------------------------------------------------------------------
//	TScope::ConvertRadiansToHoursMinutesSeconds()
//
//	Purpose:	Converts a radian ordinate into hours, minutes and seconds		
//									
//	Inputs:		inOrdinate 	is the ordinate in decimal radians to be parsed
//				outHours	is the hours portion of the ordinate
//				outMinutes	is the minutes portion of the ordinate
//				outSeconds	is the seconds portion of the ordinate
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	03/01/15	DWW			4.0.4		Fixes for LX200GPS
//----------------------------------------------------------------------
void TScope::ConvertRadiansToHoursMinutesSeconds(	double inOrdinate, 
													SInt32& outHours, 
													SInt32& outMinutes, 
													SInt32& outSeconds)
{
	PluginAssert((inOrdinate >= 0.0) && (inOrdinate <= 2.0*kPiDefine));
	inOrdinate = TScope::ModPi2(inOrdinate);

	double ordinateInDecimalHours	= inOrdinate / kRadiansPerHour;
	double ordinateHours 			= 0.0;	// Integer part of ordinateInDecimalHours
	double fractionalHours			= modf(ordinateInDecimalHours, &ordinateHours);
	
	double ordinateDecimalMinutes	= fractionalHours * kMinsPerHour;
	double ordinateMinutes 			= 0.0;	// Integer part of ordinateDecimalMinutes
	double fractionalMinutes		= modf(ordinateDecimalMinutes, &ordinateMinutes);

	double ordinateDecimalSeconds	= fractionalMinutes * kSecondsPerMinute;
	double ordinateSeconds 			= 0.0;	// Integer part of ordinateDecimalSeconds
	double fractionalSeconds		= modf(ordinateDecimalSeconds, &ordinateSeconds);

	// Round up
	fractionalSeconds += 0.5;
	if (fractionalSeconds >= 1.0)
	{
		ordinateSeconds += 1;
		if (ordinateSeconds >= kSecondsPerMinute)
		{
			ordinateSeconds = 0;
			ordinateMinutes += 1;
			
			if (ordinateMinutes >= kMinsPerHour)
			{
				ordinateMinutes = 0;
				ordinateHours += 1;
				if (ordinateHours == 24.0)
					ordinateHours = 0.0;
				PluginAssert((ordinateHours > 0.0) && (ordinateHours < 24.0));
			}
			
		}
	
	}
	
	outHours	= (SInt32)ordinateHours;
	outMinutes	= (SInt32)ordinateMinutes;
	outSeconds	= (SInt32)ordinateSeconds;
}

//----------------------------------------------------------------------
//	TScope::ConvertRadiansToDegreesMinutes()
//
//	Purpose:	Converts a radian ordinate into degrees and minutes		
//									
//	Inputs:		inOrdinate 	is the ordinate in decimal radians to be parsed
//				outDegrees	is the degrees portion of the ordinate
//				outMinutes	is the minutes portion of the ordinate
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TScope::ConvertRadiansToDegreesMinutes(double inOrdinate, 
											SInt32& outDegrees, 
											SInt32& outMinutes)
{
	double ordinateInDecimalDegrees = 0.0;	// The input ordinate in decimal degrees
	double ordinateDegrees			= 0.0;	// Integer part of ordinateInDecimalDegrees
	double fractionalDegrees		= 0.0;	// Fractional part of ordinateInDecimalDegrees
	double ordinateDecimalMinutes	= 0.0;	// Minutes of ordinate in decimal

	ordinateInDecimalDegrees = inOrdinate / kDegreesToRadians;
	fractionalDegrees = modf(ordinateInDecimalDegrees, &ordinateDegrees);
	outDegrees = (SInt32)ordinateDegrees;


	ordinateDecimalMinutes = fractionalDegrees * kMinsPerDegree;
	// add 0.5 so that we round up with the implicit conversion
	outMinutes = (SInt32)(ordinateDecimalMinutes +0.5);
	// increment the minutes if they reached >60 during the rounding up 
	if (outMinutes == 60)
	{
		outMinutes = 0;
		outDegrees++;
	}
}

//----------------------------------------------------------------------
//	TScope::ConvertRadiansToDegreesMinutesSeconds()
//
//	Purpose:	Converts a radian ordinate into degrees and minutes		
//									
//	Inputs:		inOrdinate 	is the ordinate in decimal radians to be parsed
//				outDegrees	is the degrees portion of the ordinate
//				outMinutes	is the minutes portion of the ordinate
//				outSeconds	is the seconds portion of the ordinate
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
void TScope::ConvertRadiansToDegreesMinutesSeconds(	double inOrdinate, 
													SInt32& outDegrees, 
													SInt32& outMinutes,
													SInt32& outSeconds)
{
	double ordinateInDecimalDegrees = inOrdinate * kRadiansToDegrees;
	double ordinateDegrees			= 0.0;	// Integer part of ordinateInDecimalDegrees
	double fractionalDegrees 		= modf(ordinateInDecimalDegrees, &ordinateDegrees);

	double ordinateDecimalMinutes	= fractionalDegrees * kMinsPerDegree;
	double ordinateMinutes 			= 0.0;	// Integer part of ordinateDecimalMinutes
	double fractionalMinutes		= modf(ordinateDecimalMinutes, &ordinateMinutes);

	double ordinateDecimalSeconds	= fractionalMinutes * kSecondsPerMinute;
	double ordinateSeconds 			= 0.0;	// Integer part of ordinateDecimalSeconds
	double fractionalSeconds		= modf(ordinateDecimalSeconds, &ordinateSeconds);

	// Round up
	fractionalSeconds += 0.5;
	if (fractionalSeconds >= 1.0)
	{
		ordinateSeconds += 1;
		if (ordinateSeconds >= kSecondsPerMinute)
		{
			ordinateSeconds = 0;
			ordinateMinutes += 1;
			
			if (ordinateMinutes >= kMinsPerDegree)
			{
				ordinateMinutes = 0;
				ordinateDegrees += 1;
				if (ordinateDegrees >= 360.0)
					ordinateDegrees = 0.0;
				PluginAssert((ordinateDegrees > 0.0) && (ordinateDegrees < 360.0));
			}
			
		}
	
	}
	
	outDegrees	= (SInt32)ordinateDegrees;
	outMinutes	= (SInt32)ordinateMinutes;
	outSeconds	= (SInt32)ordinateSeconds;
}

//----------------------------------------------------------------------
//	TScope::ConvertAltAzNowToRaDecJNow()
//
//	Purpose:	Converts a Alt/Az to Ra/Dec for the front window
//									
//	Inputs:		inAlt 	is the Altitude in radians
//				inAz	is the Azimuth in radians
//				outRa	is the output RA
//				outDec	is the output Dec
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/16	MK			5.0			Added due to issues with sn_CoordinateConversion
//
//----------------------------------------------------------------------
void TScope::ConvertAltAzNowToRaDecJNow( double inAlt,
										 double inAz,
										 double& outRa, 
										 double& outDec)
{
	double lat, lon, hA, JD;
	double dummy1;
	long dummy2;
	pBoolean dummy3;

	sn_GetViewerLocationInfo(kFrontWindow,&lat,&lon, &dummy1,&dummy2,&dummy3);

	//equations below treat Lon W as positive while SN takes East as negative
	// it's necessary to correct for that.
	lon = kTwicePi - lon;

	//equations below measure azimuth clockwise from the south
	//so inAz needs to be modified
	inAz += kPiDefine;
	sn_RealWorldAzimuth2Mathematical(inAz, &inAz);

	outDec = asin(sin(lat)*sin(inAlt) - cos(lat)*cos(inAlt)*cos(inAz));

	//Hour Angle
	hA	= acos((sin(inAlt)-sin(lat)*sin(outDec))/(cos(lat)*cos(outDec)));

	if (sin(inAz) > 0)
		hA = kTwicePi - hA;

	//Sidereal time (in degrees)
	sn_GetCurrentTime(kFrontWindow,&JD);
	dummy1 = (JD-2451545.0)/36525.0;
	dummy1 = 280.46061837 + 360.98564736629*(JD - 2451545.0) + dummy1*(0.000387933*dummy1 - dummy1*dummy1/38710000.0);

	outRa = dummy1/(360.0)*kTwicePi - lon - hA;

}

//----------------------------------------------------------------------
//	TScope::ConvertRaDecJNowToAltAzNow()
//
//	Purpose:	Converts a Ra/Dec to Alt/Az for the front window
//									
//	Inputs:		inRa	is the input RA in radians
//				inDec	is the input Dec in radians
//				outAlt 	is the Altitude in radians
//				outAz	is the Azimuth in radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/06/30	MK			5.0			Added due to issues with sn_CoordinateConversion
//
//----------------------------------------------------------------------
void TScope::ConvertRaDecJNowToAltAzNow( double inRa,
										 double inDec,
										 double& outAlt, 
										 double& outAz)
{
	double lat, lon, hA, JD;
	double dummy1;
	long dummy2;
	pBoolean dummy3;

	sn_GetViewerLocationInfo(kFrontWindow,&lat,&lon, &dummy1,&dummy2,&dummy3);

	//equations below treat Lon W as positive while SN takes East as negative
	// it's necessary to correct for that.
	lon = kTwicePi - lon;

	//Sidereal time (in degrees)
	sn_GetCurrentTime(kFrontWindow,&JD);
	dummy1 = (JD-2451545.0)/36525.0;
	dummy1 = 280.46061837 + 360.98564736629*(JD - 2451545.0) + dummy1*(0.000387933*dummy1 - dummy1*dummy1/38710000.0);

	//hour angle
	hA = dummy1/(360.0)*kTwicePi - lon - inRa;
	
	outAlt = asin(sin(lat)*sin(inDec) + cos(lat)*cos(inDec)*cos(hA));
	outAz = acos((sin(inDec)-sin(outAlt)*sin(lat))/(cos(outAlt)*cos(lat)));
	
	//this takes into account both arccos problems and Mathematical2RealWorld conversion
	if (sin(hA) > 0)
		outAz = kTwicePi - outAz;	
}

#pragma mark ---Constructors---
//----------------------------------------------------------------------
//	TScope::TScope
//
//	Purpose:	Default TScope constructor
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TScope::TScope(TSerialCommunicator* inScopeSerialCommunicator, scopeType inScopeType):	fIsConnected(false),
																						fIsParked(false),
																						fMustAlignInSoftware(false),
																						fCanSynch(false),
																						fCanSlew(false),					
																						fCanSetScopeTime(false),
																						fCanSetScopeLocation(false),
																						fCanDirSlew(false),
																						fCanSetSlewRate(false),
																						fCanSetSlewDirRate(false),
																						fCanPark(false),					
																						fCanUnpark(false),
																						fCanSetPark(false),
																						fCanGoHome(false),	
																						fSerialCommandEchoed(false),
																						fScopeType(inScopeType),
																						fScopeWaitTime(k500MilliSeconds),
																						fScopeEncoderResolution(0),
																						fScopeSerialCommunicator(inScopeSerialCommunicator)
{
	fScopePrettyName.Assign("");
}

//----------------------------------------------------------------------
//	TScope::~TScope
//
//	Purpose:	destruct 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TScope::~TScope(void)
{
}

#pragma mark ---Movement---

//----------------------------------------------------------------------
//	TScope::SynchScope
//
//	Purpose:	Synch the telescope to the passed Ra and Dec, that is,
//				"tells" the scope where it is pointing. Useful for alignment.
//									
//	Inputs:		synchGaze	the gaze vector to synch to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/05/05	DWW			4.5.0		Moved this to TScope
//----------------------------------------------------------------------
snErr TScope::SynchScope(PGaze* synchGaze)
{
	double inRA		= synchGaze->fGazeRa;
	double inDec	= synchGaze->fGazeDec;
	
	return this->SynchScope(inRA, inDec);
}

//----------------------------------------------------------------------
//	TScope::SynchScope
//
//	Purpose:	Synch the telescope to the passed Ra and Dec, that is,
//				"tells" the scope where it is pointing. Useful for alignment.
//									
//	Inputs:		inObject	the SN object to synch on
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/05/05	DWW			4.5.0		Moved this to TScope
//----------------------------------------------------------------------
snErr TScope::SynchScope(const PSelection inObject)
{
	double inRA		= inObject.fRightAscension;
	double inDec	= inObject.fDeclination;
	
	return this->SynchScope(inRA, inDec);
}


//----TScope-----------------------------------------------------------------
//	TScope::SlewScope
//
//	Purpose:	Slew the scope to the passed gaze.
//									
//	Inputs:		slewGaze	pointer to SN gaze to slew to
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/05/05	DWW			4.5.0		Moved this to TScope
//----------------------------------------------------------------------
snErr TScope::SlewScope(PGaze* slewGaze)
{
	double inRA		= slewGaze->fGazeRa;
	double inDec	= slewGaze->fGazeDec;
	

	return this->SlewScope(inRA, inDec);
}

//----------------------------------------------------------------------
//	TScope::SlewScope
//
//	Purpose:	Slew the scope to the passed gaze.
//									
//	Inputs:		Ra,Dec		Ra an Dec of centre of starry night gaze
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//	03/05/05	DWW			4.5.0		Moved this to TScope
//----------------------------------------------------------------------
snErr TScope::SlewScope(const PSelection inObject)
{
	double inRA		= inObject.fRightAscension;
	double inDec	= inObject.fDeclination;
	
	return this->SlewScope(inRA, inDec);
}


//----------------------------------------------------------------------
//	TScope::SlewScope
//
//	Purpose:	Slew the scope in the passed Direction.
//									
//	Inputs:		dir		Direction to slew into
//				speed   Rate for this slew
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/07/13	MK			5.0			Finalized
//----------------------------------------------------------------------

snErr TScope::SlewScope(const PScopeMotion* inMotionParam)
{
	return pluginErr_FunctionNotSupported;
}

//----------------------------------------------------------------------
//	TScope::SetSlewRate
//
//	Purpose:	Set scope's global slew rate.
//									
//	Inputs:		speed   rate of slew
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/07/16	MK			5.0			Finalized
//----------------------------------------------------------------------

snErr TScope::SetSlewRate(const ScopeSlewRateT speed)
{
	return pluginErr_FunctionNotSupported;
}

#pragma mark ---Parking and Home---

//----------------------------------------------------------------------
//	TScope::ParkScope
//
//	Purpose:	Parks the telescope
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TScope::ParkScope(void)
{
	return pluginErr_FunctionNotSupported;
}

//----------------------------------------------------------------------
//	TScope::UnparkScope
//
//	Purpose:	Un-Parks the telescope
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TScope::UnparkScope(void)
{
	return pluginErr_FunctionNotSupported;
}

//----------------------------------------------------------------------
//	TScope::SetParkPosition
//
//	Purpose:	Sets the park position of the telescope to its currennt position
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TScope::SetParkPosition(void)
{
	return pluginErr_FunctionNotSupported;
}

//----------------------------------------------------------------------
//	TScope::MoveScopeToHomePosition
//
//	Purpose:	Moves the scope to its home position
//									
//	Inputs:		none
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/01/26	DWW			5.0.2		New for Pro Plus
//----------------------------------------------------------------------

snErr TScope::MoveScopeToHomePosition(void)
{
	return pluginErr_FunctionNotSupported;
}


#pragma mark ---Communication---

//----------------------------------------------------------------------
//	TScope::GetScopeResponse()
//
//	Purpose:	Gets the scope response to a command. It should be used
//				as necessary after a command has been sent to the scope.
//									
//	Inputs:		outString will be set to the response given by the scope
//				maxNumberOfCharsToRead 		is the MAXIMUM number of characters to read back
//				maxNumberOfMillisecsToWait 	is the MAXIMUM number of milliseconds to wait
//											data to begin arriving.
//				stopFlag					character indicating that the response is complete
//											NOTE: '\0' value indicates that such a character
//												  is not to be expected
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  05/01/10	MK			5.02		added stopflag option
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TScope::GetScopeResponse(LStr255& outString, UInt32 maxNumberOfCharsToRead, double maxNumberOfMillisecsToWait, const char stopFlag)
{
	PluginAssert(&outString);
	fScopeSerialCommunicator->GetSerialInfo(outString, maxNumberOfCharsToRead, maxNumberOfMillisecsToWait, stopFlag);

	return snNoErr;
}

//----------------------------------------------------------------------
//	TScope::CloseScopeConnection
//
//	Purpose:	Close the telescope's serial port connection
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
snErr TScope::CloseScopeConnection(void)
{
	fScopeSerialCommunicator->CloseSerialComm();
	fIsConnected = false;
	return snNoErr;
}