//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		Scope.h
//
//	Contains:	Base class for all Telescope classes.
//								
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  05/03/02	MK			5.02		Support for 2-star alignment added
//  04/06/24	MK			5.0			Bugfixes and optimizations
//	01/09/15	DWW			4.0.0		New internal telescope support for Pro 4
//
//----------------------------------------------------------------------

#ifndef serialCommunicator_h
	#include "SerialCommunicator.h"
#endif

#if TARGET_SCOPE_CONFIG_UTIL
		#include "UtilityTypedefsHack.h"
#endif /*TARGET_SCOPE_CONFIG_UTIL*/

#ifndef scope_h
#define scope_h




//----------------------------------------------------------------------
//	Class:		TScope
//
//	Purpose:	Abstract base class from which all telescope classes should descend
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.02		Added GetMustAlignInSoftware(void)/fMustAlignInSoftware
//  04/06/24	MK			5.0			Added ConvertAltAzNowToRaDecJNow
//	01/09/15	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

class TScope
{
	public:
		
		static TScope*			MakeNewTScope(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType);
		static const UInt32		GetNumberOfSupportedScopes(void);
		static const LStr255*	GetPointerToFirstScopeNameString(void);
		static const LStr255*	GetPointerToScopeNameString(scopeType inScopeType);
		static pBoolean			DoesScopeTypeSupportSetTimeAndLocation(scopeType inScopeType);

		//**************************
		// GENERAL utility functions
		//**************************
		static long double 	ModPi2(long double x);
		static void 		SleepDelay(double inMillsecondsToSleepFor);
		static void			ConvertRadiansToHoursMinutesTenths(	double 	inOrdinate, 
																SInt32& outHours, 
																SInt32& outMinutes,
																SInt32& outTenthsOfMinutes);
		static void			ConvertRadiansToHoursMinutesSeconds(double 	inOrdinate, 
																SInt32& outHours, 
																SInt32& outMinutes,
																SInt32& outSeconds);
		static void			ConvertRadiansToDegreesMinutes(	double inOrdinate, 
															SInt32& outDegrees, 
															SInt32& outMinutes);
		static void			ConvertRadiansToDegreesMinutesSeconds(	double inOrdinate, 
																	SInt32& outDegrees, 
																	SInt32& outMinutes,
																	SInt32& outSeconds);
		static void			ConvertRaDecJNowToAltAzNow( double inRa,
														double inDec,
														double& outAlt, 
														double& outAz);
		static void			ConvertAltAzNowToRaDecJNow( double inAlt,
														double inAz,
														double& outRa,
														double& outDec);
	public:			
                         TScope(TSerialCommunicator* inScopeSerialCommunicator, scopeType inScopeType);
		virtual			~TScope(void);
				
	public:
	
		// Open and close the connection to the scope
		virtual snErr	OpenScopeConnection(void) = 0;
		virtual	snErr	CloseScopeConnection(void);
		
		// Synch methods
		virtual	snErr	SynchScope(double inCoordOne, double inCoordTwo) {return pluginErr_FunctionNotSupported;}
		virtual	snErr	SynchScope(PGaze* synchGaze);
		virtual	snErr	SynchScope(const PSelection inObject);
		
		// Slew methods
		virtual	snErr	SlewScope(double inCoordOne, double inCoordTwo) = 0;
		virtual	snErr	SlewScope(PGaze* slewGaze);
		virtual	snErr	SlewScope(const PSelection inObject);
		virtual	snErr	SlewScope(const PScopeMotion* inMotionParam);
		
		// Parking , Unparking, Home etc. Methods
		virtual	snErr	ParkScope(void);
		virtual	snErr	UnparkScope(void);
		virtual	snErr	SetParkPosition(void);
		virtual	snErr	MoveScopeToHomePosition(void);
		
		// Slew rate
		virtual snErr   SetSlewRate(const ScopeSlewRateT speed);
		
		// Gets the response string to a commmand sent to the scope
		snErr	GetScopeResponse(	LStr255& outString, 	
									UInt32 maxNumberOfCharsToRead,
									double maxNumberOfMillisecsToWait,
									const char stopFlag = '\0');
		
		// Ra and Dec the scope id currently pointing at
		virtual	snErr	GetScopeRaDec(double& outRA, double& outDec) = 0;
		
		// Az and Alt the scope id is currently pointing at (used for alignment)
		virtual snErr   GetScopeAzAlt(double& outAz, double& outAlt) = 0;

		// Time and Location setting for the scope
		virtual	snErr	SetScopeLocation(const double inLat, const double inLong) {return pluginErr_FunctionNotSupported;}
		virtual	snErr	SetScopeTime(const double inJulianDate, const SInt32 inTimeZoneOffsetSeconds, const pBoolean inIsDSTInEffect) {return pluginErr_FunctionNotSupported;}

		virtual	snErr	GetScopeLocation(double& outLat, double& outLong) {return pluginErr_FunctionNotSupported;}
		virtual	snErr	GetScopeTime(double& outJulianDate, long& outScopeTimeZoneOffsetSeconds) {return pluginErr_FunctionNotSupported;}
			
			
		pBoolean	GetIsConnected(void) const				{ return fIsConnected;			 }
		pBoolean	GetIsParked(void)						{ return fIsParked;				 }
		pBoolean	GetMustAlignInSoftware(void)			{ return fMustAlignInSoftware;   }
	 	
		pBoolean	GetCanSynch(void) const					{ return fCanSynch;				 }
		pBoolean	GetCanSlew(void) const					{ return fCanSlew;				 }
 		pBoolean	GetCanSetScopeTime(void) const			{ return fCanSetScopeTime;		 }
 		pBoolean	GetCanSetScopeLocation(void) const		{ return fCanSetScopeLocation;	 }
		pBoolean	GetCanDirSlew(void) const				{ return fCanDirSlew;			 }
		pBoolean	GetCanSetSlewRate(void) const			{ return fCanSetSlewRate;		 }
		pBoolean	GetCanSetSlewDirRate(void) const		{ return fCanSetSlewDirRate;	 }
 		
		pBoolean	GetCanPark(void) const					{ return fCanPark;			     }
		pBoolean	GetCanUnpark(void) const				{ return fCanUnpark;		     }
		pBoolean	GetCanSetPark(void) const				{ return fCanSetPark;		     }
		pBoolean	GetCanGoHome(void) const				{ return fCanGoHome;		     }
		
		scopeType	GetScopeType(void) const				{ return fScopeType;			 }
		double		GetScopeWaitTime(void) const			{ return fScopeWaitTime;		 }
		double		GetScopeEncoderResolution(void) const   { return fScopeEncoderResolution;}
		LStr255*	GetScopePrettyNamePtr(void)				{ return &fScopePrettyName;		 }
			
		TSerialCommunicator*	GetfScopeSerialCommunicatorPtr(void) const { return fScopeSerialCommunicator;}
	
	protected:
		pBoolean	fIsConnected;		// Note that it is the responsibility of the scope to maintain this		
		pBoolean	fIsParked;		// Note that it is the responsibility of the scope to maintain this
		pBoolean	fMustAlignInSoftware;

		pBoolean	fCanSynch;			
		pBoolean	fCanSlew;
 		pBoolean	fCanSetScopeTime;
		pBoolean	fCanSetScopeLocation;
		pBoolean	fCanDirSlew;
		pBoolean    fCanSetSlewRate;
		pBoolean	fCanSetSlewDirRate;

		pBoolean	fCanPark;					
 		pBoolean	fCanUnpark;
		pBoolean	fCanSetPark;
		pBoolean	fCanGoHome;	
		
		pBoolean	fSerialCommandEchoed;
		
 		scopeType	fScopeType;
 		LStr255 	fScopePrettyName;
		double		fScopeWaitTime;
		
		double		fScopeEncoderResolution;

		TSerialCommunicator* fScopeSerialCommunicator;
};


//----------------------------------------------------------------------
//	Class:		TMeadeScope
//
//	Purpose:	Control Meade Meade and Autostar type telescopes
//				
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/11/15	MK			5.7			Removed attempt to sync on connection close
//  04/06/23	MK			5.0			Removed unnecessary delays
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//
//----------------------------------------------------------------------

class TMeadeScope : public TScope
{

	public:
						TMeadeScope(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType);
		virtual			~TMeadeScope();

	public:
		virtual	snErr	OpenScopeConnection(void);
		virtual	snErr	CloseScopeConnection(void);

		// Synch to current gaze and Slew to current gaze and slew in direction
		virtual	snErr	SynchScope(double inCoordOne, double inCoordTwo);
		virtual	snErr	SlewScope(double inCoordOne, double inCoordTwo);
		virtual	snErr	SlewScope(const PScopeMotion* inMotionParam);
		
		virtual snErr   SetSlewRate(const ScopeSlewRateT speed);
	
		// Parking , Unparking, Home etc. Methods
		virtual	snErr	ParkScope(void);
		virtual	snErr	UnparkScope(void);
		virtual	snErr	SetParkPosition(void);
		virtual	snErr	MoveScopeToHomePosition(void);

		// Ra and Dec the scope id currently pointing at
		virtual	snErr	GetScopeRaDec(double& outRA, double& outDec);
		
		// Alt and Az that the scope is currently pointing at
		virtual snErr   GetScopeAzAlt(double& outAz, double& outAlt);

		// Time and Location setting for the scope
		virtual	snErr	SetScopeLocation(const double inLat, const double inLong); 
		virtual	snErr	SetScopeTime(const double inJulianDate, const SInt32 inTimeZoneOffsetSeconds, const pBoolean inIsDSTInEffect);

		virtual	snErr	GetScopeLocation(double& outLat, double& outLong);
		virtual	snErr	GetScopeTime(double& outJulianDate, long& outScopeTimeZoneOffsetSeconds);

		
		//*************************
		// MEADE specific helper utilities
		//*************************
	private:
		pBoolean 	CheckConnection(void);
		pBoolean 	IsSlewInProgress(void);

		void 		MakeMeadeSetRAString(double inRA, LStr255& outString);
		void 		MakeMeadeSetDecString(double inDec, LStr255& outString);
		snErr 		SetMeadeObjectRaDec( double inRA, double inDec);

		void 		MakeMeadeSetLatString(double inLat, LStr255& outString);
		void 		MakeMeadeSetLongString(double inLong, LStr255& outString);

		void 		MakeMeadeSetTimeString(long inHour, long inMinute, long inSecond, LStr255& outSetTimeString);
		void 		MakeMeadeSetDateString(long inYear, long inMonth, long inDay, LStr255& outSetDateString);

		snErr 		ConvertMeadeRAToDouble(LStr255 inRAString, double& outRA);
		snErr 		ConvertMeadeDecToDouble(LStr255 inDecString, double& outDec);
		
		void		WaitForSlewFinishedCharacterOrTimeoutPassed(void);
	
	protected:
		pBoolean	fIsHighPrecision; // Meade scopes can be in high or low precision modes
};

//----------------------------------------------------------------------
//	Class:		TCelestronScope
//
//	Purpose:	Control Celestron telescopes		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/15	MK			5.0			Removed bugs and optimized code
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//----------------------------------------------------------------------

class TCelestronScope : public TScope
{
	public:
						TCelestronScope(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType);
		virtual			~TCelestronScope();

	public:
		pBoolean		CheckAlignmentComplete(void);
		virtual snErr	OpenScopeConnection(void);

		// Synch to current gaze and Slew to current gaze and direction
		virtual	snErr	SynchScope(double inCoordOne, double inCoordTwo);
		virtual	snErr	SlewScope(double inCoordOne, double inCoordTwo);
		virtual	snErr	SlewScope(const PScopeMotion* inMotionParam);

		// Ra and Dec the scope id currently pointing at
		virtual	snErr	GetScopeRaDec(double& outRA, double& outDec);
		
		// Alt and Az that the scope is currently pointing at
		virtual snErr   GetScopeAzAlt(double& outAz, double& outAlt);
		
	private:
		pBoolean 	IsReadyForCommand(void);
		pBoolean    GetValidResponse(LStr255& outString, UInt32 numberOfCharsToRead);
		pBoolean 	GetUses32BitCoordinates(void);
		void	 	WaitForSlewFinishedCharacterOrTimeoutPassed(void);
	
	protected:
		void 		MakeAltAzSynchString(double inAlt, double inAz, LStr255& outString);
		void 		MakeRaDecSlewString(double inRA, double inDec, LStr255& outString);
//		void		MakeCelestronAltAzSlewString(double inAlt, double inAz, LStr255& outString);
		void 		Make16BitHexStringFromRadianCoord(double inCoord, LStr255& outString);
		void 		Make32BitHexStringFromRadianCoord(double inCoord, LStr255& outString);
		void 		MakeRadianCoordsFromHexString(	LStr255& inString, 
													double& outCoordOne,  //RA or Az 
													double& outCoordTwo);  //Dec or Alt
		void 		Convert16BitHexStringToRadianDouble(LStr255& inString, double& outCoord);
		void 		Convert32BitHexStringToRadianDouble(LStr255 inString, double& outCoord);
//		Not needed anymore - using strtod - MK
//		void 		ConvertHexCharToDouble(unsigned char inChar, double& outDouble);
	public:
		pBoolean	fUses32BitCoords;
};

//----------------------------------------------------------------------
//	Class:		TNexStar58
//
//	Purpose:	Control the Celestron NexStar 5 and 8 scopes. Significantly differnt
//				from other Celestron scopes.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  04/06/24	MK			5.0			Removed bugs and optimized code
//	03/05/05	DWW			4.5.0		Re-engineered for 4.5
//	00/05/29	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

class TNexStar58 : public TScope
{
	public:
						TNexStar58(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType);
		virtual			~TNexStar58();

	public:
		virtual snErr	OpenScopeConnection(void);

		// Synch to current gaze and Slew to current gaze	
		virtual	snErr	SlewScope(double inRA, double inDec);
//		Removed the following two so that functions of the parent class are used (MK - 04/06/24)
//		virtual	snErr	SlewScope(PGaze* slewGaze);
//		virtual	snErr	SlewScope(const PSelection inObject);

		// Ra and Dec the scope id currently pointing at
		virtual	snErr	GetScopeRaDec(double& outRA, double& outDec);
		
		// Alt and Az that the scope is currently pointing at
		virtual snErr   GetScopeAzAlt(double& outAz, double& outAlt);
			
		//***********************************************
		// CELESTRON NexStar5/8 specific helper utilities
		//***********************************************
	private:
	
		pBoolean 	NexStar58CommandInitialization(void);
		void		MakeNexStar58SlewString(double inRA, double inDec, LStr255& outString);
		void 		MakeNexStar58HighLowByteStringsFromRadianCoord(double inCoord, UInt8& outHigh, UInt8& outLow);
		void		MakeRadianCoordsFromNexStar58HighLowByteString(	LStr255& inString, 
																	double& outCoordOne,  //RA or Az
																	double& outCoordTwo); //Dec or Alt
		pBoolean 	WaitForSlewFinishedCharacterOrTimeoutPassed(void);
};

//----------------------------------------------------------------------
//	Class:		TOrionScope
//
//	Purpose:	Control Orion telescopes		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		2-Star alignment introduced
//	04/06/07	MK			5.0			Added support
//----------------------------------------------------------------------

class TOrionScope : public TScope
{
	public:
						TOrionScope(TSerialCommunicator *inSerialCommunicator, scopeType inScopeType);
		virtual			~TOrionScope();

	public:
		virtual snErr	OpenScopeConnection(void);

		// this is not supported by Orion - no motors
		virtual	snErr	SlewScope(double inCoordOne, double inCoordTwo);

		// Ra and Dec the scope id currently pointing at
		virtual	snErr	GetScopeRaDec(double& outRA, double& outDec);

		// Alt and Az that the scope is currently pointing at
		virtual snErr   GetScopeAzAlt(double& outAz, double& outAlt);
		
	private:
		pBoolean	alignedByPlugin;	
		pBoolean 	IsReadyForCommand(void);
	
};

#endif //scope_h
