//
//	GSXSerialCommunicator.h
//	 1998, 1999, 2000, 2001 Kyle Hammond
//	kyle@codeblazer.com
//	Use at your own risk.  All rights reserved.  Do not distribute without permission.
//

//----------------------------------------------------------------------
//	Modified by Dave Whipps of SPACE.com Canada Inc.
//	Modifications Copyright  1990-2002 by SPACE.com Canada, Inc., All rights reserved.
//
//	File:		GSXSerialCommunicator.h
//
//	Contains:	Abstract base class for dealing with Mac and Win serial ports
//							
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		GetSerialInfo can now stop waiting for data when stopFlag
//												is reached
//	01/12/12	DWW			4.0.0		New internal telescope support for Pro 4
//
//----------------------------------------------------------------------

#if !defined( __GSXSerialCommunicator_h_ )
#define __GSXSerialCommunicator_h_	1

#if PRAGMA_STRUCT_ALIGN
	// This allows you to store the serial port record on 68K and PowerPC MacOS in the same format.
	#pragma options align=power
#endif

#if TARGET_OS_MAC
	#ifndef __TEXTUTILS__
		#include <TextUtils.h>
	#endif
#endif

#if TARGET_OS_WIN32
	#include <string.h>
#endif

#include "PluginLString.h"


// A structure to contain the serial port configuration.
typedef struct {
	unsigned long	baud;			// Simply the baud rate in decimal.
#if TARGET_OS_MAC
	SInt8		dataBits;			// The number of bits per byte to send (in decimal).
	SInt8		parity;				// For parity and stopBits use the constants
	SInt8		stopBits;			// enumerated below.
#else
	char		dataBits;			// The number of bits per byte to send (in decimal).
	char		parity;				// For parity and stopBits use the constants
	char		stopBits;			// enumerated below.
#endif
} GSSerialPortInfoRecord;

#if TARGET_OS_WIN32
	const short kWin32PortNameLength = 64;
#endif



typedef struct {
#if TARGET_OS_MAC
	Str63	userPortName;		// Display this in a list for the user to choose from.
	Str63	inPortName;		// Used internally.
	Str63	outPortName;		// Used internally.
#else
	char	userPortName[ 	kWin32PortNameLength ];	// Display this in a list for the user to choose from.
	char	inPortName[ 	kWin32PortNameLength ];	// Used internally.
	char	outPortName[ 	kWin32PortNameLength ];	// Used internally.
#endif
} GSXSerialPortNameRecord, **GSXSerialPortNameRecordHandle;


// added by DW 01/11/13 for use in the PurgeFlushSerialBuffer function
enum bufferType	{
		buffer_INandOUT,
		buffer_IN,
		buffer_OUT
		};
				

enum {
	// parity field values.
	kGSSerialParityNone = 0,
	kGSSerialParityOdd,
	kGSSerialParityEven,

	// stopBits field values.
	kGSSerialStopBits10 = 0,	// 1.0 stop bit
	kGSSerialStopBits15,		// 1.5 stop bits
	kGSSerialStopBits20			// 2.0 stop bits
};


class GSXSerialCommunicator {
		// Abstract Base Class for both serial communicators.

	public:
	
		// Initialize and close serial communications.
		virtual SInt32	InitSerialComm( void ) = 0;
		virtual SInt32	CloseSerialComm( void ) = 0;

		// Change the port settings (including switching ports).
		virtual SInt32	SetPortOptions( const GSSerialPortInfoRecord *inPortInfo,
										const GSXSerialPortNameRecord *inPortName ) = 0;

		// Get current port settings.  Either parameter may be NULL if you are not interested in that info.
		virtual SInt32	GetPortOptions( GSSerialPortInfoRecord *outPortInfo, GSXSerialPortNameRecord *outPortName );

		// Send and get data over the serial port.
		virtual SInt32	SendSerialInfo( const void *inBuffer, const unsigned long inBufferSize ) = 0;
		virtual SInt32	GetSerialInfo( void *outBuffer, unsigned long *ioCount, const long inWaitTicks, const char stopFlag ) = 0;

		virtual SInt32	PurgeFlushSerialBuffer(bufferType whichBuffer) = 0;
		
		// Destructor.
		virtual 		~GSXSerialCommunicator( );


	// Implementation available to sub-classes.
	protected:
	
		GSSerialPortInfoRecord		mCurrentPortInfo;	// The current port settings
		GSXSerialPortNameRecord		mCurrentPortName;	// The current port name
		bool						mAsync;				// Are we in asynchronous mode?

		// Constructor.
		GSXSerialCommunicator( 	const GSSerialPortInfoRecord *inPortInfo, 
								const GSXSerialPortNameRecord *inPortName, 
								const bool inAsynchronous);

	private:
	
		// Can't use the default constructor or the copy constructor.
		GSXSerialCommunicator( );
		GSXSerialCommunicator( const GSXSerialCommunicator & );
};



// Tests to see if inPort1 and inPort2 refer to the same port.  The user names are not checked or changed.
bool PortRecordsEqual( const GSXSerialPortNameRecord *inPort1, const GSXSerialPortNameRecord *inPort2 );


// Inline the simple functions.

inline SInt32 GSXSerialCommunicator::GetPortOptions( GSSerialPortInfoRecord *outPortInfo, GSXSerialPortNameRecord *outPortName )
{	if ( outPortInfo != NULL )	*outPortInfo = mCurrentPortInfo;
	if ( outPortName != NULL )	*outPortName = mCurrentPortName;
	return 0;
}


inline GSXSerialCommunicator::~GSXSerialCommunicator( )
{	}


inline GSXSerialCommunicator::GSXSerialCommunicator( const GSSerialPortInfoRecord *inPortInfo,
			const GSXSerialPortNameRecord *inPortName, const bool inAsynchronous ) : mCurrentPortInfo( *inPortInfo ),
			mCurrentPortName( *inPortName ), mAsync( inAsynchronous )
{	}


inline bool PortRecordsEqual( const GSXSerialPortNameRecord *inPort1, const GSXSerialPortNameRecord *inPort2 )
{	// Compare only the in and out name; don't compare the user name.
	#if TARGET_OS_MAC
		if ( EqualString( inPort1->inPortName, inPort2->inPortName, true, true ) )
			return EqualString( inPort1->outPortName, inPort2->outPortName, true, true );

		return false;
	#else
		if ( strcmp( inPort1->inPortName, inPort2->inPortName ) == 0 )
			return ( strcmp( inPort1->outPortName, inPort2->outPortName ) == 0 );

		return false;
	#endif
}

#endif