//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		SerialCommunicator.cpp
//
//	Contains:	Wrapper classes for the two  types of serial communicators
//					
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		GetSerialInfo can now stop waiting for data when stopFlag
//												is reached
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
#include <termios.h>

#if TARGET_OS_MAC
	#include "GSSerialMacOS.h"
#endif /*TARGET_OS_MAC */

#if TARGET_OS_WIN32
	#include "GSSerialWin32.h"
#endif /*TARGET_OS_WIN32*/

#include "SerialCommunicator.h"

class GSXSerialCommunicator;
class GSSerialWin32;
class GSSerialMacOSX;

//----------------------------------------------------------------------
//	TSerialCommunicator::FindReadySerialPorts
//
//	Purpose:	Static function that forms an array of available serial ports
//				and returns the number of available ports. Depends on the OS.
//					
//	Call:		Any time.
//
//	Returns:	The number of ready serial ports.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial
//----------------------------------------------------------------------
UInt8 TSerialCommunicator::FindReadySerialPorts(GSXSerialPortNameRecordHandle *outPortInfo)
{
	#if TARGET_OS_MAC
			return GSSerialMacOSX::FindReadySerialPorts(outPortInfo);
							
	#elif TARGET_OS_WIN32
			return GSSerialWin32::FindReadySerialPorts(outPortInfo);
	#endif							
}

//----------------------------------------------------------------------
//	TSerialCommunicator::MakeNewSerialConnection
//
//	Purpose:	Static function that creates a new serial communicator object
//				depending on the OS.
//					
//	Call:		Any time.
//
//	Returns:	A pointer to a new TSerialCommunicator object.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial
//----------------------------------------------------------------------
TSerialCommunicator*	TSerialCommunicator::MakeNewSerialConnection(	const GSSerialPortInfoRecord *inPortInfo, 
																		const GSXSerialPortNameRecord *inPortName)
{
	#if TARGET_OS_MAC
			return new TMacOSXSerialCommunicator(inPortInfo, inPortName, false);
	
	#elif TARGET_OS_WIN32
			return new TWin32SerialCommunicator(inPortInfo, inPortName, false);
	#endif
}

//----------------------------------------------------------------------
//	TSerialCommunicator::TSerialCommunicator
//
//	Purpose:	Constructor
//					
//	Call:		Any time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial
//----------------------------------------------------------------------
TSerialCommunicator::TSerialCommunicator(void)
{
	fIsSerialConnectionInitialized = false;
}

//----------------------------------------------------------------------
//	TSerialCommunicator::~TSerialCommunicator
//
//	Purpose:	Destructor
//					
//	Call:		Any time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial/USB ports/
//----------------------------------------------------------------------
TSerialCommunicator::~TSerialCommunicator(void)
{
	// if the port is still open, close it now
	if(fIsSerialConnectionInitialized)
	{
		this->CloseSerialComm();
	}
	
	delete fGSXSerialConnection;
	fGSXSerialConnection = NULL;		
}

//----------------------------------------------------------------------
//	TSerialCommunicator::InitSerialComm
//
//	Purpose:	Initializes the serial port readying it for communications.
//					
//	Call:		Any time.
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::InitSerialComm( void )
{
	if(!fIsSerialConnectionInitialized)
	{
		fIsSerialConnectionInitialized = true;
		return fGSXSerialConnection->InitSerialComm();
	}
	else
	{
		// we shoud NOT try to Init (open) the serial comm if it is already open
		// (It leads to very serious problems in Win32)
		return -1;
	}	
}

//----------------------------------------------------------------------
//	TSerialCommunicator::CloseSerialComm
//
//	Purpose:	Closes the serial port after use. Note that this is VERY IMPORTANT
//				otherwise some wacky stuff can happen if another app tries to access the port.
//					
//	Call:		Any time.
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::CloseSerialComm(void)
{
	if(fIsSerialConnectionInitialized)
	{
		fIsSerialConnectionInitialized = false;
		return fGSXSerialConnection->CloseSerialComm();
	}
	else
	{
		return -1;
	}
}

//----------------------------------------------------------------------
//	TSerialCommunicator::SetPortOptions
//
//	Purpose:	Sets the port communication options (e.g. baud rate, parity, etc.) 
//					
//	Call:		Any time.
//
//	Inputs:		inPortInfo is a pointer to a struct containing all of the options (baud rate etc.
//				inPortName is a pointer to a struct containing the desired port
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::SetPortOptions(const GSSerialPortInfoRecord *inPortInfo, 
											const GSXSerialPortNameRecord *inPortName)
{
	if(fIsSerialConnectionInitialized)
		return fGSXSerialConnection->SetPortOptions(inPortInfo,inPortName);
	else
	{
		return -1;
	}
}

//----------------------------------------------------------------------
//	TSerialCommunicator::GetPortOptions
//
//	Purpose:	Return the port communication options (e.g. baud rate, parity, etc.) 
//					
//	Call:		Any time.
//
//	Outputs:	outPortInfo is a pointer to a struct containing all of the options (baud rate etc.
//				outPortName is a pointer to a struct containing the desired port
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::GetPortOptions(GSSerialPortInfoRecord *outPortInfo, 
											GSXSerialPortNameRecord *outPortName)
{	
	if(fIsSerialConnectionInitialized)
		return fGSXSerialConnection->GetPortOptions(outPortInfo, outPortName);
	else
	{
		return -1;
	}
}

//----------------------------------------------------------------------
//	TSerialCommunicator::SendSerialInfo
//
//	Purpose:	Sends a string of characters to the serial port.
//					
//	Call:		Any time.
//
//	Inputs:		inBuffer		a pointer to the string to send
//				inBufferSize	the number of characters in the buffer to send
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::SendSerialInfo(const void *inBuffer, const UInt32 inBufferSize)
{
	if(fIsSerialConnectionInitialized)
		return fGSXSerialConnection->SendSerialInfo(inBuffer, inBufferSize);
	else
		return -1;
}

//----------------------------------------------------------------------
//	TSerialCommunicator::SendSerialInfo
//
//	Purpose:	Sends a string of characters to the serial port.
//					
//	Call:		Any time.
//
//	Inputs:		inBuffer	a pointer to an LString to send (must be converted to a C string)
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32 	TSerialCommunicator::SendSerialInfo(const LString& inBuffer)
{
	if(fIsSerialConnectionInitialized)
		return fGSXSerialConnection->SendSerialInfo(&(inBuffer[1]), inBuffer.Length());
	else
		return -1;
}

//----------------------------------------------------------------------
//	TSerialCommunicator::GetSerialInfo
//
//	Purpose:	Gets a string from the serial port.
//					
//	Call:		Any time.
//
//	Inputs:		outBuffer		a pointer to the string that will be used to store the info from the port
//				ioCount			the MAXIMUM number of characters to read out from the port
//				inWaitMilliseconds	amount of milliseconds to wait until data is received
//				stopFlag		character indicating that the response is complete
//								NOTE: '\0' value indicates that such a character is not to be expected
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  05/01/10	MK			5.02		Added stopflag functionality
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::GetSerialInfo(void *outBuffer, UInt32 *ioCount, double inWaitMilliseconds, const char stopFlag = '\0')
{
	if (fIsSerialConnectionInitialized)
	{
		double waitSeconds = inWaitMilliseconds /1000.0;
		UInt32 waitTicks = (UInt32)(waitSeconds * kTicksPerSecond);
		return fGSXSerialConnection->GetSerialInfo(outBuffer, ioCount, waitTicks, stopFlag);
	}
	else
		return -1;
}

//----------------------------------------------------------------------
//	TSerialCommunicator::GetSerialInfo
//
//	Purpose:	Gets a string from the serial port.
//					
//	Call:		Any time.
//
//	Inputs:		outString				The string that will be used to store the info from the port
//				maxNumberOfCharsToRead 	The MAXIMUM number of characters to read out from the port
//				inWaitMilliseconds		Amount of milliseconds to wait until data is received
//				stopFlag				character indicating that the response is complete
//										NOTE: '\0' value indicates that such a character is not to be expected
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//  05/01/10	MK			5.02		Added stopflag functionality (waiting for data stops when stopFlag is encountered)
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::GetSerialInfo(LStr255& outString, UInt32 maxNumberOfCharsToRead, double inWaitMilliseconds, const char stopFlag = '\0')
{
	if (fIsSerialConnectionInitialized)
	{
		const	 UInt32 outbufferSize = 256;
		char	outbuffer[outbufferSize];
		// Initialize the array to zeros
		for (UInt32 i = 0; (i < outbufferSize); i++)
			{ outbuffer[i] = '0'; }
			
		UInt32 numberOfCharsActuallyRead = maxNumberOfCharsToRead; 

		double waitSeconds = inWaitMilliseconds / 1000.0;
		UInt32 waitTicks = (UInt32)(waitSeconds * kTicksPerSecond);

		SInt32 outResult = fGSXSerialConnection->GetSerialInfo(outbuffer, &numberOfCharsActuallyRead, waitTicks, stopFlag);
				
		outString.Assign(outbuffer, numberOfCharsActuallyRead);
		return outResult;
	}
	else
		return -1;
}

//----------------------------------------------------------------------
//	TSerialCommunicator::PurgeFlushSerialBuffer
//
//	Purpose:	Cleans out the serial port buffer to ensure "clean" IO
//					
//	Call:		Any time.
//
//	Inputs:		whichBuffer		the buffer type (in, out or both) to be cleaned out
//
//	Returns:	An error code (OS dependant).
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
SInt32	TSerialCommunicator::PurgeFlushSerialBuffer(bufferType whichBuffer)
{
	return fGSXSerialConnection->PurgeFlushSerialBuffer(whichBuffer);
}

//----------------------------------------------------------------------
//	TSerialCommunicator::GetIsSerialConnectionInitialized
//
//	Purpose:	Determine if the serial port has been initialized
//					
//	Call:		Any time.
//
//	Inputs:		none
//
//	Returns:	fIsSerialConnectionInitialized
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
bool	TSerialCommunicator::GetIsSerialConnectionInitialized(void) const
{
	return fIsSerialConnectionInitialized;
}

//----------------------------------------------------------------------
//	TSerialCommunicator::SetIsSerialConnectionInitialized
//
//	Purpose:	Sets if the serial port has been initialized
//					
//	Call:		Any time.
//
//	Inputs:		hasBeenInitialized	whether or not the port has been initialized
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
void	TSerialCommunicator::SetIsSerialConnectionInitialized(bool hasBeenInitialized)
{
	fIsSerialConnectionInitialized = hasBeenInitialized;
}

#if TARGET_OS_MAC

#pragma mark ---MACOSX---
//----------------------------------------------------------------------
//	TMacOSXSerialCommunicator::TMacOSXSerialCommunicator
//
//	Purpose:	Constructor
//
// 	Note:		Kyle says the OSX code does not work asynchronously, so 
//				initialize inAsynchronous to false in TSerialCOmmunicator::MakeNewSerialConnection.
//
//	Call:		Any time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial ports
//----------------------------------------------------------------------
TMacOSXSerialCommunicator::TMacOSXSerialCommunicator(const GSSerialPortInfoRecord *inPortInfo,
                                                    const GSXSerialPortNameRecord *inPortName,
													const bool inAsynchronous)
{	
	fGSXSerialConnection = new GSSerialMacOSX(inPortInfo, inPortName, inAsynchronous);
	PluginAssert(fGSXSerialConnection);
}

//----------------------------------------------------------------------
//	TMacOSXSerialCommunicator::~TMacOSXSerialCommunicator
//
//	Purpose:	Destructor
//					
//	Call:		Any time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial/USB ports/
//
//----------------------------------------------------------------------
TMacOSXSerialCommunicator::~TMacOSXSerialCommunicator(void)
{
}	

#endif /*TARGET_OS_MAC*/

#pragma mark ---WIN32---
#if TARGET_OS_WIN32
//----------------------------------------------------------------------
//	TWin32SerialCommunicator::TWin32SerialCommunicator
//
//	Purpose:	Constructor
//					
//	Call:		Any time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial/USB ports/
//
//----------------------------------------------------------------------
TWin32SerialCommunicator::TWin32SerialCommunicator(	const GSSerialPortInfoRecord *inPortInfo, 
													const GSXSerialPortNameRecord *inPortName,
													const pBoolean inAsynchronous)
{
	fGSXSerialConnection = new GSSerialWin32(inPortInfo, inPortName, inAsynchronous);
	PluginAssert(fGSXSerialConnection);
}

//----------------------------------------------------------------------
//	TWin32SerialCommunicator::~TWin32SerialCommunicator
//
//	Purpose:	Destructor
//					
//	Call:		Any time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/15	DWW			4.0.0		New support for communicating with serial/USB ports/
//
//----------------------------------------------------------------------
TWin32SerialCommunicator::~TWin32SerialCommunicator(void)
{
}
#endif /*TARGET_OS_WIN32*/











