//
//	GSSerialMacOS.cpp
//	 1998, 1999, 2000, 2001 Kyle Hammond
//	kyle@codeblazer.com
//	Use at your own risk.  All rights reserved.  Do not distribute without permission.
//

//----------------------------------------------------------------------
//	Modified by Dave Whipps of Imaginova Canada Inc.
//	Modifications Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		GSXSerialMacOS.cpp
//
//	Contains:	Class for dealing with Mac OSX serial ports
//							
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/12	MK			5.02		Reading can now stop when a specified flag character is encountered
//										This could speed up response drastically.
//	01/12/12	DWW			4.0.0		New internal telescope support for Pro 4
//
//----------------------------------------------------------------------

#include "GSSerialMacOS.h"

// Debugging Stuff
#if qXDebug
FILE* debugOutputFile = fopen("scopePluginDebug.txt", "w");
int fileErr = fprintf(debugOutputFile, "This is the first line in the file\n");
int flushErr = fflush(debugOutputFile);
#endif

SInt32 GetErrNo(void) 
{
	#ifdef error
		return error;
	#else
		#ifdef __error
			return *(__error());
		#else
			return -1;
		#endif
	#endif
}		

#ifndef MAXPATHLEN
const SInt32 MAXPATHLEN = 1024;
#endif

static SInt8 local_iterateSerialDevices( io_iterator_t inSerialPortIterator, GSXSerialPortNameRecordHandle *outPortInfo );

SInt32 GSSerialMacOSX::InitSerialComm( void )
{	
    SInt32	result = noErr;
    char	portPath[ MAXPATHLEN ];
       
    CopyPascalStringToC( mCurrentPortName.outPortName, portPath );
    
    mFileDescriptor = open( portPath, O_RDWR | O_NOCTTY | O_NDELAY );
    
    if ( mFileDescriptor == -1 )
        result = GetErrNo();
    
    if ( result == noErr ) 
    {
        if ( fcntl( mFileDescriptor, F_SETFL, 0 ) == -1 )
            result = ::GetErrNo();
    }
    
    if ( result == noErr ) 
    {
        if ( tcgetattr( mFileDescriptor, &mOriginalTTYAttrs ) == -1 )
            result = ::GetErrNo();
    }
    
    if ( result == noErr )
        result = SetPortOptions( &mCurrentPortInfo, &mCurrentPortName );
    
    return result;
}

SInt32 GSSerialMacOSX::CloseSerialComm( void )
{	
	SInt32	result = noErr;

	if ( mFileDescriptor != -1 )
        {
            // Traditionally it is good to reset a serial port back to
            // the state in which you found it.  Let's continue that tradition.
            if ( tcsetattr( mFileDescriptor, TCSANOW, &mOriginalTTYAttrs ) == -1 )
                // An error occurred.
			result = ::GetErrNo();
            close( mFileDescriptor );
	}

	return result;
}

SInt32 GSSerialMacOSX::SetPortOptions( const GSSerialPortInfoRecord *inPortInfo, const GSXSerialPortNameRecord *inPortName )
{	
	OSErr		err;

	// DW Changed this to initialize the port options to the Original Attributes.
	struct termios	options = mOriginalTTYAttrs;

	if ( !PortRecordsEqual( inPortName, &mCurrentPortName ) )
        {
            // Need to switch the port.
            err = CloseSerialComm( );
            
            if (err == noErr)
            {
                    mCurrentPortInfo = *inPortInfo;
                    mCurrentPortName = *inPortName;
                    err = InitSerialComm( );
            }

            return err;
	}

	mCurrentPortInfo = *inPortInfo;

	options.c_cflag = (CLOCAL | CREAD);

	// Parity:
	switch ( inPortInfo->parity )
	{
		case kGSSerialParityOdd:	options.c_cflag |= PARODD | PARENB;		break;
		case kGSSerialParityEven:	options.c_cflag |= PARENB;				break;
		case kGSSerialParityNone:	break; // Normal case.
	}

	// Stop bits:
	switch ( inPortInfo->stopBits )
	{
		case kGSSerialStopBits20:	options.c_cflag |= CSTOPB;	break;
		case kGSSerialStopBits15:	break; // Can't do 1.5 bits, so default to using 1 bit
		case kGSSerialStopBits10:	break; // Normal case.
	}

	// Data bits:
	switch ( inPortInfo->dataBits )
	{
		case 5:	options.c_cflag |= CS5;	break;
		case 6:	options.c_cflag |= CS6;	break;
		case 7:	options.c_cflag |= CS7;	break;
		case 8:	options.c_cflag |= CS8;	break;
	}

	// Baud rate.
	options.c_ispeed = options.c_ospeed = inPortInfo->baud;

	// Set raw input, one second timeout
	// These options are documented in the man page for termios
	// (in Terminal enter: man termios)
	options.c_lflag = mOriginalTTYAttrs.c_lflag & ( ~(ICANON | ECHO | ECHOE | ISIG) );
	options.c_oflag = mOriginalTTYAttrs.c_oflag & ( ~OPOST );
	options.c_cc[ VMIN ] = 0;
	options.c_cc[ VTIME ] = 10;

	// Set the options
	if ( tcsetattr( mFileDescriptor, TCSANOW, &options ) == -1 )
            return ::GetErrNo();
		
	return noErr;
}

SInt32 GSSerialMacOSX::SendSerialInfo( const void *inBuffer, const unsigned long inBufferSize )
{	
	SInt32	result = noErr;
	size_t	numBytes = 0;

	if ( inBuffer == nil || inBufferSize == 0 )
		return paramErr;

	if ( mFileDescriptor != -1 ) 
        {
            numBytes = write( mFileDescriptor, const_cast<void *>( inBuffer ), inBufferSize );
            if ( numBytes == (size_t) -1 )
                // An error occurred.
				result = ::GetErrNo();
			else // Some debugging stuff
			{
#if qXDebug
				fprintf(debugOutputFile, "Sent: %s | %d bytes\n", inBuffer, inBufferSize);
				fflush(debugOutputFile);
#endif
			}
		} 
	else
		// Report a "File Not Open" error.
		result = fnOpnErr;

	return result;
}

// Looks for a MAXIMUM of ioCount characters and returns the actual number found
SInt32 GSSerialMacOSX::GetSerialInfo( void *outBuffer, unsigned long *ioCount, const long inWaitTicks, const char stopFlag = '\0' )
{	
	char	*bufPtr;
	SInt32	result = noErr;
	UInt32	finishTime;
	size_t	numBytes, totalBytesRead = 0;

	if (outBuffer == nil || ioCount == nil)
		return paramErr;

	if (mFileDescriptor != -1)
	{
		bufPtr = reinterpret_cast<char *>(outBuffer);
		finishTime = TickCount() + inWaitTicks;
		
		while ((totalBytesRead < *ioCount) && (result == noErr))
		{
			// We're still waiting for data.
			numBytes = read(mFileDescriptor, bufPtr, *ioCount - totalBytesRead);
			
			if ( numBytes == (size_t)-1 )
				result = ::GetErrNo();
			else if ( numBytes > 0 )
			{
				// We read at least one byte, move the buffer pointer and add to the total bytes read count.
				bufPtr += numBytes;
				totalBytesRead += numBytes;
				
				if ((stopFlag != '\0') && (*(bufPtr-1) == stopFlag))
					{
						*ioCount = totalBytesRead;
						return result;
					}
			}

			// Check the timer at the end to see if we're supposed to keep going or not.
			// This way, we at least hit the read command once, even if inWaitTicks is zero.
			if (TickCount() >= finishTime)
				break;
		}

		*ioCount = totalBytesRead;
	} 
	else
		result = fnOpnErr; // Report a "File Not Open" error.
	
	return result;
}


//----------------------------------------------------------------------
//	GSSerialMacOSX::PurgeFlushSerialBuffer
//
//	Purpose:	Cleans out the serial port buffer to ensure "clean" IO
//					
//	Call:		Any time.
//
//	Inputs:		whichBuffer		the buffer type (in, out or both) to be cleaned out
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/11/13	DWW			4.0.0		New support for communicating with serial/USB ports/
//----------------------------------------------------------------------
SInt32 GSSerialMacOSX::PurgeFlushSerialBuffer(bufferType whichBuffer)
{
	SInt32 flushResult = -1;
	OSErr  err  = noErr;
	
	switch(whichBuffer)
	{
		case buffer_INandOUT:
			flushResult = tcflush(mFileDescriptor, TCIOFLUSH);
			break;
		case buffer_IN:
			flushResult = tcflush(mFileDescriptor, TCIFLUSH);
			break;
		case buffer_OUT:
			flushResult = tcflush(mFileDescriptor, TCOFLUSH);
			break;
		default:
			break;
	}
	
	if(flushResult == -1)
		err = ::GetErrNo();
	else
		err = noErr;
	
	return err;					
}

SInt8 GSSerialMacOSX::FindReadySerialPorts( GSXSerialPortNameRecordHandle *outPortInfo)
{
	kern_return_t			kernResult; 
	mach_port_t				masterPort;
	CFMutableDictionaryRef	classesToMatch;
	io_iterator_t			serialPortIterator;
	SInt8					count = 0;

	kernResult = IOMasterPort( MACH_PORT_NULL, &masterPort );
	if ( KERN_SUCCESS == kernResult ) 
	{
        // Serial devices are instances of class IOSerialBSDClient
		classesToMatch = IOServiceMatching( kIOSerialBSDServiceValue );
		if ( classesToMatch != NULL ) 
		{
			// Match to only RS-232 serial devices.
			CFDictionarySetValue( classesToMatch, CFSTR(kIOSerialBSDTypeKey), CFSTR(kIOSerialBSDAllTypes)); //  kIOSerialBSDRS232Type

			kernResult = IOServiceGetMatchingServices( masterPort, classesToMatch, &serialPortIterator );

			if ( KERN_SUCCESS == kernResult )
				// Now, iterate through the results and stuff the serial info into the outPortInfo.
				count = local_iterateSerialDevices( serialPortIterator, outPortInfo );
		}

		IOObjectRelease( serialPortIterator );	// Release the iterator.
	}

	return count;
}

static SInt8 local_iterateSerialDevices(io_iterator_t inSerialPortIterator, GSXSerialPortNameRecordHandle *outPortInfo )
{ 
	io_object_t		serialService;
	kern_return_t	kernResult = KERN_FAILURE;
	int				index; 
	int				count = 0;
	OSErr			err;
	Boolean			result;
	char			bsdPath[ MAXPATHLEN ];
	CFRange			searchRange;

	// Initialize the returned path
	*bsdPath = '\0';
	index = -1;

	// Go through all services in the iterator.
	while ( ( serialService = IOIteratorNext( inSerialPortIterator ) ) != nil ) {
		CFStringRef	portNameAsCFString;
		CFStringRef	bsdPathAsCFString;
		
		// Get the user viewable name for the serial port.
		portNameAsCFString = (CFStringRef)IORegistryEntryCreateCFProperty( serialService, CFSTR(kIOTTYDeviceKey),
																		   kCFAllocatorDefault, 0 );
		searchRange = CFStringFind(portNameAsCFString,CFSTR("modem"),kCFCompareCaseInsensitive);
		if ( searchRange.location == kCFNotFound ) {
			
			// Allocate more memory for the information we need.
			if ( *outPortInfo == nil )
				*outPortInfo = (GSXSerialPortNameRecordHandle)NewHandle( sizeof(GSXSerialPortNameRecord) );
			else
				SetHandleSize( (Handle)*outPortInfo, GetHandleSize( (Handle)*outPortInfo )
							   + sizeof(GSXSerialPortNameRecord) );
			err = MemError( );
			if ( err == noErr ) {
				count++;
				//index = ( GetHandleSize( (Handle)*outPortInfo ) / sizeof(GSXSerialPortNameRecord) ) - 1;
				index++;
			}
			
			if ( portNameAsCFString ) {
				char		portName[ 128 ];
				
				result = CFStringGetCString( portNameAsCFString, portName, sizeof(portName), kCFStringEncodingASCII );
				CFRelease( portNameAsCFString );
				
				if ( result ) {
					// Copy the user viewable name into the outPortInfo.
					HLock( reinterpret_cast<Handle>( *outPortInfo ) );
					CopyCStringToPascal( portName, (**outPortInfo)[ index ].userPortName );
					HUnlock( reinterpret_cast<Handle>( *outPortInfo ) );
					kernResult = KERN_SUCCESS;
				}
			}
			
			// Get the BSD path to the "file" that we'll need to open to use the serial port.
			bsdPathAsCFString = (CFStringRef) IORegistryEntryCreateCFProperty( serialService, CFSTR(kIOCalloutDeviceKey),
																			   kCFAllocatorDefault, 0 );
			if ( bsdPathAsCFString ) {
				result = CFStringGetCString( bsdPathAsCFString, bsdPath, MAXPATHLEN, kCFStringEncodingASCII );
				CFRelease( bsdPathAsCFString );
				if ( result ) {
					// Copy the path for the incoming port name and the outgoing port name.
					HLock( reinterpret_cast<Handle>( *outPortInfo ) );
					CopyCStringToPascal( bsdPath, (**outPortInfo)[ index ].inPortName );
					CopyCStringToPascal( bsdPath, (**outPortInfo)[ index ].outPortName );
					HUnlock( reinterpret_cast<Handle>( *outPortInfo ) );
				}
			}
			
			// We have sucked this service dry of information so release it now.
			(void) IOObjectRelease( serialService );
		}
    }
	
    return count;
}