// ===========================================================================
// ===========================================================================
//	Classes for handling Pascal Strings
//
//	*******************************************************
//	NOTE: 	THIS FILE HAS BEEN MODIFIED FROM ITS ORIGINAL
//		FORM TO FIT WITH THE REQUIREMENTS OF THE STARRY
//		NIGHT TELESCOPE CONTROLLER PLUGIN.
//	******************************************************* 
//
//	LString		- Abstract base class
//	LStr255		- Wrapper class for Str255
//	TString<T>	- Template class for any string type (array of unsigned char)


#ifndef _H_LString
#define _H_LString
#pragma once

#ifdef Length
	#undef Length
#endif


class LStr255;	// Forward declaration
// ---------------------------------------------------------------------------
//	 LString
// ---------------------------------------------------------------------------
//	Abstract base class for a Pascal-style string

class LString {
public:
									// String Comparison Function
									//   left == right, returns 0
									//   left >  right, returns 1
									//   left <  right, returns -1
	typedef	SInt16	(*CompareFunc)(
                                        const void*	inLeft,
                                        const void*	inRight,
                                        UInt8		inLeftLength,
                                        UInt8		inRightLength);

	virtual			~LString() = 0;		// Pure virtual destructor enforces
							//  that LString is abstract

					// Accessors
	UInt8 			Length() const			{ return mStringPtr[0]; }
	SInt16			ShortLength() const		{ return (SInt16) mStringPtr[0]; }
	SInt32			LongLength() const		{ return (SInt32) mStringPtr[0]; }
	UInt16			GetMaxLength() const	{ return mMaxBytes; }
	Ptr				TextPtr()				{ return (Ptr) (mStringPtr + 1); }
	const Ptr		ConstTextPtr() const	{ return (Ptr) (mStringPtr + 1); }

					// Conversion Operatores
					operator StringPtr()	{ return mStringPtr; }
					operator ConstStringPtr() const	{ return mStringPtr; }

					operator SInt32() const;
                                
					operator double() const;
					operator long double() const;

	void			SetCompareFunc( CompareFunc inCompareFunc )
					{
						mCompareFunc = inCompareFunc;
					}

									// Accessing individual characters

#if TARGET_OS_MAC  // This ptrdiff_t resolves some erros in the CW project compile
	UInt8& 			operator [] (ptrdiff_t		inPosition)
						{
							return mStringPtr[inPosition];
						}

	const UInt8&	operator [] (ptrdiff_t		inPosition) const
						{
							return mStringPtr[inPosition];
						}
#else
	UInt8& 			operator [] (UInt8		inPosition)
						{
							return mStringPtr[inPosition];
						}

	const UInt8&	operator [] (UInt8		inPosition) const
						{
							return mStringPtr[inPosition];
						}
#endif
									// Assignment operators

	LString&		operator = (const LString&	inString)
						{
							return Assign(inString.ConstTextPtr(), inString.Length());
						}

	LString&		operator = (ConstStringPtr	inStringPtr)
					{
						return Assign(inStringPtr);
					}

	LString& 		operator = ( unsigned char inChar )
					{
						return Assign(inChar);
					}

	LString&		operator = ( char inChar )
					{
						return Assign(inChar);
					}

	LString& 		operator = ( const char* inCString )
					{
						return Assign(inCString);
					}

	LString&		operator = ( SInt32 inNumber )
					{
						return Assign(inNumber);
					}

	LString&		operator = ( SInt16 inNumber )
					{
						return Assign(inNumber);
					}

                                // Assignment functions
	LString&		Assign(	const LString&	inString,
                                        UInt8 inStartPos = 1,
                                        UInt8 inCount = 255);

	LString&		Assign(	ConstStringPtr	inStringPtr);

	LString&		Assign(	unsigned char	inChar);

	LString&		Assign(	char inChar)
                                {
                                    return Assign((unsigned char) inChar);
                                }

	LString&		Assign(	const char* inCString)
                                {
                                    return Assign(inCString, CStringLength(inCString));
                                }

	LString&		Assign(	const void* inPtr, UInt8 inLength);

	LString&		Assign(	SInt32 inNumber);

	LString&		Assign( SInt16 inNumber)
                                {
                                    return Assign((SInt32) inNumber);
                                }

                                // Assign & Increment operators
                                //   Appends to the end of a String

	LString&		operator += (const LString&	inString)
                                {
                                        return Append(inString);
                                }

	LString&		operator += (ConstStringPtr	inStringPtr)
                                {
                                        return Append(inStringPtr);
                                }

	LString&		operator += (const char*	inCString)
                                {
                                        return Append(inCString, CStringLength(inCString));
                                }

	LString&		operator += (unsigned char	inChar)
                                {
                                        return Append(inChar);
                                }

	LString&		operator += (char			inChar)
                                {
                                        return Append(inChar);
                                }

	LString&		operator += (SInt32 inNumber)
                                {
                                        return Append(inNumber);
                                }

	LString&		operator += (SInt16 inNumber)
                                {
                                        return Append(inNumber);
                                }

	LString&		Append(	const LString&	inString)
                                {
                                        return Append(&inString[1], inString[0]);
                                }

	LString&		Append(	ConstStringPtr	inStringPtr)
                                {
                                        return Append(inStringPtr + 1, inStringPtr[0]);
                                }

	LString&		Append( const char* inCString)
                                {
                                    return Append(inCString, CStringLength(inCString));
                                }

	LString&		Append(	unsigned char	inChar);

	LString&		Append(	char inChar)
                                {
                                    return Append((unsigned char) inChar);
                                }

	LString&		Append(	SInt32 inNumber);

	LString&		Append( SInt16 inNumber)
                                {
                                    return Append((SInt32) inNumber);
                                }

	LString&		Append(	const void* inPtr, UInt8 inLength);

                                // Comparison, Object to Input
                                //		Object == Input, returns 0
                                //		Object >  Input, returns 1
                                //		Object <  Input, returns -1
	SInt16			CompareTo( const LString& inString) const
                                {
                                    return (*mCompareFunc)(mStringPtr + 1, &inString[1],  mStringPtr[0], inString[0]);
                                }

	SInt16			CompareTo( ConstStringPtr inStringPtr) const
                                {
                                        return (*mCompareFunc)(mStringPtr + 1, inStringPtr + 1, mStringPtr[0], inStringPtr[0]);
                                }

	SInt16			CompareTo( unsigned char inChar) const
                                {
                                        return (*mCompareFunc)(mStringPtr + 1, &inChar, mStringPtr[0], 1);
                                }

	SInt16			CompareTo(
							const void*		inPtr,
							UInt8			inLength) const
						{
							return (*mCompareFunc)(mStringPtr + 1, inPtr,
												   mStringPtr[0], inLength);
						}

									// Find Substring
									//   Returns index of start of substring,
									//	 searching forward from inStartPos
	UInt8			Find(	const LString&	inString,
							UInt8			inStartPos = 1) const
						{
							return Find(&inString[1], inString[0], inStartPos);
						}

	UInt8			Find(	ConstStringPtr	inStringPtr,
							UInt8			inStartPos = 1) const
						{
							return Find(inStringPtr + 1, inStringPtr[0], inStartPos);
						}

	UInt8			Find(	unsigned char	inChar,
							UInt8			inStartPos = 1) const
						{
							return Find(&inChar, 1, inStartPos);
						}

	UInt8			Find(	const void*		inPtr,
							UInt8			inLength,
							UInt8			inStartPos = 1) const;

									// Reverse Find Substring
									//   Returns index of start of substring,
									//   searching backward from inStartPos
	UInt8			ReverseFind(
							const LString&	inString,
							UInt8			inStartPos = 255) const
						{
							return ReverseFind(&inString[1], inString[0], inStartPos);
						}

	UInt8			ReverseFind(
							ConstStringPtr	inStringPtr,
							UInt8			inStartPos = 255) const
						{
							return ReverseFind(inStringPtr + 1, inStringPtr[0], inStartPos);
						}

	UInt8			ReverseFind(
							unsigned char	inChar,
							UInt8			inStartPos = 255) const
						{
							return ReverseFind(&inChar, 1, inStartPos);
						}

	UInt8			ReverseFind(
							const void*		inPtr,
							UInt8			inLength,
							UInt8			inStartPos = 255) const;

									// Check if a string begins with a substring
	Boolean			BeginsWith(
							const LString&	inString) const
						{
							return BeginsWith(&inString[1], inString[0]);
						}

	Boolean			BeginsWith(
							ConstStringPtr	inStringPtr) const
						{
							return BeginsWith(inStringPtr + 1, inStringPtr[0]);
						}

	Boolean			BeginsWith(
							unsigned char	inChar) const
						{
							return BeginsWith(&inChar, 1);
						}

	Boolean			BeginsWith(
							const void*		inPtr,
							UInt8			inLength) const;

									// Check if a string ends with a substring
	Boolean			EndsWith(
							const LString&	inString) const
						{
							return EndsWith(&inString[1], inString[0]);
						}

	Boolean			EndsWith(
							ConstStringPtr	inStringPtr) const
						{
							return EndsWith(inStringPtr + 1, inStringPtr[0]);
						}

	Boolean			EndsWith(
							unsigned char	inChar) const
						{
							return EndsWith(&inChar, 1);
						}

	Boolean			EndsWith(
							const void*		inPtr,
							UInt8			inLength) const;

					// Find String within another String
					// Returns index of start of String
	UInt8			FindWithin(
							const LString&	inString,
							UInt8			inStartPos = 1) const
						{
							return (UInt8) FindWithin(&inString[1], inString[0], inStartPos);
						}

	UInt8			FindWithin(
							ConstStringPtr	inStringPtr,
							UInt8			inStartPos = 1) const
						{
							return (UInt8) FindWithin(inStringPtr + 1, inStringPtr[0], inStartPos);
						}

	UInt8			FindWithin(
							unsigned char	inChar,
							UInt8			inStartPos = 1) const
						{
							return (UInt8) FindWithin(&inChar, 1, inStartPos);
						}

	UInt32			FindWithin(
							const void*		inPtr,
							UInt32			inLength,
							UInt32			inStartPos = 1) const;

					// Reverse Find String within another String
					//    Returns index of start of String
	UInt8			ReverseFindWithin(
							const LString&	inString,
							UInt8			inStartPos = 255) const
						{
							return (UInt8) ReverseFindWithin(&inString[1], inString[0], inStartPos);
						}

	UInt8			ReverseFindWithin(
							ConstStringPtr	inStringPtr,
							UInt8			inStartPos = 255) const
						{
							return (UInt8) ReverseFindWithin(inStringPtr + 1, inStringPtr[0], inStartPos);
						}

	UInt8			ReverseFindWithin(
							unsigned char	inChar,
							UInt8			inStartPos = 1) const
						{
							return (UInt8) ReverseFindWithin(&inChar, 1, inStartPos);
						}

	UInt32			ReverseFindWithin(
							const void*		inPtr,
							UInt32			inLength,
							UInt32			inStartPos = 0xFFFFFFFF) const;

									// Copy Substring
									//    Returns a LStr255 object for a
									//    substring of the String
	LStr255			operator()(
							UInt8			inStartPos,
							UInt8			inCount) const;

									// Inserting substrings staring at the
									//   specified position
	LString&		Insert(
							const LString&	inString,
							UInt8			inAtIndex)
						{
							return Insert(&inString[1], inString[0], inAtIndex);
						}

	LString&		Insert(
							ConstStringPtr	inStringPtr,
							UInt8			inAtIndex)
						{
							return Insert(inStringPtr + 1, inStringPtr[0], inAtIndex);
						}

	LString&		Insert(
							unsigned char	inChar,
							UInt8			inAtIndex)
						{
							return Insert(&inChar, 1, inAtIndex);
						}

	LString&		Insert(
							const void*		inPtr,
							UInt8			inLength,
							UInt8			inAtIndex);

                                // Removing substring starting at a given
                                //   position with a given length
	LString&		Remove(
                                        UInt8	inStartPos,
                                        UInt8	inCount);

                                // Replacing one substring with another
	LString&		Replace(
                                        UInt8		inStartPos,
                                        UInt8		inCount,
                                        const LString&	inString)
                                {
                                        return Replace(inStartPos, inCount, &inString[1], inString[0]);
                                }

	LString&		Replace(
                                        UInt8		inStartPos,
                                        UInt8		inCount,
                                        ConstStringPtr	inStringPtr)
                                {
                                        return Replace(inStartPos, inCount, inStringPtr + 1, inStringPtr[0]);
                                }

	LString&		Replace(
                                        UInt8		inStartPos,
                                        UInt8		inCount,
                                        unsigned char	inChar)
                                {
                                        return Replace(inStartPos, inCount, &inChar, 1);
                                }

        LString&	Replace(
                                UInt8		inStartPos,
                                UInt8		inCount,
                                const void*	inPtr,
                                UInt8		inLength);

                        // String Comparison Functions

                        // Toolbox CompareText() routine using the
                        //   rules for the current Script
                        // Simple byte-by-byte value comparison
	static SInt16	CompareBytes(
                                        const void*	inLeft,
                                        const void*	inRight,
                                        UInt8		inLeftLength,
                                        UInt8		inRightLength);

			// Case-insensitive comparison
	static SInt16	CompareIgnoringCase(
                                            const void*		inLeft,
                                            const void*		inRight,
                                            UInt8		inLeftLength,
                                            UInt8		inRightLength);


				// Public utility functions

						// C string length limited to 255 characters
	static UInt8		CStringLength(const char* inCString);

						// Copy a Pascal string
	static StringPtr	CopyPStr(	ConstStringPtr	inSourceString,
									StringPtr outDestString,
									SInt16 inDestSize = sizeof(Str255));

						// Convert C to Pascal string in place
	static StringPtr	CToPStr(char* ioStr);

						// Convert Pascal to C string in place
	static char*		PToCStr(StringPtr ioStr);

						// Append one Pascal string to another
	static StringPtr	AppendPStr( StringPtr ioBaseString,
									ConstStringPtr inAppendString,
									SInt16 inDestSize = sizeof(Str255));

protected:
					// Constructor
					// Protected since LString is abstract
					LString( SInt16 inMaxBytes, StringPtr inStringPtr);

                                // Utility functions
	void			LoadFromStringPtr( ConstStringPtr inStringPtr)
                                {
                                    LoadFromPtr(inStringPtr + 1, inStringPtr[0]);
                                }

	void			LoadFromPtr(const void* inPtr, UInt8 inLength);


protected:
	StringPtr		mStringPtr;	// Ptr to storage allocated by a subclass
	CompareFunc		mCompareFunc;	// String comparison function
	UInt16			mMaxBytes;	// Max size, including length byte
};


// ---------------------------------------------------------------------------
//	 Global String Addition Operators
// ---------------------------------------------------------------------------
//	Handles "adding", which means concatenating, String objects with:
//		(1) Another String object		leftString + rightString
//		(2) A pointer to a string		String + StringPtr, StringPtr + String
//		(3) A character					String + char, char + String
//
//	The result of String addition is a LStr255


LStr255	operator + (
				const LString&	inLeftString,
				const LString&	inRightString);
				
LStr255	operator + (
				const LString&	inLeftString,
				ConstStringPtr	inRightStringPtr);
				
LStr255	operator + (
				ConstStringPtr	inLeftStringPtr,
				const LString&	inRightString);
				
LStr255	operator + (
				const LString&	inLeftString,
				StringPtr		inRightStringPtr);
				
LStr255	operator + (
				StringPtr		inLeftStringPtr,
				const LString&	inRightString);
				
LStr255	operator + (
				const LString&	inLeftString,
				unsigned char	inRightChar);
				
LStr255	operator + (
				unsigned char	inLeftChar,
				const LString&	inRightString);



// ---------------------------------------------------------------------------
//	 Global String Comparison Operators
// ---------------------------------------------------------------------------
//	Comparison of String objects with:
//		(1) Another String object		leftString ? rightString
//		(2) A pointer to a string		String ? StringPtr, StringPtr ? String
//		(3) A character					String ? char, char ? String

// ------------------------------------	Equals
inline
Boolean operator == (
				const LString&	inLhs,
				const LString&	inRhs)
			{
				return inLhs.CompareTo(inRhs) == 0;
			}

inline
Boolean operator == (
				const LString&	inLhs,
				ConstStringPtr	inRhs)
			{
				return inLhs.CompareTo(inRhs) == 0;
			}

inline
Boolean operator == (
				ConstStringPtr	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) == 0;
			}

inline
Boolean operator == (
				const LString&	inLhs,
				unsigned char	inRhs)
			{
				return inLhs.CompareTo(inRhs) == 0;
			}

inline
Boolean operator == (
				unsigned char	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) == 0;
			}

// ------------------------------------	Not Equals
inline
Boolean operator != (
				const LString&	inLhs,
				const LString&	inRhs)
			{
				return inLhs.CompareTo(inRhs) != 0;
			}

inline
Boolean operator != (
				const LString&	inLhs,
				ConstStringPtr	inRhs)
			{
				return inLhs.CompareTo(inRhs) != 0;
			}

inline
Boolean operator != (
				ConstStringPtr	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) != 0;
			}

inline
Boolean operator != (
				const LString&	inLhs,
				unsigned char	inRhs)
			{
				return inLhs.CompareTo(inRhs) != 0;
			}

inline
Boolean operator != (
				unsigned char	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) != 0;
			}

// ------------------------------------	Greater Than
inline
Boolean operator > (
				const LString&	inLhs,
				const LString&	inRhs)
			{
				return inLhs.CompareTo(inRhs) > 0;
			}

inline
Boolean operator > (
				const LString&	inLhs,
				ConstStringPtr	inRhs)
			{
				return inLhs.CompareTo(inRhs) > 0;
			}

inline
Boolean operator > (
				ConstStringPtr	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) < 0;
			}

inline
Boolean operator > (
				const LString&	inLhs,
				unsigned char	inRhs)
			{
				return inLhs.CompareTo(inRhs) > 0;
			}

inline
Boolean operator > (
				unsigned char	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) < 0;
			}

// ------------------------------------	Less Than
inline
Boolean operator < (
				const LString&	inLhs,
				const LString&	inRhs)
			{
				return inLhs.CompareTo(inRhs) < 0;
			}

inline
Boolean operator < (
				const LString&	inLhs,
				ConstStringPtr	inRhs)
			{
				return inLhs.CompareTo(inRhs) < 0;
			}

inline
Boolean operator < (
				ConstStringPtr	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) > 0;
			}

inline
Boolean operator < (
				const LString&	inLhs,
				unsigned char	inRhs)
			{
				return inLhs.CompareTo(inRhs) < 0;
			}

inline
Boolean operator < (
				unsigned char	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) > 0;
			}

// ------------------------------------	Greater Than or Equals
inline
Boolean operator >= (
				const LString&	inLhs,
				const LString&	inRhs)
			{
				return inLhs.CompareTo(inRhs) >= 0;
			}

inline
Boolean operator >= (
				const LString&	inLhs,
				ConstStringPtr	inRhs)
			{
				return inLhs.CompareTo(inRhs) >= 0;
			}

inline
Boolean operator >= (
				ConstStringPtr	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) <= 0;
			}

inline
Boolean operator >= (
				const LString&	inLhs,
				unsigned char	inRhs)
			{
				return inLhs.CompareTo(inRhs) >= 0;
			}

inline
Boolean operator >= (
				unsigned char	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) <= 0;
			}

// ------------------------------------	Less Than or Equals
inline
Boolean operator <= (
				const LString&	inLhs,
				const LString&	inRhs)
			{
				return inLhs.CompareTo(inRhs) <= 0;
			}

inline
Boolean operator <= (
				const LString&	inLhs,
				ConstStringPtr	inRhs)
			{
				return inLhs.CompareTo(inRhs) <= 0;
			}

inline
Boolean operator <= (
				ConstStringPtr	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) >= 0;
			}

inline
Boolean operator <= (
				const LString&	inLhs,
				unsigned char	inRhs)
			{
				return inLhs.CompareTo(inRhs) <= 0;
			}

inline
Boolean operator <= (
				unsigned char	inLhs,
				const LString&	inRhs)
			{
				return inRhs.CompareTo(inLhs) >= 0;
			}

#pragma mark === LStr255 ===

// ---------------------------------------------------------------------------
//	 LStr255
// ---------------------------------------------------------------------------
//	A String with a maximum of 255 characters

class LStr255 : public LString {
public:
									// Constructors
					LStr255();

					LStr255( const LStr255 &	inOriginal);

					LStr255( const LString &	inOriginal);

					LStr255(ConstStringPtr	inStringPtr);

					LStr255(unsigned char	inChar);

					LStr255(char			inChar);

					LStr255(const char*		inCString);

					LStr255(const void*		inPtr,
							UInt8			inLength);

					LStr255(Handle			inHandle);

					LStr255(SInt32			inNumber);

					LStr255(SInt16			inNumber);

									// Assignment operators

	LStr255&		operator = (const LString&	inString)
						{
							LString::operator=(inString);
							return *this;
						}

	LStr255&		operator = (ConstStringPtr	inStringPtr)
						{
							LString::operator=(inStringPtr);
							return *this;
						}

	LStr255& 		operator = (unsigned char	inChar)
						{
							LString::operator=(inChar);
							return *this;
						}

	LStr255& 		operator = (char		inChar)
						{
							LString::operator=(inChar);
							return *this;
						}

	LStr255&		operator = (const char*	inCString)
						{
							LString::operator=(inCString);
							return *this;
						}

	LStr255&		operator = (SInt32		inNumber)
						{
							LString::operator=(inNumber);
							return *this;
						}

	LStr255&		operator = (SInt16		inNumber)
						{
							LString::operator=(inNumber);
							return *this;
						}

private:
	Str255			mString;
};


#pragma mark === LStringRef ===

// ---------------------------------------------------------------------------
//	 LStringRef
// ---------------------------------------------------------------------------
//	String class which uses external storage
//
//	Client supplies a pointer to storage and a maximum length

class	LStringRef : public LString {
public:
						LStringRef(
								SInt16		inMaxBytes,
								StringPtr	inStringPtr);

						LStringRef(
								const LStringRef&	inStringRef);

	LStringRef&			operator = (
								const LStringRef&	inStringRef);

	void				SetStringRef(
								UInt16		inMaxBytes,
								StringPtr	inStringPtr);

									// Assignment operators

	LStringRef&		operator = (const LString&	inString)
						{
							LString::operator=(inString);
							return *this;
						}

	LStringRef&		operator = (ConstStringPtr	inStringPtr)
						{
							LString::operator=(inStringPtr);
							return *this;
						}

	LStringRef& 	operator = (unsigned char	inChar)
						{
							LString::operator=(inChar);
							return *this;
						}

	LStringRef& 	operator = (char			inChar)
						{
							LString::operator=(inChar);
							return *this;
						}

	LStringRef&		operator = (const char*		inCString)
						{
							LString::operator=(inCString);
							return *this;
						}

	LStringRef&		operator = (SInt32			inNumber)
						{
							LString::operator=(inNumber);
							return *this;
						}

	LStringRef&		operator = (SInt16			inNumber)
						{
							LString::operator=(inNumber);
							return *this;
						}

};


#if PP_Uses_Pragma_Import
	#pragma import reset
#endif

#endif
