// Transfrm.h : mathematical routines used for 2-star alignment
// Copyright (c) 2005 Imaginova Canada Ltd.
// Written by David Ek (for ASCOM DSC plugin), modified by Lawrence D. Lopez and Marko Kudjerski for Intelliscope

#include <math.h>
#include "transfrm.h"

static dircos_t fwdMatrix[3]; //use to find scope from sky coords
static dircos_t revMatrix[3]; //use to find sky from scope- coords

int InvertMatrix( dircos_t* regMatrix, dircos_t* invMatrix )
{
   double v[3][6];
   double tempVal;
   int i;
   int j;
   int k;
   int r;                 // pivot row

   for (i=0;i<3;i++) {
      v[0][i] = regMatrix[i].l;
      v[1][i] = regMatrix[i].m;
      v[2][i] = regMatrix[i].n;
   }

   for (i=3;i<6;i++)
      for (j=0;j<3;j++)
         v[j][i] = 0.0;

   v[0][3] = 1.0;
   v[1][4] = 1.0;
   v[2][5] = 1.0;

   for (i=0;i<3;i++) {

      // find pivot row

      r=i;
      for (j=i+1;j<3;j++)
         if ( fabs(v[j][i]) > fabs(v[r][i]) )
            r=j;

      // swap rows, if needed.

      if (r != i)
         for (j=0;j<6;j++) {
            tempVal = v[i][j];
            v[i][j] = v[r][j];
            v[r][j] = tempVal;
         }

      // normalize pivot row:

      tempVal = v[i][i];

      // check to make sure we're not normalizing using a zero value

      if ( fabs(tempVal) < 1.0E-5 ) {
         return 0;
	  }

      for (j=i;j<6;j++)
         v[i][j] = v[i][j]/tempVal;

      // reduce rows:

      for (j=0;j<3;j++)
         if (j != i) {
            tempVal = v[j][i];
            for (k=i;k<6;k++)
               v[j][k] = v[j][k] - v[i][k] * tempVal;
         }
   }

   for (i=0;i<3;i++) {
      invMatrix[i].l = v[0][i+3];
      invMatrix[i].m = v[1][i+3];
      invMatrix[i].n = v[2][i+3];
   }

   return 1;
}

int CoordsToCosines( double az, double alt, dircos_t* cosines )
{
   double cosAlt = cos(alt);
   double sinAlt = sin(alt);
   double cosAz = cos(az);
   double sinAz = sin(az);
   cosines->l = cosAlt*cosAz;
   cosines->m = cosAlt*sinAz;
   cosines->n = sinAlt;
   return 1;
}

int CosinesToCoords( dircos_t* cosines, double* az, double* alt )
{
   int result = Normalize( cosines );

   if (result == 0) {
      return 0;
   }

   *az = atan2(cosines->m, cosines->l);
   while (*az<0.)
         *az+=kTwicePi;

   *alt = asin(cosines->n);
   return 1;
}

int InitMatrix( dircos_t* scopeCosines, dircos_t* skyCosines )
{
   dircos_t invSkyCosines[3];
   int result;
   
   result = InvertMatrix( skyCosines, invSkyCosines );
   
   if (result == 0) {
	  
      return 0;
   }

   result = MatrixMult3x3( scopeCosines, invSkyCosines, fwdMatrix );

   if (result == 0) {
      return 0;
   }

   result = InvertMatrix( fwdMatrix, revMatrix );

   if (result == 0) {
      return 0;
   } else
      return 1;

}

int ScopeToSky( dircos_t* scopeCosines, dircos_t* skyCosines )
{
   int result = MatrixMult3x1( revMatrix, scopeCosines, skyCosines );
   return result;
}

int SkyToScope( dircos_t* skyCosines, dircos_t* scopeCosines )
{
   int result = MatrixMult3x1( fwdMatrix, skyCosines, scopeCosines );
   return result;
}

double DegToRad( double deg )
{
   return deg*kDegreesToRadians;
}

double RadToDeg( double rad )
{
   return rad/kDegreesToRadians;
}

double HrsToRad( double hrs )
{
   return hrs*kRadiansPerHour;
}

double RadToHrs( double rad )
{
   return rad/kRadiansPerHour;
}

int MatrixMult3x3( dircos_t* matrix1, dircos_t* matrix2, dircos_t* result )
{
   int i;

   for (i=0;i<3;i++) {
      result[i].l = matrix1[0].l * matrix2[i].l +
                    matrix1[1].l * matrix2[i].m +
                    matrix1[2].l * matrix2[i].n;

      result[i].m = matrix1[0].m * matrix2[i].l +
                    matrix1[1].m * matrix2[i].m +
                    matrix1[2].m * matrix2[i].n;

      result[i].n = matrix1[0].n * matrix2[i].l +
                    matrix1[1].n * matrix2[i].m +
                    matrix1[2].n * matrix2[i].n;
   }

   return 1;
}

int MatrixMult3x1( dircos_t* matrix1, dircos_t* matrix2, dircos_t* result )
{
   result->l = matrix1[0].l * matrix2->l +
               matrix1[1].l * matrix2->m +
               matrix1[2].l * matrix2->n;

   result->m = matrix1[0].m * matrix2->l +
               matrix1[1].m * matrix2->m +
               matrix1[2].m * matrix2->n;

   result->n = matrix1[0].n * matrix2->l +
               matrix1[1].n * matrix2->m +
               matrix1[2].n * matrix2->n;

   return 1;
}

int CrossProduct( dircos_t vector1, dircos_t vector2, dircos_t* crossProduct )
{
   crossProduct->l = vector1.m * vector2.n - vector1.n * vector2.m;
   crossProduct->m = vector1.n * vector2.l - vector1.l * vector2.n;
   crossProduct->n = vector1.l * vector2.m - vector1.m * vector2.l;

   return 1;
}

int Normalize( dircos_t* vector )
{
   double mag = sqrt(vector->l*vector->l + vector->m*vector->m + vector->n*vector->n);

   if (mag <= 0.0)
      return 0;

   vector->l=vector->l/mag;
   vector->m=vector->m/mag;
   vector->n=vector->n/mag;

   return 1;
}


void cal_jd (int month, float day, int year, double* jd)
{
	int m, y;
	long b, d, c;

	m = month;
	y = year;

	if (m < 3) {
	    m += 12;
	    y -= 1;
	}

	//If the date is in Julian Calendar then b=0, otherwise some
	// calculations must be done
	if (year < 1582 || year == 1582 && (month < 10 || month == 10 && day < 15)) 
	    b = 0;
	else {
	    int a;
	    a = y/100;
	    b = 2 - a + (int)(a/4);
	}

	c = (long)(365.25*(y+4716));

	d = (long)(30.61*(m+1));

	*jd = c + d + day + b - 1524.5;
}