// Transfrm.h : mathematical routines used for 2-star alignment
// Copyright (c) 2005 Imaginova Canada Ltd.
// Written by David Ek (for ASCOM DSC plugin), modified by Lawrence D. Lopez and Marko Kudjerski for Intelliscope


#ifndef __TRANSFRM_H
#define __TRANSFRM_H

typedef struct {
   double l;
   double m;
   double n;
} dircos_t;

/* function InitMatrix

   Description: computes transformation matrix to convert from scope to
   sky coordinates and vice versa.

   Inputs:

   dircos_t* scopeCosines: pointer to array of three dircos_t structs,
                           each holding the direction cosines for a
                           star in scope coordinates
   dircos_t* skyCosines:   pointer to array of three dircos_t structs,
                           each holding the direction cosines for a
                           star in sky coordinates (same stars as in
                           scopeCosines)

   Outputs:

   none (though stores the resulting transformation matrix internally
   for use by ScopeToSky and SkyToScope)

   Returns:

   1 if successful
   0 if not successful
*/

int InitMatrix( dircos_t* scopeCosines, dircos_t* skyCosines );


/* function ScopeToSky

   Description: computes direction cosines in sky coordinates from
   direction cosines in scope coordinates.

   Inputs:

   dircos_t* scopeCosines: pointer to dircos_t struct
                           holding the direction cosines for a
                           star in scope coordinates

   Outputs:

   dircos_t* skyCosines:   a filled dircos_t struct
                           holding the direction cosines for the
                           star in sky coordinates
   Returns:

   1 if successful
   0 if not successful
*/

int ScopeToSky( dircos_t* scopeCosines, dircos_t* skyCosines );

/* function SkyToScope

   Description: computes direction cosines in scope coordinates from
   direction cosines in sky coordinates.

   Inputs:

   dircos_t* skyCosines:   pointer to dircos_t struct
                           holding the direction cosines for a
                           star in sky coordinates

   Outputs:

   dircos_t* scopeCosines: a filled dircos_t struct
                           holding the direction cosines for the
                           star in scope coordinates
   Returns:

   1 if successful
   0 if not successful
*/

int SkyToScope( dircos_t* skyCosines, dircos_t* scopeCosines );

/* function CoordsToCosines

   Description: computes direction cosines given an azimuth (or RA)
   and altitude (or Dec).

   Inputs:

   double az:   azimuth, or RA, in radians
   double alt:  altitude, or declination, in radians

   Outputs:

   dircos_t* cosines:  a filled dircos_t struct holding the direction
                       cosines for the given az and alt

   Returns:

   1 if successful
   0 if not successful
*/

int CoordsToCosines( double az, double alt, dircos_t* cosines );

/* function CosinesToCoords

   Description: computes azimuth (or RA) and altitude (or Dec) given
   a set of direction cosines.

   Inputs:

   dircos_t* cosines:  pointer to a dircos_t struct holding the direction
                       cosines for the given az and alt

   Outputs:

   double* az:   azimuth, or RA, in radians
   double* alt:  altitude, or declination, in radians

   Returns:

   1 if successful
   0 if not successful
*/

int CosinesToCoords( dircos_t* cosines, double* az, double* alt );

/* function DegToRad

   Description: computes an angle in radians given an angle in degrees.

   Inputs:

   double deg: the angle to be converted, in decimal degrees.

   Returns:

   the corresponding angle in radians.
*/

double DegToRad( double deg );

/* function RadToDeg

   Description: computes an angle in degrees given an angle in radians.

   Inputs:

   double rad: the angle to be converted, in radians.

   Returns:

   the corresponding angle in degrees.
*/

double RadToDeg( double rad );

/* function HrsToRad

   Description: computes an angle in radians given an angle in hours.

   Inputs:

   double hrs: the angle to be converted, in decimal hours.

   Returns:

   the corresponding angle in radians.
*/

double HrsToRad( double hrs );

/* function RadToHrs

   Description: computes an angle in hours given an angle in radians.

   Inputs:

   double rad: the angle to be converted, in radians.

   Returns:

   the corresponding angle in hours.
*/

double RadToHrs( double rad );

/* function MatrixMult3x3

   Description: multiplies two three-by-three matrices.

   Inputs:

   dircos_t* matrix1: array of three dircos_t structs, each element
                      defining a column of the matrix.
   dircos_t* matrix2: array of three dircos_t structs, each element
                      defining a column of the matrix.

   Outputs:

   dircos_t* result: the result of the multiplication of matrix1 x matrix2.

   Returns:

   1 if successful
   0 if not successful
*/

int MatrixMult3x3( dircos_t* matrix1, dircos_t* matrix2, dircos_t* result );

/* function MatrixMult3x1

   Description: multiplies a 3x3 matrix times a 1-column 3-row matrix

   Inputs:

   dircos_t* matrix1: array of three dircos_t structs, each element
                      defining a column of the matrix.
   dircos_t* matrix2: pointer to a dircos_t struct defining the single
                      column of the matrix.

   Outputs:

   dircos_t* result: a dircos_t struct defining the single column of the
                     matrix holding the result of the multiplication of
                     matrix1 x matrix2.

   Returns:

   1 if successful
   0 if not successful
*/

int MatrixMult3x1( dircos_t* matrix1, dircos_t* matrix2, dircos_t* result );

/* function InvertMatrix

   Description: inverts a 3x3 matrix.

   Inputs:

   dircos_t* regMatrix: an array of three dircos_t structs, one for each
                        column of the matrix to be inverted.

   Outputs:

   dircos_t* invMatrix: an array of three dircos_t structs filled with
                        the columns of the inverted matrix.

   Returns:

   1 if successful,
   0 if not successful
*/

int InvertMatrix( dircos_t* regMatrix, dircos_t* invMatrix );

/* function CrossProduct

   Description: computes the cross product vector1 x vector2

   Inputs:

   dircos_t vector1, vector2: the vectors to multiply

   Outputs:

   dircos_t* crossProduct: the resulting cross product.

   Returns:

   1 if successful,
   0 if not successful
*/

int CrossProduct( dircos_t vector1, dircos_t vector2, dircos_t* crossProduct );

/* function Normalize

   Description: normalizes a vector into a unit vector

   Inputs:

   dircos_t* vector: the vector to normalize

   Outputs:

   dircos_t* vector: the normalized vector

   Returns:

   1 if successful
   0 if not successful
*/

int Normalize( dircos_t* vector );


/* function cal_jd

	Description: calculated Julian date given

	Inputs: 

	int month: month to use in calculations

	double day:   day to use in calculations

	int year:  year to use in calculations

	Outputs:

	double* jd: Julian Date for the given information

	Returns: no return value
*/

void cal_jd (int month, float day, int year, double* jd);


#endif